/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author:  
 *   $Date: 
 ***************************************************************/
 
/** \file
 * \brief planar biconnected augmentation approximation algorithm
 * 
 * \autho
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_PLANAR_AUGMENTATION_H
#define OGDF_PLANAR_AUGMENTATION_H

#include <ogdf/module/AugmentationModule.h>
#include <ogdf/basic/List.h>
#include <ogdf/decomposition/DynamicBCTree.h>

namespace ogdf {


enum paStopCause {paPlanarity, paCDegree, paBDegree, paRoot};


class PlanarAugmentation;
class labelStruct;


typedef labelStruct* label;

/**
 * \brief auxiliary class for the planar augmentation algorithm
 * 
 *   A label contains several pendants, a parent- and a head- node.
 *   The head node is a cutvertex in the correspondign BC-Tree.
 *   The pendants can be connected by edges so planarity is maintained.
 */
class labelStruct{

friend class PlanarAugmentation;

private:

	/**
	 * \brief the "parent" of the pendants in the BC-Tree, m_parent is a b-vertex or a c-vertex
	 * if it is a b-vertex m_parent != 0
	 * otherwise m_parent == 0 and the parent is the head node
	 * m_head is always != 0
	 */
	node m_parent;
	/**
	 * \brief the cutvertex and perhaps (see above) the parent node
	 */	
	node m_head;
	/**
	 * \brief list with all pendants of the label
	 */		
	List<node> m_pendants;
	/**
	 * \brief the stop cause that occurs when traversing from the pendants to the bc-tree-root
	 * 		  computed in labelMatcher::followPath()
	 */	
	paStopCause m_stopCause;

public:
	labelStruct(node parent, node cutvertex, paStopCause sc = paBDegree);

	bool isBLabel();
	bool isCLabel();

	//! return pendant with number nr, starts counting at 0
	node getPendant(int nr);	

	node getFirstPendant();
	node getLastPendant();
	
    //! return number of pendants
	int size();
	
	void removePendant(node pendant);
	void removePendant(ListIterator<node> it);
	void removeFirstPendant();
	
	void addPendant(node pendant);
	void deleteAllPendants();
	
	//! return the parent node. If the label is a c-label it returns m_head
	node parent();
	//! returns the head node
	node head();
	
	void setParent(node newParent){
		m_parent = newParent;
	}
	
	void setHead(node newHead){
		m_head = newHead;
	}
	
	paStopCause stopCause(){
		return m_stopCause;	
	}
	
	void stopCause(paStopCause sc){
		m_stopCause = sc;	
	}	

}; // class labelStruct



/**
 * \brief Implementation of the algorithm for biconnectivity augmentation
 * 		  based on the paper "A new approximation algorithm for the Planar
 *		  Augmentation Problem" by P. Mutzel and S. Fjalko (1998).
 * 
 * The class \a PlanarAugmentation implements an augmentation algorithm
 * that augments a graph to a biconnected graph. In addition, if the graph was
 * planar before augmentation, the resulting graph will be biconnected and
 * planar.
 * The algorithm uses (dynamic) BC-trees and achieves biconnectivity by
 * inserting edges between nodes of pendants (that are leaves in the bc-tree).
 * The guaranteed approximation-quality is 5/3.
 */
class PlanarAugmentation : public AugmentationModule {

public:
	PlanarAugmentation() { }
	~PlanarAugmentation() { }

protected:
	/**
	 * \brief the calling method for the algorithm
	 * 
	 * \param G is the working graph
	 * \param L is the list of all new edges
	 */
	void doCall(Graph& G, List<edge>& L);

	
private:
	/**
	 * \brief counts the number of planarity tests
	 */
	int m_nPlanarityTests;
	
	/**
	 * \brief the working graph
	 */
	Graph* m_pGraph;
	/**
	 * \brief the corresponding BC-Tree
	 */
	DynamicBCTree* m_pBCTree;
	
	/**
	 * \brief the inserted edges by the algorithm
	 */
	List<edge>* m_pResult;
	
	/**
	 * \brief list of all labels, sorted by size (decreasing)
	 */
	List<label> m_labels;
	/**
	 * \brief list of all pendants (leaves in the BC-Tree)
	 */
	List<node> m_pendants;

	/**
	 * \brief list of pendants that has to be deleted after each reduceChain
	 */
	List<node> m_pendantsToDel;
	
	/**
	 * \brief the label a BC-Node belongs to
	 */
	NodeArray<label> m_belongsTo;
	/**
	 * \brief the list iterator in m_labels if the node in the BC-Tree is a label 
	 */
	NodeArray< ListIterator<label> > m_isLabel;

	/**
	 * \brief stores for each node of the bc-tree the children that have an adjacent bc-node 
	 * 				 that doesn't belong to the same parent-node
	 * 
	 * this is necessary because the bc-tree uses an union-find-data-structure to store 
	 * dependancies beteween bc-nodes. The adjacencies in the bc-tree won't be updated.
	 */
	NodeArray< SList<adjEntry> > m_adjNonChildren; 	


private:

	/**
	 * \brief main function for planar augmentation
	 */
	void augment();
	
	/**
	 * \brief makes the graph connected by new edges between pendants of
	 *  	  the connected components
	 */
	void makeConnectedByPendants();

	/**
	 * \brief Is called for every pendant-node. It traverses to the 
	 * 		  root and creates a label or updates one.
	 * 
	 * \param p is a pendant in the BC-Tree
	 * \param labelOld is the old label of p
	 */
	void reduceChain(node p, label labelOld = 0);

	/**
	 * \brief Is called in reduceChain. It traverses to the root and checks 
	 * 		  several stop conditions.
	 * 
	 * \param v is a node of the BC-Tree
	 * \param last is the last found C-vertex in the BC-Tree, is modified by
	 * 		  the method
	 * \return the stop-cause
	 */	
	paStopCause followPath(node v, node& last);

	/**
	 * \brief checks planarity for a new edge (v1,v2) in the original graph.
	 * 
	 * \param v1, v2 are nodes of the original graph
	 * \return true iff the graph (including the new edge) is planar 
	 */	
	bool planarityCheck(node v1, node v2);
	
	/**
	 * \brief returns a node that belongs to bc-node v and is adjacent to the cutvertex
	 * 
	 * \param v is a node in the BC-Tree
	 * \param cutvertex is the last cutvertex found
	 * \return a node of the original graph
	 */	
	node adjToCutvertex(node v, node cutvertex = 0);
	
	/**
	 * \brief traverses from pendant to ancestor and returns the
	 *  	  last node before ancestor on the path
	 */
	node findLastBefore(node pendant, node ancestor);

	/**
	 * \brief deletes the pendant p, removes it from the corresponding label
	 * 		  and updates the label-order
	 */	
	void deletePendant(node p, bool removeFromLabel = true);
	/**
	 * \brief adds a pendant p to the label l and updates the label-order
	 */	
	void addPendant(node p, label& l);
	
	/**
	 * \brief connects two pendants
	 * 
	 * \return the new edge in the original graph
	 */	 
	edge connectPendants(node pendant1, node pendant2);
	/**
	 * \brief removes all pendants of a label
	 */	
	void removeAllPendants(label& l);

	/**
	 * \brief connects all pendants of label l with new edges 
	 */
	void joinPendants(label& l);

	/**
	 * \brief connects the only pendant of l with a computed ancestor
	 */	
	void connectInsideLabel(label& l);

	/**
	 * \brief inserts label l into m_labels by decreasing order 
	 * 
	 * \return the corresponding list iterator
	 */
	ListIterator<label> insertLabel(label l);

	/**
	 * \brief deletes label l
	 */
	void deleteLabel(label& l, bool removePendants = true);

	/**
	 * \brief inserts edges between pendants of label first and second.
	 * 		  first.size() is grater or than second.size() or equal
	 */
	void connectLabels(label first, label second);

	/**
	 * \brief creates a new label and inserts it into m_labels
	 */
	label newLabel(node cutvertex, node p, paStopCause whyStop);

	/**
	 * \brief finds two matching labels, so all pendants can be connected 
	 * 		  without losing planarity
	 *
	 * \param first is the label with maximum size, modified by the method
	 * \param second is the matching label, modified by the method:
	 * 		  0 if no matching is found
	 * \return true iff a matching label is found
	 */
	bool findMatching(label& first, label& second);
	
	/**
	 * \brief checks if the pendants of label a and label b can be connected
	 * 		  without creating a new pendant
	 */	
	bool connectCondition(label a, label b);

	/**
	 * \brief updates the adjNonChildren-data
	 * 
	 * \param newBlock is a new created block of the BC-Tree
	 * \param path is the path in the BC-Tree between the two connected nodes
	 */
	void updateAdjNonChildren(node newBlock, SList<node>& path);

	/**
	 * \brief modifies the root of the BC-Tree that newRoot replaces oldRoot
	 */
	void modifyBCRoot(node oldRoot, node newRoot);

	/**
	 * \brief major updates caused by the new edges
	 * 
	 * \param newEdges is a list of all new edges
	 */	
	void updateNewEdges(SList<edge>* newEdges);

	/**
	 * \brief cleanup
	 */	
	void terminate();
	
};	// class PlanarAugmentation


} // namespace ogdf

#endif
