/*
 * $Revision: 1.8 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2007/07/24 14:13:44 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration and implementation of ArrayBuffer class.
 * 
 * \author Markus Chimani
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_ARRAY_BUFFER_H
#define OGDF_ARRAY_BUFFER_H

#include <ogdf/basic/Array.h>

namespace ogdf {

//! An array that keeps track of the number of inserted elements; also usable as an efficient stack.
/**
 * This is a growable array (with some initial size \a s) which starts out being empty. Using 
 * stack functions you can put elements into and out of it. The initial array size is automatically 
 * expanded if neccessary, but never automatically shrunken. You may also access the elements it 
 * contains using the []-operator. Tha valid indices are 0..(\a s - 1).
 */
template<class E> 
class ArrayBuffer : private Array<E> {
	int num; //!< The number of elements in te buffer
public:	
	//! Constructs an empty ArrayBuffer, without initial memory allocation.
	ArrayBuffer() : Array<E>(), num(0) {}
	//! Constructs an empty ArrayBuffer, allocating memory for up to \a size elements.
	explicit ArrayBuffer(int size) : Array<E>(size), num(0) {}
	
	//! Reinitializes the array, clearing it, and without initial memory allocation.
	void init() { Array<E>::init(); }
	//! Reinitializes the array, clearing it, and allocating memory for up to \a size elements.
	void init(int size) { Array<E>::init(size); }

	//! Clears the buffer
	void clear() { num =0; }
	
	//! Returns the newest element of the buffer.
	const E &top() const { OGDF_ASSERT(num>0); return Array<E>::operator[](num-1); };
	//! Returns the newest element of the buffer.
	E &top() { OGDF_ASSERT(num>0); return Array<E>::operator[](num-1); };
	
	//! Puts a new element in the buffer.
	void push(E e) {
		if(num == Array<E>::size())
			Array<E>::grow(num); // double the size
		Array<E>::operator[](num++) = e; 
	};

	//! Removes the newest element from the buffer.
	void pop() { OGDF_ASSERT(num>0); --num; };
	//! Removes the newest element from the buffer and returns it.
	E popRet() { OGDF_ASSERT(num>0); return Array<E>::operator[](--num); };
	
	//! Returns true if the buffer is empty, false otherwise.
	bool empty() const { return !num; }
	//! Returns number of elements in the buffer.
	int count() const { return num; }
	
	//! Returns a reference to the element at position \a i.
	const E &operator[](int i) const {
		OGDF_ASSERT(0 <= i && i < num)
		return Array<E>::operator[](i);
	}
	//! Returns a reference to the element at position \a i.
	E &operator[](int i) {
		OGDF_ASSERT(0 <= i && i < num)
		return Array<E>::operator[](i);
	}	

	//! Generates a compact copy holding the current elements.
	/**
	 * Creates a copy of the ArrayBuffer and stores it into 
	 * the given Array \a A. 
	 * \it A has exactly the neccessary size to hold all 
	 * elements in the buffer.
	 * 
	 * This method uses an elementwise operator=.
	 * If you need a bitcopy of the buffer, use \it compactMemcpy 
	 * instead; if you need a traditional array copy (using the Array's 
	 * copy-constructor) use \it compactCpyCon instead. 
	 */
	void compactCopy(Array<E>& A2) const {
		OGDF_ASSERT(this != &A2);
		if(num) {
			A2.init(num);
			for(int i = num; i-->0;)
				A2[i] = (*this)[i];
		} else
			A2.init(0);
	}

	//! Generates a compact copy holding the current elements.
	/**
	 * Creates a copy of the ArrayBuffer and stores it into 
	 * the given Array \a A. 
	 * \it A has exactly the neccessary size to hold all 
	 * elements in the buffer.
	 * 
	 * This method uses the Array's copy constructur. If you 
	 * need a bitcopy of the buffer, use \it compactMemcpy 
	 * instead; if you neeed a elementwise operator=-copy, use
	 * \it compactCopy instead.
	 */
	void compactCpycon(Array<E>& A2) const {
		OGDF_ASSERT(this != &A2);
		if(num) {
			int tmp = Array<E>::m_high; // thank god i'm a friend of Array
			Array<E>::m_high = num-1; // fake smaller size
			A2.copy(*this); // copy
			Array<E>::m_high = tmp;
		} else
			A2.init(0);
	}

	//! Generates a compact copy holding the current elements.
	/**
	 * Creates a copy of the ArrayBuffer and stores it into 
	 * the given Array \a A. 
	 * \it A has exactly the neccessary size to hold all 
	 * elements in the buffer.
	 * 
	 * This method uses memcpy. If you need a traditional 
	 * arraycopy using a copy constructur, use \a compactCopy 
	 * instead; if you neeed a elementwise operator=-copy, use
	 * \it compactCopy instead.
	 */
	void compactMemcpy(Array<E>& A2) const {
		OGDF_ASSERT(this != &A2);
		if(num) {
			A2.init(num);
			memcpy(A2.m_pStart,this->m_pStart,sizeof(E)*num);
		} else
			A2.init(0);
	}

	//! Performs a linear search for element \a x.
	/**
	 * Warning: linear running time!
	 * Note that the linear search runs from back to front.
	 * \return the index of the found element, and low()-1 if not found.
	 */
	int linearSearch (const E& x) const {
		int i;
		for(i = num; i-->0;)
			if(x == Array<E>::m_vpStart[i]) break;
		return i;

	}

	//! Performs a linear search for element \a x with comparer \a comp.
	/**
	 * Warning: linear running time!
	 * Note that the linear search runs from back to front.
	 * \return the index of the found element, and low()-1 if not found.
	 */
	int linearSearch (const E& x, Comparer<E> &comp) const {
		int i;
		for(i = num; i-->0;)
			if(comp.equal(x, Array<E>::m_vpStart[i])) break;
		return i;
	}
};

} //namespace

#endif // OGDF_ARRAY_BUFFER_H
