/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2005/11/18 15:25:27 $ 
 ***************************************************************/
 
/** \file
 * \brief Abstract base class for structures on graphs, that need
 *        to be informed about graph changes (e.g. cluster graphs).
 *
 * Follows the observer pattern: graphs are observable 
 * objects that can inform observers on changes made to their
 * structure.
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_GRAPH_STRUCTURE_H
#define OGDF_GRAPH_STRUCTURE_H


#include <ogdf/basic/List.h>
#include <ogdf/basic/Graph.h>

namespace ogdf {

//
// in embedded graphs, adjacency lists are given in clockwise order.
//


//class Graph;
//class node;
//class edge;


//----------------------------------------------------------
// GraphStructure
// abstract base class
// derived classes have to overload nodeDeleted, nodeAdded
// edgeDeleted, edgeAdded
// these functions should be called by Graph before (delete)
// and after (add) its structure
//----------------------------------------------------------
class GraphStructure {
	friend class Graph;

public:
	GraphStructure() : m_pGraph(0) {} //todo: setgraph
	GraphStructure(const Graph* G) : m_pGraph(G) 
	{
		m_itGList = G->registerStructure(this);
	}//constructor
	virtual ~GraphStructure() 
	{
		if (m_pGraph) m_pGraph->unregisterStructure(m_itGList);
	}//destructor

	// associates structure with different graph
	void reregister(const Graph *pG) {
		//small speedup: check if == m_pGraph
		if (m_pGraph) m_pGraph->unregisterStructure(m_itGList);
		if ((m_pGraph = pG) != 0) m_itGList = pG->registerStructure(this);
	}

	virtual void nodeDeleted(node v) = 0;
	virtual void nodeAdded(node v)   = 0;
	virtual void edgeDeleted(edge e) = 0;
	virtual void edgeAdded(edge e)   = 0;
	virtual void reInit()            = 0; 
	virtual void cleared()           = 0;//Graph cleared

	const Graph*  getGraph() const {return m_pGraph;}

protected:
	const Graph* m_pGraph; //underlying graph
	//List entry in graphs list of all registered graphstructures
	ListIterator<GraphStructure*> m_itGList;


};

} //end namespace ogdf

#endif
