/*
 * $Revision: 1.2 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:00 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration and implementation of HashingArray class.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_HASHING_ARRAY_H
#define OGDF_HASHING_ARRAY_H


#include <ogdf/basic/Hashing.h>


namespace ogdf {


/**
 * \brief Indexed arrays using hashing for element access.
 *
 * A hashing array can be used like a usual array but has a general
 * index type. The only requirement is that a hash() function is defined for the
 * index type. The class uses two template parameters:
 *   - \a I is the index type.
 *   - \a E is the element type.
 *
 * The hashing array is only defined for a subset <I>I<SUB>def</SUB></I> of the
 * index set (set of all elements of the index type). At construction, this set
 * is empty. Whenever an index is assigned an element, this index is added
 * to <I>I<SUB>def</SUB></I>. There are also method for testing if an index
 * is defined (is in <I>I<SUB>def</SUB></I>).
 * 
 * <H3>Example</H3>
 * The following code snippet demonstrates how to use a hashing array. First,
 * the example inserts elements into a hashing array simulating a tiny
 * German&ndash;English dictionary, then it prints some elements via array
 * access, and finally it iterates over all defined indices and prints the
 * dictionary entries. We use a the const reference \a Hc, since we want to
 * avoid that array access for undefined indices creates these elements.
 *
 * \code
 *   HashingArray<String,String> H("[undefined]");
 *   const HashingArray<String,String> &Hc = H;
 *
 *   H["Hund"]  = "dog";
 *   H["Katze"] = "cat";
 *   H["Maus"]  = "mouse";
 *
 *   cout << "Katze:   " << Hc["Katze"]   << endl;
 *   cout << "Hamster: " << Hc["Hamster"] << endl;
 *
 *   cout << "\nAll elements:" << endl;
 *   HashConstIterator<String,String> it;
 *   for(it = Hc.begin(); it.valid(); ++it)
 *     cout << it.key() << " -> " << it.info() << endl;
 * \endcode
 *
 * The produced output is as follows:
 * \code
 * Katze:   cat
 * Hamster: [undefined]
 *
 * Alle Elemente:
 * Hund -> dog
 * Maus -> mouse
 * Katze -> cat
 * \endcode
 */
template<class I, class E>
class HashingArray : private Hashing<I,E> {
	E m_defaultValue; //! The default value for elements.

public:
	//! Creates a hashing array; the default value is the default value of the element type.
	HashingArray() : Hashing<I,E>() { }

	//! Creates a hashing array with default value \a defaultValue.
	HashingArray(const E &defaultValue) : Hashing<I,E>(), m_defaultValue(defaultValue) { }

	//! Copy constructor.
	HashingArray(const HashingArray<I,E> &A) : Hashing<I,E>(A), m_defaultValue(A.m_defaultValue) { }

	//! Returns an iterator to the first element in the list of all elements.
	HashConstIterator<I,E> begin() const { return Hashing<I,E>::begin(); }

	//! Returns the number of defined indices (= number of elements in hash table).
	int size() const { return Hashing<I,E>::size(); }

	//! Returns the element with index \a i.
	const E &operator[](const I &i) const {
		HashElement<I,E> *pElement = Hashing<I,E>::lookup(i);
		if (pElement) return pElement->m_info;
		else return m_defaultValue;
	}

	//! Returns a reference to the element with index \a i.
	E &operator[](const I &i) {
		HashElement<I,E> *pElement = Hashing<I,E>::lookup(i);
		if (!pElement) pElement = Hashing<I,E>::fastInsert(i,m_defaultValue);
		return pElement->m_info;
	}

	//! Returns true iff index \a i is defined.
	bool isDefined(const I &i) const {
		return Hashing<I,E>::member(i);
	}

	//! Undefines index \a i.
	void undefine(const I &i) {
		Hashing<I,E>::del(i);
	}

	//! Assignment operator.
	HashingArray<I,E> &operator=(const HashingArray<I,E> &A) {
		m_defaultValue = A.m_defaultValue;
		Hashing<I,E>::operator =(A);
		return *this;
	}

	//! Undefines all indices.
	void clear() { Hashing<I,E>::clear(); }
};


} // end namespace ogdf

#endif
