/*
 * $Revision: 1.7 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:38:01 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares & Implements Binary Heap, and Top10Heap
 * 
 * \author Markus Chimani
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_MIN_HEAP_H
#define OGDF_MIN_HEAP_H

#include<ogdf/basic/Array.h>

namespace ogdf {

template<class Score, class X> class Valued {
	Score v;
	X x;
public:
	Valued() {};  //be carefull, bro!
	Valued(Score vt, X xt) : v(vt), x(xt) {}
	Valued(const Valued& V) : v(V.v), x(V.x) {}
	Score value() const { return v; }
	X item() const { return x;}
	bool operator<(const Valued<Score,X>& V) const { return v<V.v; }
	bool operator<=(const Valued<Score,X>& V) const { return v<=V.v; }
	bool operator>(const Valued<Score,X>& V) const { return v>V.v; }
	bool operator>=(const Valued<Score,X>& V) const { return v>=V.v; }
	bool operator==(const Valued<Score,X>& V) const { return v==V.v; }
	bool operator!=(const Valued<Score,X>& V) const { return v!=V.v; }
};


template<class X>
class MinHeap {	
private:
	Array<X> data; // array starts at index 1
	int num;
public:
	MinHeap(int size) : data(1, size), num(0) {}

	bool empty() const { return num == 0; }
	int count() const { return num; }
	int size() const { return data.size(); }

	const X& top() const {
		return data[1];
	}
	
	void push(X& x) {
		X y;
		if(num == size())
			data.grow(size(),y); // double the size & init with nulls
		data[++num] = x;
		heapup(num);
	}
	
	X pop() {
		data.swap(1, num--);
		heapdown();
		return data[num+1];
	}
	
	const X& operator[](int idx) const { // ATTN: simulate index starting at 0, to be "traditional" to the outside!!!
		return data[idx+1];
	}
	

protected:
	void heapup(int idx) {
		int papa;
		while( (papa = idx/2) > 0) {
			if( data[papa] > data[idx] ) {
				data.swap(papa, idx);
				idx = papa;
			} else return; //done
		}
	}
	
	void heapdown() {
		int papa = 1;
		int son;
		while(true) {
			if( (son = 2*papa) < num && data[son+1] < data[son] )
				son++;
			if( son <= num && data[son] < data[papa]) {
				data.swap(papa, son);
				papa = son;
			} else return;
		}		
	}	
};

template<class X>
class Top10Heap : protected MinHeap<X> { // favors the 10 highest values...
public:
	enum PushResult { Accepted, Rejected, Swapped };
	
	static bool successful(PushResult r) { return r != Rejected; }
	static bool returnedSomething(PushResult r) { return r != Accepted; }

	Top10Heap() : MinHeap<X>(10) {}
	Top10Heap(int size) : MinHeap<X>(size) {}

	bool empty() const { return MinHeap<X>::empty(); }
	bool full() const { return count() == size(); }
	int count() const { return MinHeap<X>::count(); }
	int size() const { return MinHeap<X>::size(); }

	PushResult push(X& x, X& out) { // returns element that got kicked out - out is uninitialized if heap wasn't full (i.e. PushResult equals Accepted)
		PushResult ret = Accepted;
		if(count() == size()) {
			if(MinHeap<X>::top() >= x) {// reject new item since it's too bad
				out = x;
				return Rejected;
			}
			out = MinHeap<X>::pop(); // remove worst first
			ret = Swapped;
		}
		MinHeap<X>::push(x);
		return ret;
	}
	
	void pushBlind(X& x) {
		if(count() == size()) {
			if(MinHeap<X>::top() >= x) // reject new item since it's too bad
				return;
			MinHeap<X>::pop(); // remove worst first
		}
		MinHeap<X>::push(x);
	}
		
	const X& operator[](int idx) const { // ATTN: simulate index starting at 0, to be "traditional" to the outside!!!
		return MinHeap<X>::operator[](idx);
	}
};

} // end namespace ogdf


#endif
