/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class TwoDHashArray.
 *
 * This is a class implementing a 2-dimensional Hash array.
 * It uses templates for the keys and the data of the objects
 * stored in it.
 * 
 * \author Ren Weiskircher
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#include <ogdf/basic/HashingArray.h>
#include <ogdf/basic/TwoDHashIterator.h>

#ifndef OGDF_TWODHASHARRAY_H
#define OGDF_TWODHASHARRAY_H

namespace ogdf {

	template< class I, class E >
	class TwoDHashArray {
	public:
		TwoDHashArray() : m_size(0) {}
		// Sets the default element
		TwoDHashArray(const E&def) : m_defaultValue(def), m_size(0) {}

		//The copy constructor
		TwoDHashArray(const TwoDHashArray<I,E> &td) :
			m_data(td.m_data),
			//the copy constructor of HashingArray produces deep copies, so
			//we can rely on it to copy the elements in the HashingArrays
			//stored in the HashingArray

			m_defaultValue(td.m_defaultValue),
			m_size(td.m_size){}

		TwoDHashArray &operator=(const TwoDHashArray<I,E> &td) {
			m_data = td.m_data;
			//again, we rely on the HashingArray operator for a deep copy
			m_size = td.m_size;

			m_defaultValue = td.m_defaultValue;

			return *this;

		}

		~TwoDHashArray(){};

		// returns const reference to an entry

		const E &entry(const I&i, const I&j) const{

			if(m_data.isDefined(i))

				if(m_data[i].isDefined(j))

					return((m_data[i])[j]);

			return m_defaultValue;}

		// returns reference to an entry

		E &entry(const I&i, const I&j) {

			if(m_data.isDefined(i))

				if(m_data[i].isDefined(j))

					return((m_data[i])[j]);

			(m_data[i])[j] = m_defaultValue;

			m_size++;

			return ((m_data[i])[j]);

		}

		// returns if an entry for a certain position is defined

		bool isDefined(const I&i, const I&j) const {

			if(m_data.isDefined(i)) return m_data[i].isDefined(j);

			else return false;

		}

		// undefine entry at index i,j

		void undefine(const I&i, const I&j) {

			if(m_data.isDefined(i)) 

				if(m_data[i].isDefined(j)) {

					m_data[i].undefine(j);

					m_size--;

				}

		}

		//initializes a TwoDHashIterator

		TwoDHashIterator<I,E> begin() const {

			TwoDHashIterator<I,E> tdi;

			tdi.m_it1 = m_data.begin();

			if(tdi.m_it1.valid())

				tdi.m_it2 = tdi.m_it1.info().begin();

			return tdi;

		}

		//returns the number of defined elements in the table

		int size() const {

			return m_size;

		}

		//clears all entries in the table

		void clear() {

			m_data.clear();

			//this should call the destructors for all the HashingArrays

			//and they should call the destructors for all elements in

			//the table

			m_size = 0;

		}

	private:

		HashingArray< I,HashingArray< I,E > > m_data;

		//this is where the data goes

		E m_defaultValue;

		int m_size; //number of defined elements in the table



	};

}

#endif
