/*
 * $Revision: 1.12 $
 * 
 * last checkin:
 *   $Author: groneman $ 
 *   $Date: 2007/06/27 14:30:11 $ 
 ***************************************************************/
 
/** \file
 * \brief Derived class of Graph providing additional functionality
 * to handle clustered graphs.
 * 
 * \author Sebastian Leipert, Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_CLUSTER_GRAPH_H
#define OGDF_CLUSTER_GRAPH_H

#include <ogdf/basic/NodeArray.h>
#include <ogdf/basic/Stack.h>
#include <ogdf/basic/GraphStructure.h>


namespace ogdf {

class ClusterGraph;
class ClusterGraphObserver;

//---------------------------------------------------------
// ClusterElement
// Structure for clusters
//---------------------------------------------------------
class ClusterElement : private GraphElement {

	friend class ClusterGraph;
	friend class GraphList<ClusterElement>;

	int						m_id;
	int						m_depth;      // depth in cluster tree
										  // updated by clustergraph
	List<node>				m_entries;    // cluster nodes
	List<ClusterElement*>	m_children;   // cluster children 
	ClusterElement			*m_parent;    // parent
	ClusterElement			*m_pPrev;     // postorder predecessor
	ClusterElement			*m_pNext;     // postorder successor
	ListIterator<ClusterElement*> m_it;   // position of cluster within
										  // m_children of parent.

	List<adjEntry>			m_adjEntries; // adjacency list
										  // Don't use a GraphList ! 
	                                      // This messes with the adjacency 
										  // list of the underlying graph

	#ifdef OGDF_DEBUG
	// we store the graph containing this node for debugging purposes
	const ClusterGraph *m_pClusterGraph;
	#endif

	
	void init(List<node> &nodes) {
		while (!nodes.empty())
			m_entries.pushBack(nodes.popFrontRet());
	}

	List<ClusterElement*> &getChildren(){
		return m_children;
	}

	List<node> &getNodes(){
		return m_entries;
	}
	
	// Function that traverses the inclusion tree and adds nodes to \a clusterNodes.
	// Invoked by public function getClusterNodes(List<node> &clusterNodes)
	void getClusterInducedNodes(List<node> &clusterNodes);


public:


	// construction
	#ifdef OGDF_DEBUG
	ClusterElement(const ClusterGraph *pClusterGraph,int id):
		m_id(id),m_depth(0),m_parent(0),m_pPrev(0),m_pNext(0),m_it(0),		
		m_pClusterGraph(pClusterGraph) {};
	#else
	ClusterElement(int id):
		m_id(id), m_depth(0),m_parent(0),m_pPrev(0),m_pNext(0),m_it(0) {};
	#endif


	#ifdef OGDF_DEBUG
	const ClusterGraph *graphOf() const { return m_pClusterGraph; }
	#endif

	
	// Iteration over ClusterArrays
	// returns (unique) index
	int index() const { return m_id; }
	//returns depth in clustertree
	int depth() const { return m_depth; }
	int& depth()	  { return m_depth; }
	// returns successor in list of all clusters
	ClusterElement* succ() const { return (ClusterElement*)m_next; }
	// returns predecessor in list of all clusters
	ClusterElement* pred() const { return (ClusterElement*)m_prev; }

	// returns postorder successor in list of all clusters
	ClusterElement* pSucc() const { return m_pNext; }
	// returns postorder predecessor in list of all clusters
	ClusterElement* pPred() const { return m_pPrev; }

	// Iteration over tree structures.

	// Children access
	ListConstIterator<ClusterElement*> cBegin() const{ return m_children.begin();}
	ListConstIterator<ClusterElement*> crBegin() const{ return m_children.rbegin();}
	int cCount(){ return m_children.size();}
	// Node access
	ListIterator<node> nBegin(){ return m_entries.begin();}
	ListConstIterator<node> nBegin() const{ return m_entries.begin();}
	int nCount(){ return m_entries.size();}

	ClusterElement* parent(){return m_parent;}


	// returns first adjacency entry
	ListConstIterator<adjEntry> firstAdj() const { return m_adjEntries.begin();  }
	ListIterator<adjEntry> firstAdj() { return m_adjEntries.begin();  }
	// returns last adjacency entry
	ListConstIterator<adjEntry> lastAdj () const { return m_adjEntries.rbegin(); }
	ListIterator<adjEntry> lastAdj () { return m_adjEntries.rbegin(); }

	// Determines recursively all nodes contained in cluster by traversing the inclusion tree
	// starting at current ClusterElement.
	void getClusterNodes(List<node> &clusterNodes);

	OGDF_NEW_DELETE

};// class ClusterElement




typedef ClusterElement *cluster;


#define forall_cluster_adj(adj,c)\
ogdf::ListIterator<adjEntry> OGDF_VAR(__LINE__);\
for(OGDF_VAR(__LINE__)=(c)->firstAdj();\
	ogdf::test_forall_adj_entries_of_cluster(OGDF_VAR(__LINE__),(adj));\
OGDF_VAR(__LINE__)=OGDF_VAR(__LINE__).succ())

#define forall_cluster_rev_adj(adj,c)\
ogdf::ListIterator<adjEntry> OGDF_VAR(__LINE__);\
for(OGDF_VAR(__LINE__)=(c)->lastAdj();\
	ogdf::test_forall_adj_entries_of_cluster(OGDF_VAR(__LINE__),(adj));\
OGDF_VAR(__LINE__)=OGDF_VAR(__LINE__).pred())

#define forall_cluster_adj_edges(e,c)\
ogdf::ListIterator<adjEntry> OGDF_VAR(__LINE__);\
for(OGDF_VAR(__LINE__)=(c)->firstAdj();\
	ogdf::test_forall_adj_edges_of_cluster(OGDF_VAR(__LINE__),(e));\
OGDF_VAR(__LINE__)=OGDF_VAR(__LINE__).succ())



inline bool test_forall_adj_entries_of_cluster(ListIterator<adjEntry> &it, adjEntry &adj)
{
	if (it.valid()) { adj = (*it);return true; }
	else return false;
}

inline bool test_forall_adj_edges_of_cluster(ListIterator<adjEntry> &it, edge &e)
{
	adjEntry adj = (*it);
	if (adj) { e = adj->theEdge(); return true; }
	else return false;
}

inline bool test_forall_adj_edges_of_cluster(adjEntry &adj, edge &e)
{
	if (adj) { e = adj->theEdge(); return true; }
	else return false;
}


class ClusterArrayBase;
template<class T>class ClusterArray;

//---------------------------------------------------------
// iteration macros
//---------------------------------------------------------

// iteration over all nodes clusters of clusterGraph C
#define forall_clusters(c,C) for((c)=(C).firstCluster(); (c); (c)=(c)->succ())
#define forall_postOrderClusters(c,C)\
for((c)=(C).firstPostOrderCluster(); (c); (c)=(c)->pSucc())




//---------------------------------------------------------
// ClusterGraph
// clustered graphs
//---------------------------------------------------------
class ClusterGraph : public GraphStructure {

	GraphList<ClusterElement> m_clusters; // list of all cluster

	const Graph *m_pGraph;			// associated graph

	int		m_nClusters;			// number of clusters
	int		m_clusterIdCount;		// id assigned to next created cluster
	int		m_clusterArrayTableSize;// current table size of cluster arrays
	
	mutable cluster m_postOrderStart;       // first cluster in postorder;
	cluster	m_rootCluster;			// root cluster

	bool    m_adjAvailable;			// true if the adjacency list for each
									// cluster is available.
	bool    m_allowEmptyClusters;   // defines if empty clusters are deleted 
	                                // immediately if generated by operations

	// Storing for every node its cluster.
	NodeArray<cluster>				m_nodeMap;
	// Storing for every node its position within the childrenlist of its cluster
	NodeArray<ListIterator<node> >  m_itMap;   
	// registered cluster arrays
	mutable ListPure<ClusterArrayBase*> m_regClusterArrays; 
	mutable ListPure<ClusterGraphObserver*> m_regObservers; //!< The registered graph structures.

public:

	// construction
	ClusterGraph();

	// Construction of a new cluster graph. All nodes
	// are children of the root cluster
	ClusterGraph(const Graph &G);

	ClusterGraph(const ClusterGraph &C);

	//Constructs a ClusterGraph copy of the ClusterGraph C
	//Underlying graph G is made a copy of C.getGraph()
	ClusterGraph(const ClusterGraph &C,Graph &G);
    //As above, additionally store the new copies of
	//the original nodes, edges, and clusters in
	//arrays originalNodeTable, edgeCopy, originalClusterTable
	ClusterGraph(const ClusterGraph &C,Graph &G,
			     ClusterArray<cluster> &originalClusterTable,
				 NodeArray<node> &originalNodeTable);
	ClusterGraph(const ClusterGraph &C,Graph &G,
			     ClusterArray<cluster> &originalClusterTable,
				 NodeArray<node> &originalNodeTable,
				 EdgeArray<edge> &edgeCopy);

	// Destruction
	virtual ~ClusterGraph();

	// returns max. used cluster index
	int maxClusterIndex() const { return m_clusterIdCount-1; }

	// clear
	void clear();
	//don't delete root
	void semiClear();

	void init(const Graph &G);
	
	// conversion to const Graph reference
	operator const Graph &() const { return *m_pGraph; }

	// assignment
	ClusterGraph &operator=(const ClusterGraph &C);

	// Removes all clusters from the cluster subtree rooted at cluster C
	// except for cluster C itself
	void clearClusterTree(cluster C);

	const Graph & getGraph() const {return *m_pGraph;}

	// Inserts a new cluster. Makes it a child of a cluster Parent
	cluster newCluster(cluster Parent, int id = -1);

	//creates empty cluster with id clusterid and parent parent
	cluster createEmptyCluster(const cluster parent = 0, int clusterId = -1);
	//creates new cluster containing nodes in parameter list
	//nodes are reassigned, 
	//returns created cluster
	//( if you turn off m_allowEmptyclusters, an emptied cluster is deleted
	//(except if all nodes are put into the same cluster) )
	cluster createCluster(SList<node>& nodes, const cluster parent = 0);

	// Deletes cluster c
	// All subclusters become children of parent cluster
	// Precondition: c is not the root cluster
	void delCluster(cluster c);

	// Returns the root cluster
	cluster rootCluster() const {return m_rootCluster;};

	// Returns the cluster a node belongs to
	inline cluster clusterOf(node v) const{ 
		OGDF_ASSERT(v->graphOf() == m_pGraph)
		return m_nodeMap[v];
	}

	// returns number of Clusters
	int numberOfClusters() const { return m_nClusters; }
	int clusterIdCount() const { return m_clusterIdCount;} //upper bound for IDs

	// returns table size of cluster arrays associated with this graph
	int clusterArrayTableSize() const { return m_clusterArrayTableSize; }

	//****************
	//cluster move
	void moveCluster(cluster c, cluster newParent);
	//****************
	//Node assignments

	// Assigns a node to a new cluster
	void assignNode(node v, cluster C);

	// Reassigns a node to a new cluster
	void reassignNode(node v, cluster C);

	// Unassigns a node of cluster
	void unassignNode(node v);

	//remove the assignment entries for nodes
	//checks if node is currently not assigned
	inline void removeNodeAssignment(node v)
	{
		if (m_nodeMap[v]) //iff == 0, itmap == 0 !!?
		{
			cluster C2 = m_nodeMap[v];
			C2->m_entries.del(m_itMap[v]);
			m_nodeMap[v] = 0; 
			m_itMap[v] = 0;   
		}
	}

	//user function to clear grap/tree structure, create root,...
	//inserted mainly for use in gmlparser
	//check this again
	void reInit(Graph& G) 
	{
		reinitGraph(G);
	}

	//******************
	//tree queries / depth issues

	//set the depth update behaviour
	void setUpdateDepth(bool b) const 
	{
		m_updateDepth = b;
		//make sure that depth cant be used anymore
		//(even if it may still be valid a little while)
		if (!b) m_depthUpToDate = false;
	}

	//update depth information in subtree after delCluster
	void pullUpSubTree(cluster c);

	//compute depth of cluster tree, running time O(C)
	//maybe later we should provide a permanent depth member update
	int treeDepth() const
	{
		//initialize depth at first call
		if (m_updateDepth && !m_depthUpToDate)
			computeSubTreeDepth(rootCluster());
		if (!m_updateDepth) THROW(AlgorithmFailureException);
		int l_depth = 1;
		cluster c;
		forall_clusters(c, *this)
		{
			if (c->depth() > l_depth) l_depth = c->depth();
		}

		return l_depth;
	}

	void computeSubTreeDepth(cluster c) const;
	//returns depth of cluster c in cluster tree, root 1
	//should be called instead of direct c->depth call to assure
	//valid depth
	int& clusterDepth(cluster c) const 
	{
		//initialize depth at first call
		if (m_updateDepth && !m_depthUpToDate)
			computeSubTreeDepth(rootCluster());
		//if depth is requested but not updated
		if (!m_updateDepth) THROW(AlgorithmFailureException);
		//debug check
#ifdef OGDF_DEBUG
		cluster cp = c->parent();
		int lastDepth = c->depth();
		while (cp != 0)
		{
			bool b = (lastDepth == cp->depth()+1);
			OGDF_ASSERT(b)
			lastDepth = cp->depth();
			cp = cp->parent();
		}
		OGDF_ASSERT(lastDepth == 1);
#endif
		//end
		OGDF_ASSERT(c->depth() != 0)
		return c->depth();
	}

	//in order to allow creation of new clusters of a set
	//of chosen nodes as a child of their lowest common cluster
	cluster commonCluster(SList<node>& nodes);

	//returns the lowest common cluster in cluster
	//tree that v and w belong to
	//precondition: v,w are nodes in the graph
	cluster commonCluster(node v, node w) const;

	//returns the lowest common cluster lca and the highest
	//ancestors on the path to lca
	cluster commonClusterLastAncestors(node v, 
									   node w,
									   cluster& c1,
									   cluster& c2) const;
	cluster commonClusterPath(node v, 
							  node w,
							  List<cluster>& eL) const;
	cluster commonClusterAncestorsPath(node v, 
  									   node w,
									   cluster& c1,
								   	   cluster& c2,
									   List<cluster>& eL) const;

	// registers a cluster array
	ListIterator<ClusterArrayBase*> registerArray(ClusterArrayBase *pClusterArray) const;
	// unregisters a cluster array
	void unregisterArray(ListIterator<ClusterArrayBase*> it) const;

	//! Registers a ClusterGraphObserver.
	ListIterator<ClusterGraphObserver*> registerObserver(ClusterGraphObserver *pObserver) const;
	
	//! Unregisters a ClusterGraphObserver.
	void unregisterObserver(ListIterator<ClusterGraphObserver*> it) const;
	
	//returns list of clusters that are empty or only contain empty clusters
	//the list is constructed in a an order that allows deletion and reinsertion
	//we never set rootcluster to be one of the empty clusters!!
	//if checkClusters is given, only list elements are checked
	//to allow efficient checking in the case
	//that you know which clusters were recently changed (e.g. node reass.)
	void emptyClusters(SList<cluster>& emptyCluster, SList<cluster>* checkCluster = 0);

	//returns true if cluster has only one node and no children
	inline bool emptyOnNodeDelete(cluster c) //virtual?
	{
		//if (!c) return false; //Allows easy use in loops
		return (c->nCount() == 1) && (c->cCount() == 0);
	}
	inline bool emptyOnClusterDelete(cluster c) //virtual?
	{
		//if (!c) return false; //Allows easy use in loops
		return (c->nCount() == 0) && (c->cCount() == 1);
	}
	// returns first cluster in list of all clusters
	cluster firstCluster() const { return m_clusters.begin (); }
	// returns last cluster in list of all cluster
	cluster lastCluster () const { return m_clusters.rbegin(); }
	// returns first cluster in list of post ordered clusters
	cluster firstPostOrderCluster() const { 
		if (!m_postOrderStart) postOrder();
		return m_postOrderStart; 
	}

	// returns list of all clusters in m_clusters
	template<class CLUSTERLIST>
	void allNodes(CLUSTERLIST &clusters) const {
		clusters.clear();
		for (cluster c = m_clusters.begin(); c; c = c->succ())
			clusters.pushBack(c);
	}

	// Collapses all nodes in the list to the first node. 
	// Multiedges are removed.
	template<class NODELIST>
	void collaps(NODELIST &nodes,Graph &G){
		OGDF_ASSERT(&G == m_pGraph);
		m_adjAvailable = false;

		m_postOrderStart = 0;
		node v = nodes.popFrontRet();
		while (!nodes.empty())
		{
			node w = nodes.popFrontRet();
			adjEntry adj = w->firstAdj();
			while (adj !=0)
			{
				adjEntry succ = adj->succ();
				edge e = adj->theEdge();
				if (e->source() == v || e->target() == v)
					G.delEdge(e);
				else if (e->source() == w)
					G.moveSource(e,v);
				else
					G.moveTarget(e,v);
				adj = succ;
			}
			//because nodes can already be unassigned (they are always
			//unassigned if deleted), we have to check this
			/*
			if (m_nodeMap[w])
			{
				cluster c = m_nodeMap[w];
				c->m_entries.del(m_itMap[w]);
			}
			*/
			//removeNodeAssignment(w);
			G.delNode(w);
		}
	}

	// returns list of all edges adjacent to cluster c in edges
	template<class EDGELIST>
	void adjEdges(cluster c, EDGELIST &edges) const {
		edges.clear();
		edge e;
		if (m_adjAvailable)
		{
			forall_cluster_adj_edges(e,c)
				edges.pushBack(e);
		}
	}

	// returns list of all adjacency entries adjacent to cluster c in entries
	template<class ADJLIST>
	void adjEntries(cluster c, ADJLIST &entries) const {
		entries.clear();
		adjEntry adj;
		if (m_adjAvailable)
		{
			forall_cluster_adj(adj,c)
				entries.pushBack(adj);
		}
	}


	template<class LISTITERATOR>
	void makeAdjEntries(cluster c,LISTITERATOR start){
		adjEntry adj;
		c->m_adjEntries.clear();
		LISTITERATOR its;
		for (its = start; its.valid(); its++)
		{
			adj = (*its);
			c->m_adjEntries.pushBack(adj);
		}
	}

	//**************************
	//file output
	// writes graph in GML format to file fileName
	void writeGML(const char *fileName);

	// writes graph in GML format to output stream os
	void writeGML(ostream &os);

	// writes graph in GraphWin GML format to file fileName
	void writeGraphWinGML(const char *fileName);

	// writes graph in GraphWin GML format to output stream os
	void writeGraphWinGML(ostream &os);

	void writeGraphWinGML(const char *fileName,
						  NodeArray<int> &color);

	void writeGraphWinGML(ostream &os,NodeArray<int> &color);

	//bool readGraphWinGML(const char *fileName,Graph &G);

	//bool readGraphWinGML(istream &is,Graph &G);

	//only read the cluster information for the given graph
	//G is not really a necessary parameter, only checked
	bool readClusterInfoGML(const char* fileName, Graph &G);

	// checks the consistency of the data structure
	// (for debugging only)
	bool consistencyCheck();

	// checks the for combinatorial cluster planar embedding
	// (for debugging only)
	bool representsCombEmbedding();

	void adjAvailable(bool val){ m_adjAvailable = val;};

protected:
	//creates new cluster containing nodes in parameter list
	//cluster has index clusterid if given
	cluster doCreateCluster(SList<node>& nodes, 
		const cluster parent, int clusterId = -1);
	//creates new cluster containing nodes in parameter list
	//fills empty clusters in list, cluster has index clusterid if given
	cluster doCreateCluster(SList<node>& nodes, 
		SList<cluster>& emptyCluster,
		const cluster parent, int clusterId = -1);

	//save last search run number for commoncluster
	mutable ClusterArray<int>* m_lcaSearch;
	mutable int m_lcaNumber;
	mutable ClusterArray<cluster>* m_vAncestor;
	mutable ClusterArray<cluster>* m_wAncestor;

	void copyLCA(const ClusterGraph &C, ClusterArray<cluster>* clusterCopy=0);
	//int m_treeDepth; //should be implemented and updated in operations?

	mutable bool m_updateDepth; //should depth of clusters always be updated after
					    //cluster tree structure change
	mutable bool m_depthUpToDate; //is depth of all clusters currently uptodate

	//adjust the post order structure for moved clusters
	//we assume that c is inserted via pushback in newparent->children
	void updatePostOrder(cluster c, cluster oldParent, cluster newParent);

	//computes new predecessor for SUBTREE at moved cluster c
	//0 if c==root 
	cluster postOrderPredecessor(cluster c) const;
	//leftmost cluster in subtree rooted at c, gets predecessor of subtree
	cluster leftMostCluster(cluster c) const;

	//functions inherited from GraphStructure:
	//define how to cope with graph changes
	virtual void nodeDeleted(node v) 
	{ 
		bool cRemove = false;
		cluster c = clusterOf(v);
		if (!c) return;
		//never allow totally empty cluster
		//if ((emptyOnNodeDelete(c)) &&
		//	(c != rootCluster()) ) cRemove = true;
		unassignNode(v); 
		if (cRemove && !m_allowEmptyClusters) //parent exists
		{
			cluster nonEmpty = c->parent();
			cluster cRun = nonEmpty;
			delCluster(c);
			while ((cRun != rootCluster()) && (cRun->nCount() + cRun->cCount() == 0))
			{
				nonEmpty = cRun->parent();
				delCluster(cRun);
				cRun = nonEmpty;
			}

		}
	}
	virtual void nodeAdded(node v)   
	{
		assignNode(v, rootCluster());
	};
	virtual void edgeDeleted(edge /* e */) {};
	virtual void edgeAdded(edge /* e */)   {};
	virtual void reInit()            {}; 
	virtual void cleared()			 
	{
		//we don't want a complete clear, as the graph still exists
		//and can be updated from input stream
		semiClear();
	};//Graph cleared

private:

	//performs a copy of the cluster structure of C,
	//the underlying graph stays the same
	void shallowCopy(const ClusterGraph &C);

	//perform a deep copy on C, C's underlying
	//graph is copied into G
	void deepCopy(const ClusterGraph &C,Graph &G);

	void deepCopy(const ClusterGraph &C,Graph &G,	
				  ClusterArray<cluster> &originalClusterTable,
				  NodeArray<node> &originalNodeTable);
	void deepCopy(const ClusterGraph &C,Graph &G,	
				  ClusterArray<cluster> &originalClusterTable,
				  NodeArray<node> &originalNodeTable,
				  EdgeArray<edge> &edgeCopy);


	void clearClusterTree(cluster c,List<node> &attached);

	void initGraph(const Graph &G);

	//we already have a graph set, now read a new one
	void reinitGraph(const Graph &G);

	//new cluster with given id, precondition: id not used
	cluster newCluster(int id);

	cluster newCluster();

	void postOrder() const;

	void checkPostOrder() const;

	void postOrder(cluster c,SListPure<cluster> &S) const;

	void reinitArrays();


	// recursively write the cluster structure in GML
	void writeCluster(ostream &os,
					  NodeArray<int> &nId,
					  ClusterArray<int> & cId,
					  int &nextId,
					  cluster c,
					  String ttt);

	// recursively write the cluster structure in GraphWin GML
	void writeGraphWinCluster(ostream &os,
					  NodeArray<int> &nId,
					  NodeArray<String> &nStr,
					  ClusterArray<int> &cId,
					  ClusterArray<String> &cStr,
					  int &nextId,
					  cluster c,
					  String ttt);

};





ostream &operator<<(ostream &os, ogdf::cluster c);


} // end namespace ogdf

#endif
