/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/29 09:10:49 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares ClusterGraphAttributes, an extension of class
 * GraphAttributes,  to store clustergraph layout informations 
 * like cluster cage positions and sizes that can be accessed 
 * over the cluster/cluster ID
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_ATTRIBUTED_CLUSTER_GRAPH_H
#define OGDF_ATTRIBUTED_CLUSTER_GRAPH_H

#include <ogdf/basic/HashingArray.h>
#include <ogdf/cluster/ClusterArray.h>

#include <ogdf/cluster/ClusterGraph.h>
#include <ogdf/basic/GraphAttributes.h>

//#include <ogdf/GmlParser.h>


namespace ogdf {

	class GmlParser;

	//stores information
	class ClusterInfo
	{
		public:
			ClusterInfo() 
			{
				//&m_fillPattern = GraphAttributes::bpNone; //NoBrush
				m_lineStyle = GraphAttributes::esSolid;
			}

			double m_x, m_y; //position of lower left corner 
			double m_w, m_h; //width and height

			double m_lineWidth; //width of rectangle border line

			String m_color;  //color of rectangle
			String m_fillColor;  //color of fill area
			String m_backColor;  //background color
			GraphAttributes::EdgeStyle m_lineStyle;  //rectangle line style
			GraphAttributes::BrushPattern m_fillPattern; //brush pattern of fill area
			String m_label;  //name label

			int m_clusterID; //the ID of the cluster of which the info is stored
	};


	//cluster attributes can be set over cluster or cluster id
	class ClusterGraphAttributes : public GraphAttributes
	{
		private:
			ClusterArray<String> m_clusterTemplate; // name of cluster template

		public:
			ClusterGraphAttributes() : GraphAttributes(), m_pClusterGraph(0) {} 
			ClusterGraphAttributes(ClusterGraph& cg, long initAttributes = 0);

			virtual ~ClusterGraphAttributes() {}

			virtual void init(ClusterGraph &cg, long initAttributes = 0);
			virtual void initAtt(long initAttributes = 0)
			{GraphAttributes::initAttributes(initAttributes);}

//			operator const ClusterGraph& () const {return *m_pClusterGraph;}
//			operator const Graph& () const {return m_pClusterGraph->getGraph();}

			const ClusterGraph& constClusterGraph() const {return *m_pClusterGraph;}

			int clusterID(node v) {return m_pClusterGraph->clusterOf(v)->index();}
			cluster clusterOf(node v) {return m_pClusterGraph->clusterOf(v);}

			int maxClusterID() const {return m_pClusterGraph->clusterIdCount()-1;}

			//*****************************************************************
			//data access by ID
			//returns x position of the cluster cages lower left corner
			double clusterXPos(int clusterID) const {return m_clusterInfo[clusterID].m_x;} 
			double& clusterXPos(int clusterID) {return m_clusterInfo[clusterID].m_x;} 
			//returns y position of the cluster cages lower left corner
			double clusterYPos(int clusterID) const {return m_clusterInfo[clusterID].m_y;} 
			double& clusterYPos(int clusterID) {return m_clusterInfo[clusterID].m_y;} 
			//returns cluster cage height
			double clusterHeight(int clusterID) const {return m_clusterInfo[clusterID].m_h;}
			double& clusterHeight(int clusterID) {return m_clusterInfo[clusterID].m_h;}
			//returns cluster cage width
			double clusterWidth(int clusterID) const {return m_clusterInfo[clusterID].m_w;}
			double& clusterWidth(int clusterID) {return m_clusterInfo[clusterID].m_w;}
			//returns cluster line width
			double clusterLineWidth(int clusterID) const 
				{return m_clusterInfo[clusterID].m_lineWidth;}
			double& clusterLineWidth(int clusterID) 
				{return m_clusterInfo[clusterID].m_lineWidth;}
			//returns cluster fill color
			String clusterFillColor(int clusterID) const 
				{return m_clusterInfo[clusterID].m_fillColor;}
			String& clusterFillColor(int clusterID) 
				{return m_clusterInfo[clusterID].m_fillColor;}
			//returns cluster fill color
			GraphAttributes::BrushPattern clusterFillPattern(int clusterID) const 
				{return m_clusterInfo[clusterID].m_fillPattern;}
			GraphAttributes::BrushPattern& clusterFillPattern(int clusterID) 
				{return m_clusterInfo[clusterID].m_fillPattern;}
			
			// returns label of cluster c
			const String &clusterLabel(int clusterID) const 
				{return m_clusterInfo[clusterID].m_label;}
			String &clusterLabel(int clusterID) 
				{return m_clusterInfo[clusterID].m_label;}

			const ClusterInfo& clusterInfo(int clusterID) const {return m_clusterInfo[clusterID];}
		    ClusterInfo& clusterInfo(int clusterID) {return m_clusterInfo[clusterID];}
			
			//*****************************************************************
			//data access by cluster
			//returns x position of the cluster cages lower left corner
			double clusterXPos(cluster c) const {return m_clusterInfo[c->index()].m_x;} 
			double& clusterXPos(cluster c) {return m_clusterInfo[c->index()].m_x;} 
			//returns y position of the cluster cages lower left corner
			double clusterYPos(cluster c) const {return m_clusterInfo[c->index()].m_y;} 
			double& clusterYPos(cluster c)  {return m_clusterInfo[c->index()].m_y;} 
			//returns cluster cage height
			double clusterHeight(cluster c) const {return m_clusterInfo[c->index()].m_h;}
			double& clusterHeight(cluster c) {return m_clusterInfo[c->index()].m_h;}
			//returns cluster cage width
			double clusterWidth(cluster c) const {return m_clusterInfo[c->index()].m_w;}
			double& clusterWidth(cluster c) {return m_clusterInfo[c->index()].m_w;}
			
			// returns label of cluster c
			const String &clusterLabel(cluster c) const 
				{return m_clusterInfo[c->index()].m_label;}
			String &clusterLabel(cluster c) 
				{return m_clusterInfo[c->index()].m_label;}

			const String &templateCluster(cluster c) const { return m_clusterTemplate[c]; }
			// returns template of cluster c
			String &templateCluster(cluster c) { return m_clusterTemplate[c]; }

			const ClusterInfo& clusterInfo(cluster c) const {return m_clusterInfo[c->index()];}
			ClusterInfo& clusterInfo(cluster c) {return m_clusterInfo[c->index()];}

			const String &clusterColor(cluster c) const 
				{return  m_clusterInfo[c->index()].m_color;}
			// returns color of cluster c
			String &clusterColor(cluster c) 
				{return  m_clusterInfo[c->index()].m_color;}

			//fill and background color
			const String &clusterFillColor(cluster c) const 
				{return  m_clusterInfo[c->index()].m_fillColor;}
			// returns color of cluster c
			String &clusterFillColor(cluster c) 
				{return  m_clusterInfo[c->index()].m_fillColor;}
			const String &clusterBackColor(cluster c) const 
				{return  m_clusterInfo[c->index()].m_backColor;}
			// returns color of cluster c
			String &clusterBackColor(cluster c) 
				{return  m_clusterInfo[c->index()].m_backColor;}

			//pen and brush styles
			const GraphAttributes::EdgeStyle &clusterLineStyle(cluster c) const 
				{return  m_clusterInfo[c->index()].m_lineStyle;}
			// returns line style of cluster c
			GraphAttributes::EdgeStyle &clusterLineStyle(cluster c) 
				{return  m_clusterInfo[c->index()].m_lineStyle;}

			const GraphAttributes::BrushPattern &clusterFillPattern(cluster c) const 
				{return  m_clusterInfo[c->index()].m_fillPattern;}
			// returns pattern of cluster c
			GraphAttributes::BrushPattern &clusterFillPattern(cluster c) 
				{return  m_clusterInfo[c->index()].m_fillPattern;}

			const double &clusterLineWidth(cluster c) const 
				{return  m_clusterInfo[c->index()].m_lineWidth;}
			// returns pattern of cluster c
			double &clusterLineWidth(cluster c) 
				{return  m_clusterInfo[c->index()].m_lineWidth;}

			//set pattern over index
			void setClusterFillPattern(cluster c, int i)
			{
				m_clusterInfo[c->index()].m_fillPattern = intToPattern(i);
			}//setclusterfillpattern
			//set style over index
			void setClusterLineStyle(cluster c, int i)
			{
				m_clusterInfo[c->index()].m_lineStyle = intToStyle(i);
			}//setclusterlinestyle

			

			const DRect boundingBox() const;

			// writes attributed clustergraph in GML format to file fileName
			void writeGML(const char *fileName);
			void writeGML(ostream& os);

			bool readClusterGML(const char* fileName, 
								ClusterGraph& CG, 
								Graph& G);
			bool readClusterGML(istream& is, 
								ClusterGraph& CG, 
								Graph& G);

			//will be private
			//read Cluster Graph with Attributes, base graph G, from fileName
			//with a given gmlparser (has objecttree)
			//don't need is if parser given!
			bool readClusterGraphGML(const char* fileName, 
									 ClusterGraph& CG, 
									 Graph& G,
									 GmlParser& gml);
			//read from input stream
			bool readClusterGraphGML(istream& is, 
									 ClusterGraph& CG, 
									 Graph& G,
									 GmlParser& gml);

		protected:
			const ClusterGraph* m_pClusterGraph;//only points to existing graphs

		private:
			//information on the cluster positions, index is cluster ID
			HashingArray<int, ClusterInfo> m_clusterInfo;

			// recursively write the cluster structure in GML
		void writeCluster(ostream &os,
						  NodeArray<int> &nId,
						  ClusterArray<int> & cId,
						  int &nextId,
						  cluster c,
						  String indent);

		// recursively write the cluster structure in GraphWin GML
		void writeGraphWinCluster(ostream &os,
						  NodeArray<int> &nId,
						  int &nextId,
						  cluster c,
						  String indent);

	}; //AttributedCluster


} // end namespace ogdf


#endif
