/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:37:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares ClusterOrthoLayout which represents an orthogonal
 * planar drawing algorithm for c-planar c-connected Clustergraphs
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif


#ifndef OGDF_CLUSTER_ORTHO_LAYOUT_H
#define OGDF_CLUSTER_ORTHO_LAYOUT_H


#include <ogdf/module/UMLPlanarLayoutModule.h>
#include <ogdf/orthogonal/OrthoRep.h>
#include <ogdf/cluster/ClusterPlanRep.h>
#include <ogdf/cluster/CPlanarEdgeInserter.h>


namespace ogdf {

	enum OrthoDir;


//---------------------------------------------------------
// ClusterOrthoLayout
// represents planar orthogonal drawing algorithm for
// c-planar c-connected Clustergraphs
//---------------------------------------------------------
class ClusterOrthoLayout : public UMLPlanarLayoutModule
{
public:
	// constructor
	ClusterOrthoLayout();

  
	// calls planar UML layout algorithm. Input is a planarized representation
	// PG of a connected component of the graph, output is a layout of the
	// (modified) planarized representation in drawing
	void call(ClusterPlanRep &PG, adjEntry adjExternal, Layout &drawing);
	//version for non c-planar graphs
	void call(ClusterPlanRep &PG, 
			  adjEntry adjExternal, 
			  Layout &drawing,
			  List<NodePair>& npEdges,
			  List<edge>& newEdges,
			  Graph& originalGraph);

	void call(PlanRepUML & /* PG */, adjEntry /* adjExternal */, Layout & /* drawing */) {}

	//
	// options

	// the minimum distance between edges and vertices
	double separation() const {
		return m_separation;
	}

	void separation(double sep) {
		m_separation = sep;
	}

	// cOverhang * separation is the minimum distance between the glue point
	// of an edge and a corner of the vertex boundary
	double cOverhang() const {
		return m_cOverhang;
	}

	void cOverhang(double c) {
		m_cOverhang = c;
	}


	// the distance from the tight bounding box to the boundary of the drawing
	double margin() const {
		return m_margin;
	}

	void margin(double m) {
		m_margin = m;
	}


	// the prefered direction of generalizations
	OrthoDir preferedDir() const {
		return m_preferedDir;
	}

	void preferedDir(OrthoDir dir) {
		m_preferedDir = dir;
	}

	// cost of associations
	int costAssoc() const {
		return m_costAssoc;
	}

	void costAssoc(int c) {
		m_costAssoc = c;
	}

	// cost of generalizations
	int costGen() const {
		return m_costGen;
	}

	void costGen(int c) {
		m_costGen = c;
	}

    //set the option profile, thereby fixing a set of drawing options
    void optionProfile(int i) {m_optionProfile = i;} 

	//set alignment option
	void align(bool b) {m_align = b;}

	//set scaling compaction
	void scaling(bool b) {m_useScalingCompaction = b;}

	//set generic options by setting field bits, 
	//necessary to allow setting over base class pointer
	//bit 0 = alignment
	//bit 1 = scaling
	//bit 2 = progressive/traditional
	//=> 0 is standard
	//spaeter nur noch optionprofile und die abfrage ueber memberfunc
	virtual void setOptions(int optionField) 
	{
		if (optionField & 1) m_align = true;
		else m_align = false;
		if (optionField & 2) m_useScalingCompaction = true;
		else m_useScalingCompaction = false;
		if (optionField & 4) m_orthoStyle = 1; 
		else m_orthoStyle = 0; //traditional
	}


private:
	// compute bounding box and move final drawing such that it is 0 aligned
	// respecting margins
	void computeBoundingBox(const ClusterPlanRep &PG, Layout &drawing);


	// options
	double m_separation;
	double m_cOverhang;
	double m_margin;
	OrthoDir m_preferedDir;
    int m_optionProfile;
	int m_costAssoc;
	int m_costGen;
    //align merger sons on same level
    bool m_align;
	//settings for scaling compaction
	bool m_useScalingCompaction;
	int m_scalingSteps;
	//mainly used for OrthoFormers traditional/progressive
    int m_orthoStyle;
};


} // end namespace ogdf


#endif
