/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 12:40:06 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class ClusterPlanarizationLayout
 * Planarization approach for cluster graphs
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_CLUSTER_PLANARIZATION_LAYOUT_H
#define OGDF_CLUSTER_PLANARIZATION_LAYOUT_H



#include <ogdf/module/UMLLayoutModule.h>
#include <ogdf/module/PlanarSubgraphModule.h>
#include <ogdf/module/EdgeInsertionModule.h>
#include <ogdf/module/UMLPlanarLayoutModule.h>
#include <ogdf/module/CCLayoutPackModule.h>
#include <ogdf/basic/ModuleOption.h>

#include <ogdf/cluster/ClusterGraph.h>
#include <ogdf/cluster/CconnectClusterPlanarEmbed.h>
#include <ogdf/cluster/ClusterPlanRep.h>
#include <ogdf/cluster/ClusterOrthoLayout.h>


namespace ogdf {


struct ClusterPosition
{
	double m_minx, m_maxx, m_miny, m_maxy, m_width, m_height;
};

//---------------------------------------------------------
// ClusterPlanarizationLayout
// planarization approach for drawing UML diagrams
//---------------------------------------------------------
class ClusterPlanarizationLayout : public UMLLayoutModule
{
public:
	// construction / destruction
	ClusterPlanarizationLayout();
	virtual ~ClusterPlanarizationLayout() { }

	// call UML layout algorithm. Input graph is given by umlGraph, layout
	// information are in attributes of umlGraph
	// this is not yet implemented for UMLClusterGraphs because there
	// is no definition of UML clusters right now
	// must be overridden by derived classes

	virtual void call(UMLGraph& /* umlG */) {THROW(AlgorithmFailureException);}
	//virtual void call(Graph& G, UMLGraph& umlGraph, ClusterGraph& cGraph);

	//call with ClusterGraphAttributes input for standard orthogonal layout
	virtual void call(Graph& G, 
					  ClusterGraphAttributes& acGraph, 
					  ClusterGraph& cGraph,
					  bool simpleCConnect = true);
	virtual void call(Graph & /* G */, ClusterGraph& /* cGraph */) {}

	// returns current page ratio (= desired width / height of layout)
	double pageRatio() const {
		return m_pageRatio;
	}

	// sets page ratio
	void pageRatio(double ratio) {
		m_pageRatio = ratio;
	}

	// module option for the computation of the planar subgraph
	void setSubgraph(PlanarSubgraphModule *pSubgraph) {
		m_subgraph.set(pSubgraph);
	}

	// module option for the edge insertion module
	void setInserter(EdgeInsertionModule *pInserter) {
		m_inserter.set(pInserter);
	}

	// module option for the planar layout algorithm
	void setPlanarLayouter(UMLPlanarLayoutModule *pPlanarLayouter) {
		m_planarLayouter.set(pPlanarLayouter);
	}

	// module option for the arrangement of connected components
	void setPacker(CCLayoutPackModule *pPacker) {
		m_packer.set(pPacker);
	}


	// returns number of crossings in layout produced in last call
	int numberOfCrossings() const {
		return m_nCrossings;
	}


	/*
    //insert supersink in connected component represented by PG
    void insertSuperSink(PlanRepUML& PG, UMLGraph& UG, int indexCC);
	*/

    //exit if precondition violated
    void assureDrawability(ClusterGraphAttributes& PG);
    //return if no precondition violated
    bool checkDrawability(ClusterGraphAttributes& PG);

    //draw hierarchies in same direction
    void syncHierarchies(bool b) {m_syncTrees = b;}

	//draw hierarchy nodes corresponding to their level
	void alignSons(bool b) 
	{
		int opts = m_planarLayouter.get().getOptions();

		if (b) m_planarLayouter.get().setOptions(opts | umlOpAlign); 
	    else  m_planarLayouter.get().setOptions(opts & ~umlOpAlign);
	}

	//set the option field for the orthogonal layouter
	void setLayouterOptions(int ops)
	{m_planarLayouter.get().setOptions(ops);}


protected:

	void computeClusterPositions(ClusterPlanRep& CP, 
								 Layout drawing,
								 HashingArray<int, ClusterPosition>& CA) 
	{

		edge e;
		forall_edges(e, CP)
		{
			if (CP.isClusterBoundary(e))
			{
				ClusterPosition cpos;
				//only boundaries => no originals
				//bool sOr = (CP.original(e->source()) != 0);
				//bool tOr = (CP.original(e->target()) != 0);
				//minimum vertex position values
				double minx, maxx, miny, maxy;
				//if (sOr && tOr)
				//{
					minx = min(drawing.x(e->source()), drawing.x(e->target()));
					maxx = max(drawing.x(e->source()), drawing.x(e->target()));
					//( drawing.x(e->source()) < drawing.x(e->target()) ?
					//		 drawing.x(e->source()) : drawing.x(e->target()) );
					miny = min(drawing.y(e->source()), drawing.y(e->target()));
					maxy = max(drawing.y(e->source()), drawing.y(e->target()));
				//double y = ( drawing.y(e->source()) < drawing.y(e->target()) ?
				//			 drawing.y(e->source()) : drawing.y(e->target()) );
/*
				}//else
				else
				{
					if (sOr) 
					{
						minx = maxx = drawing.x(e->source());
						miny = maxy = drawing.y(e->source());
					}
					else
					{
						if (tOr)
						{
							minx = maxx = drawing.x(e->target());
							miny = maxy = drawing.y(e->target());
						}//if
						else
						{
						  int i;
						  i++;
						}
					}//else
				
				}//else
				*/

				//check if x,y of lower left corner have to be updated
				if (CA.isDefined(CP.ClusterID(e)))
				{
					//ClusterPosition& cpos = CA[CP.ClusterID(e)];
					cpos = CA[CP.ClusterID(e)];
		
					double preValmaxx = cpos.m_maxx;
					double preValmaxy = cpos.m_maxy;

					if (cpos.m_minx > minx) cpos.m_minx = minx;
					if (cpos.m_miny > miny) cpos.m_miny = miny;
					if (preValmaxx < maxx) cpos.m_maxx = maxx;
					if (preValmaxy < maxy) cpos.m_maxy = maxy;

				}//if
				else
				{
					cpos.m_minx = minx;
					cpos.m_maxx = maxx;
					cpos.m_miny = miny;
					cpos.m_maxy = maxy;
				}//else

				//not necessary for all boundary edges, but we only have the ids
				//to adress, they may have holes
				cpos.m_width = cpos.m_maxx - cpos.m_minx;
				cpos.m_height = cpos.m_maxy - cpos.m_miny;

				//write values back
				//if (sOr || tOr)
				CA[CP.ClusterID(e)] = cpos;

			}//if is Boundary
		}//foralledges
	}//computeClusterPositions

private:
	// options
	ModuleOption<PlanarSubgraphModule>  m_subgraph;
	ModuleOption<EdgeInsertionModule>   m_inserter;
	ModuleOption<UMLPlanarLayoutModule> m_planarLayouter;
	ModuleOption<CCLayoutPackModule>    m_packer;

	double m_pageRatio;


	// information
	int m_nCrossings;

    //option: all hierarchies in the same direction
    bool m_syncTrees;

    //option: rearrange labels
    bool m_arrangeLabels;

    //temporary changes to avoid errors
    List<edge> m_fakedGens; //made to associations
    bool m_fakeTree;
    virtual void postProcess(ClusterGraphAttributes& PG); //redo
};


} // end namespace ogdf


#endif
