/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 12:40:05 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares class DavidsonHarel which implements the
 * Davidson-Harel approach for drawing graphs.
 * 
 * \author Rene Weiskircher
 * 
 * \todo: use node sizes for initial radius
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_DAVIDSON_HAREL_H
#define OGDF_DAVIDSON_HAREL_H


#include <ogdf/internal/energybased/EnergyFunction.h>


namespace ogdf {

	//! tuning of radius
	/**
	 * if called with tpFree, the settings are used unchanged,
	 * if tpCoarse, worse candidate energies are allowed and a 
	 * bigger initial radius is used.
	 * you can use these parameters to override the initial
	 * standard settings
	 */
	enum TuningParameter {tpFree, tpFine, tpCoarse};


//! Class implementing the Davidson-Harel approach for drawing graphs
class DavidsonHarel {
public:
	DavidsonHarel();
	//! Sets the start temperature to startTemp
	void setStartTemperature(int startTemp);
    //! Sets number of iterations for each temperature step to step
	void setNumberOfIterations(int steps);
	//! adds an Energy Function F with a certain weight
	void addEnergyFunction(EnergyFunction *F, double weight);
	//! returns a list of the names of the energy functions
	List<String> returnEnergyFunctionNames();
	//! returns a list of the weights of the energy functions
	List<double> returnEnergyFunctionWeights();
	//! calls the DavidsonHarel method for graph AG with tuning parameter fineTune
	void call(GraphAttributes &AG, TuningParameter fineTune = tpFree);
	~DavidsonHarel() {};

private:

	TuningParameter m_fineTune; //!< should worse candidates be allowed, set in call()

	int m_temperature; //!< Temperature during the annealing process
	double m_shrinkingFactor; //!< factor for radius
	double m_diskRadius; //!< Radius of the disk around the old position of a vertex where the new position will be
	double m_energy; //!< Current energy of the system
	List<EnergyFunction*> m_energyFunctions; //!< list of the energy functions
	int m_numberOfIterations;//!< number of iterations per temperature step
	const static int m_defaultTemp;//!< the default starting temp defined in cpp file
	const static double m_PI;//!< the constant PI
	const static double m_defaultRadius;//!< the default starting radius set in cpp file
	const static int m_iterationMultiplier;//!< per default,the number of iterations per temperature are set as a constant multiple of the number of vertices. The default multiplier is set in the cpp file
	const static double m_coolingFactor;//!< the fraction by which the temperature is lowered after a temperature step is finished
	const static double m_shrinkFactor; //!< the constant by which the radius of the circle around each vertex is shrunk when the temperature is lowered
	List<double> m_weightsOfEnergyFunctions;//!< list of the weights for the energy functions
	List<node> m_nonIsolatedNodes; //!< list of nodes with degree greater 0
	//! reset the parameters for subsewuent runs
	void initParameters(); 
	//! randomly computes a node and a new position for that node
	node computeCandidateLayout(const GraphAttributes &, DPoint &) const;
	//! test if new energy value satisfies annealing property (only better if m_fineTune)
	bool testEnergyValue(double newVal);
	//! computes a random number between zero and one
	inline double randNum() const;
	//! compute the first disk radius as the half the diamter of the enclosing rectangle
	void computeFirstRadius(const GraphAttributes &AG);
	//! computes the energy of the initial layout and stores it in m_energy
	void computeInitialEnergy();
	//! computes position for the vertices of degree zero
	void placeIsolatedNodes(GraphAttributes &AG) const;
	//! fake assignment operator (dummy to avoid copying)
	DavidsonHarel& operator=(const DavidsonHarel &dh);
	//! fake copy constructor (dummy to avoid copying)
	DavidsonHarel(const DavidsonHarel &dh) {};
};

} //end namespace
#endif
