/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 12:40:06 $ 
 ***************************************************************/
 
/** \file
 * \brief Declares class DavidsonHarelLayout, which is a front-end
 * for the fDavidsonHarel class.
 * 
 * \author Rene Weiskircher
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/



#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_DAVIDSON_HAREL_LAYOUT_H
#define OGDF_DAVIDSON_HAREL_LAYOUT_H


#include <ogdf/module/LayoutModule.h>
#include <ogdf/energybased/DavidsonHarel.h>


namespace ogdf {

//! easy way to set fixed costs
enum SettingsParameter {spStandard, spRepulse, spPlanar}; //tuning of costs
//! easy way to set temperature and iterations
enum SpeedParameter {sppFast, sppMedium, sppHQ};


//! A frontend for the DavidsonHarel class. 
/**
 * It contains standard values for the weights of the energy functions. 
 * It also prepares the graph by dealing with isolated vertices, self-
 * loops and multi-edges.
 */
class DavidsonHarelLayout : public LayoutModule
{
public:
	DavidsonHarelLayout();
	~DavidsonHarelLayout(){}

	//! calls the layout function for graph AG
	void call(GraphAttributes &AG);

	//! fix the cost values to special configurations
    void fixSettings(SettingsParameter sp);

	//! set the fineness of the candidate disc radisu
	void setStepping(TuningParameter tp);

	//! more convenient way of setting the speed of the algorithm. Influences number
	//! of iterations per temperature step, starting temperature, cooling factor
	void setSpeed(SpeedParameter sp);

	//! set the preferred edge length Multiplier for attraction
	//! this is bad design, cause you dont need to have an attraction function,
	//! DH is purely modular and independent with its cost functions
	void setPreferredEdgeLengthMultiplier(double multi) {m_multiplier = multi;}
	//! sets the weight for the corresponding energy function
	void setRepulsionWeight(double w);
	//! reporta the weight for the corresponding energy function
	double getRepulsionWeight() const {return m_repulsionWeight;}
	//! sets the weight for the corresponding energy function
	void setAttractionWeight(double);
	//! reporta the weight for the corresponding energy function
	double getAttractionWeight() const {return m_attractionWeight;}
	//! sets the weight for the corresponding energy function
	void setNodeOverlapWeight(double);
	//! reporta the weight for the corresponding energy function
	double getNodeOverlapWeight() const {return m_nodeOverlapWeight;}
	//! sets the weight for the corresponding energy function
	void setPlanarityWeight(double);
	//! reporta the weight for the corresponding energy function
	double getPlanarityWeight() const {return m_planarityWeight;}
	//! sets the starting temperature
	void setStartTemperature(int);
	//! reports the starting temperature
	int getStartTemperature() const {return m_startTemperature;}
	//! sets number of iterations per temperature step
	void setNumberOfIterations(int);
	//! reporta number of iterations per temperature step
	int getNumberOfIterations() const {return m_numberOfIterations;}
private:
	double m_repulsionWeight;//!< weight for repulsion energy
	double m_attractionWeight;//!< weight for attraction energy
	double m_nodeOverlapWeight;//!< weight for node overlap energy
	double m_planarityWeight;//!< weight for edge crossing energy
	int m_startTemperature;//!< temperature at the start of the optimization
	int m_numberOfIterations;//!< number of iterations per temperature step
	TuningParameter m_stepping;//!< this defines if worst solutions can be chosen
	SpeedParameter m_speed;     //!< you can override this by manually setting iter=0
	double m_multiplier;//!< by default, number of iterations per temperature step is number of vertices multiplied by multiplier
	bool m_crossings; //!< should crossings be computed
};

}
#endif
