/*
 * $Revision: 1.9 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/30 14:27:17 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of Fast Multipole Multilevel Method (FM^3).
 * 
 * \author Stefan Hachul
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2006 oreas GmbH
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif
 
#ifndef OGDF_FMMMLAYOUT_H
#define OGDF_FMMMLAYOUT_H

#include <ogdf/basic/Graph.h>
#include <ogdf/module/LayoutModule.h>
#include <ogdf/basic/geometry.h>
#include "../../src/energybased/FruchtermanReingold.h"
#include "../../src/energybased/NMM.h"

  
namespace ogdf {

	class Rectangle;

/**
 * \brief The fast multipole multivelvel layout algorithm.
 */
class FMMMLayout : public LayoutModule
  {
  public:

     //enumerations
     enum PageFormatType { pfPortrait, pfLandscape, pfSquare };
     enum QualityVsSpeed { qvsGorgeousAndEfficient, qvsBeautifulAndFast,
			   qvsNiceAndIncredibleSpeed};
     enum EdgeLengthMeasurement { elmMidpoint, elmBoundingCircle };
     enum AllowedPositions { apAll, apInteger, apExponent };
     enum TipOver { toNone, toNoGrowingRow, toAlways };
     enum PreSort { psNone, psDecreasingHeight, psDecreasingWidth };
     enum GalaxyChoice { gcUniformProb, gcNonUniformProbLowerMass, 
			 gcNonUniformProbHigherMass };
     enum MaxIterChange { micConstant, micLinearlyDecreasing, micRapidlyDecreasing };
     enum InitialPlacementMult { ipmSimple, ipmAdvanced };
     enum ForceModel { fmFruchtermanReingold, fmEades, fmNew };
     enum RepulsiveForcesMethod { rfcExact, rfcGridApproximation, rfcNMM };
     enum StopCriterion { scFixedIterations, scThreshold, 
			    scFixedIterationsOrThreshold };
     enum InitialPlacementForces { ipfUniformGrid, ipfRandomTime, ipfRandomRandIterNr};
     enum ReducedTreeConstruction { rtcPathByPath, rtcSubtreeBySubtree };
     enum SmallestCellFinding { scfIteratively, scfAluru };
  
     //! Creates an instance of the layout algorithm.
     FMMMLayout();
 
     //destructor
     ~FMMMLayout(); 
 
 
     //! Calls the algorithm for graph \a AG and returns the layout information in \a AG.
     void call(GraphAttributes &AG);
 
     //! Extended algorithm call: Allows to pass desired lengths of the edges. 
     /**
	  * The EdgeArray \a edgeLength must be valid for AG.constGraph() and its values must 
      * be positive.
	  */
     void call(GraphAttributes &AG,   //graph and layout
	       const EdgeArray<double> &edgeLength); //factor for desired edge lengths 

     //! Extended algorithm call: Calls the algorithm for graph \a AG.
     /**
	  * Returns layout information in \a AG and a simple drawing is saved in file \a ps_file
      * in postscript format (Nodes are drawn as uniformly sized circles). 
	  */
     void call(GraphAttributes &AG, char* ps_file);
 
     //! Extend algorithm call: Allows to pass desired lengths of the edges.
	 /**
	  * The EdgeArray \a edgeLength must be valid for AG.constGraph() and its values must
	  * be positive.
      * A simple drawing is saved in file ps_file in postscript format (Nodes are drawn 
      * as uniformly sized circles).  
	  */
     void call(GraphAttributes &AG,   //graph and layout
	       const EdgeArray<double> &edgeLength, //factor for desired edge lengths
	       char* ps_file); 

     //GetCpuTime
     //! Returns the runtime (=CPU-time) of the layout algorithm in seconds. 
     double getCpuTime();
  
     //UseHighLevelOptions 
     //! Defines if the high level options or the low level options are used in calls of the algorithm FMMMLayout.
     bool useHighLevelOptions() const { return m_useHighLevelOptions; }
     void useHighLevelOptions(bool uho) { m_useHighLevelOptions = uho; } 

     //%%%%%%%%%%%%%%%%%%%%%%%%%%% high level options %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     //%%%%%%%%%%%%%%%%%%%%%%%%%%(most relevant options)%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     //PageFormat
     //! Defines the desired aspect ratio of the drawing area.
     /**
	  *  - \a pfPortrait: A4 page in portrait orientation
      *  - \a pfLandscape: A4 page in landscape orientation
      *  - \a pfSquare: square page format
	  */
     PageFormatType pageFormat() const { return m_pageFormat; }
     void pageFormat(PageFormatType t) { m_pageFormat = t; }
 
     //UnitEdgeLength:
     //! The desired unit length of every edge.
     double unitEdgeLength() const { return m_unitEdgeLength; }
     void unitEdgeLength(double x) {m_unitEdgeLength = (( x > 0.0) ? x : 1);}

     //NewInitialPlacement
     /**
	  * Defines if the initial placement of the nodes at the coarsest multilevel
      * is varied for each distinct call of FMMMLayout or keeps always the same. 
	  */
     bool newInitialPlacement() const { return m_newInitialPlacement; }
     void newInitialPlacement(bool nip) { m_newInitialPlacement = nip; }
  
     //QualityVersusSpeed:
     //! Indicates if the algorithm is tuned either for best quality or best speed. 
	 /**
      *  - \a qvsGorgeousAndEfficient: gorgeous quality and efficient speed
      *  - \a qvsBeautifulAndFast: beautiful quality and fast speed
      *  - \a qvsNiceAndIncredibleSpeed: nice quality and incredible speed
	  */
     QualityVsSpeed qualityVersusSpeed() const { return m_qualityVersusSpeed; }
     void qualityVersusSpeed(QualityVsSpeed qvs) {m_qualityVersusSpeed = qvs; }
        
     //%%%%%%%%%%%%%%%%%%%%% low level options %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     //%%%%%%%%(note: those options are for experts or interested people only)%%%%%% 

     //%%%%%%%%%%%%%%%%%%%%% (a) general options %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     //RandSeed
     //! Sets the seed of the random number generator. 
     void randSeed(int p) { m_randSeed = ((0<=p) ? p : 1);}
     int randSeed() const {return m_randSeed;}
         
     //EdgeLengthMeasurement
     //! Indicates how the length of an edge is measured.
	 /**
	  * Possible values:
      *   - \a elmMidpoint: from center to center
      *   - \a elmBoundingCircle: the distance between the two tight circles bounding the 
      *     graphics of two adjacent nodes
	  */
     EdgeLengthMeasurement edgeLengthMeasurement() const { return 
							     m_edgeLengthMeasurement; }
     void edgeLengthMeasurement(EdgeLengthMeasurement elm) { m_edgeLengthMeasurement = 
							       elm; }
 
     //AllowedPositions
     //! Defines which positions for a node are allowed.
	 /**
	  * Possibly values:
      *   - \a apAll: every position is allowed
      *   - \a apInteger: only integer positions in the range depending on the number of 
      *     nodes
      *   - \a apExponent: only integer positions in the range of -2^MaxIntPosExponent to
      *     2^MaxIntPosExponent
	  */
     AllowedPositions allowedPositions() const { return m_allowedPositions; }
     void allowedPositions(AllowedPositions ap) { m_allowedPositions = ap; }
 
     //MaxIntPosExponent
     //! Defines the exponent used when allowedPositions() == \a apExponent.
     int maxIntPosExponent() const { return m_maxIntPosExponent; }
     void maxIntPosExponent(int e) {m_maxIntPosExponent = (((e >= 31)&&(e<=51))? 
							   e : 31);}

     //%%%%%%%%%%%%%%%%%%% (b) options for divide et impera step %%%%%%%%%%%%%%%%%%%%

     //PageRatio
     //! Defines the desired aspect ratio of the rectangular drawing area.
     double pageRatio() const { return m_pageRatio; }
     void pageRatio(double r) {m_pageRatio = (( r > 0) ? r : 1);}
 
     //StepsForRotatingComponents
     /**
	  * Indicates the number of times that each connected component is rotated with 
      * angles between 0 and 90 degree to obtain a bounding rectangle with small area.
	  */
     int stepsForRotatingComponents() const { return m_stepsForRotatingComponents; }
     void stepsForRotatingComponents(int n) { m_stepsForRotatingComponents = 
						((0<=n) ? n : 0);}
 
     //TipOverCCs
     //! Defines in which case it is allowed to tip over drawings of connected components.
	 /**
	  * Possible values:
      *   - \a toNone: not allowed at all
      *   - \a toNoGrowingRow: only if the height of the packing row does not grow
      *   - \a toAlways: always allowed
	  */
     TipOver tipOverCCs() const { return m_tipOverCCs; }
     void tipOverCCs(TipOver to) { m_tipOverCCs = to; }
 
     //MinDistCC
     //! The minimal distance between connected components.
     double minDistCC() const { return m_minDistCC; }
     void minDistCC(double x) { m_minDistCC = (( x > 0) ? x : 1);}
  
     //PresortCCs
     //! Defines if the connected components are sorted before the packing algorithm is applied.
	 /**
	  * Possible values:
      *   - \a psNone: no sorting
      *   - \a psDecreasingHeight: sorted by decreasing height
      *   - \a psDecreasingWidth: sorted by decreasing width
	  */
     PreSort presortCCs() const { return m_presortCCs; }
     void presortCCs(PreSort ps) { m_presortCCs = ps; }
 
     //%%%%%%%%%%%%%%%%%%%%%% (c) options for multilevel step   %%%%%%%%%%%%%%%%%%%%%%%%%

     //MinGraphSize
     /**
	  * The number of nodes of a graph in the multilevel representation for which no
      * more collapsing of galaxies is performed (i.e. the graph at the highest level).
	  */
     int minGraphSize() const { return m_minGraphSize; }
     void minGraphSize(int n) { m_minGraphSize = ((n >= 2)? n : 2);}
 
     //GalaxyChoice
     //! Defines how sun nodes of galaxies are selected.
	 /**
	  * Possible values:
      *   - \a gcUniformProb: selecting by uniform random probability
      *   - \a gcNonUniformProbLowerMass: selecting by non-uniform probability depending on 
      *     the star masses (prefering nodes with lower star mass)
      *   - \a gcNonUniformProbHigherMass: as above but prefering nodes with higher star mass
	  */
     GalaxyChoice galaxyChoice() const { return m_galaxyChoice; }
     void galaxyChoice(GalaxyChoice gc) { m_galaxyChoice = gc; }
 
     //RandomTries
     /**
	  * The number of tries to get a random node with minimal star mass (used in case
      * of galaxyChoice() == gcNonUniformProbLowerMass and galaxyChoice() == 
      * gcNonUniformProbHigherMass).
	  */
     int randomTries() const { return m_randomTries; }
     void randomTries(int n) {m_randomTries = ((n>=1)? n: 1);}
 
     //MaxIterChange
     //! Defines how MaxIterations is changed in subsequent multilevels.
	 /**
	  * Possible values:
      *   - \a micConstant: kept constant at the force calculation step at every level
      *   - \a micLinearlyDecreasing: linearly decreasing from MaxIterFactor*FixedIterations
      *     to FixedIterations
      *   - \a micRapidlyDecreasing: rapdily decreasing from MaxIterFactor*FixedIterations
      *      to FixedIterations
	  */
     MaxIterChange maxIterChange() const { return m_maxIterChange; }
     void maxIterChange(MaxIterChange mic) { m_maxIterChange = mic; }
 
     //MaxIterFactor
     /**
	  * The factor used for decrasing MaxIterations (in case of maxIterChange() ==
      * micLinearlyDecreasing or maxIterChange() == micRapidlyDecreasing).
	  */
     int maxIterFactor() const { return m_maxIterFactor; }
     void maxIterFactor(int f) { m_maxIterFactor = ((f>=1) ? f : 1 );}   
 
     //InitialPlacementMulti
     //! Defines how the initial placement is generated.
	 /**
	  * Possible values:
      *   - \a ipmSimple: only using information about placement of nodes on higher levels
      *   - \a ipmAdvanced: using additional information about the placement of all inter
      *   - \a  solar system nodes
	  */
     InitialPlacementMult initialPlacementMult() const { return 
							     m_initialPlacementMult; }
     void initialPlacementMult(InitialPlacementMult ipm) { 
       m_initialPlacementMult = ipm; }

     //%%%%%%%%%%%%%%%%%%%% (d) options tor force calculation step %%%%%%%%%%%%%%%%%%%%%%

     //ForceModel
     //! Defines the used force model.
	 /**
	  * Possibly values:
      *   - \a fmFruchtermanReingold: model of Fruchterman and Reingold
      *   - \a fmEades: model of Eades
      *   - \a fmNew: new model
	  */
     ForceModel forceModel() const { return m_forceModel; }
     void forceModel(ForceModel fm) { m_forceModel = fm; }
 
     //SpringStrength
     //! The strength of the springs.
     double springStrength() const { return m_springStrength; }
     void springStrength(double x) { m_springStrength  = ((x > 0)? x : 1);}
 
     //RepForcesStrength
     //! The strength of the repulsive forces.
     double repForcesStrength() const { return m_repForcesStrength; }
     void repForcesStrength(double x) { m_repForcesStrength =((x > 0)? x : 1);}
          
     //RepulsiveForcesCalcualtion
     //! Defines how to calculate repulsive forces.
	 /**
	  * Possible values:
      *   - \a rfcExact: exact calculation (slow)
      *   - \a rfcGridApproximation: grid approxiamtion (inaccurate)
      *   - \a rfcNMM: like in NMM (= New Multipole Method; fast and accurate)
	  */
     RepulsiveForcesMethod repulsiveForcesCalculation() const {
         return m_repulsiveForcesCalculation;
     }
     void repulsiveForcesCalculation(RepulsiveForcesMethod rfc) {
         m_repulsiveForcesCalculation = rfc;
     }
 
     //StopCriterion
     //! Defines the stop criterion.
	 /**
	  * Possible values:
      *   - \a rscFixedIterations: stop if fixedIterations() is reached
      *   - \a rscThreshold: stop if threshold() is reached
      *   - \a rscFixedIterationsOrThreshold: stop if fixedIterations() or threshold() 
      *     is reached
	  */
     StopCriterion stopCriterion() const { return m_stopCriterion; }
     void stopCriterion(StopCriterion rsc) { m_stopCriterion = rsc; }
 
     //Threshold
     //! Threshold for stop criterion.
	 /**
	  * (If the average absolute value of all forces in 
      * an iteration is less then threshold() then stop.)
	  */
     double threshold() const { return m_threshold; }
     void threshold(double x) {m_threshold = ((x > 0) ? x : 0.1);}
       
     //FixedIterations
     //! fixed  number of iterations for stop criterion.
     int fixedIterations() const { return m_fixedIterations; }
     void fixedIterations(int n) { m_fixedIterations = ((n >= 1) ? n : 1);}
         
     //ForceScalingFactor
     //! The scaling factor for the forces.
     double forceScalingFactor() const { return m_forceScalingFactor; }
     void forceScalingFactor(double f) { m_forceScalingFactor = ((f > 0) ? f : 1);}
        
     //CoolTemperature
     /**
	  * If set to true, forces are scaled by coolValue()^(actual iteration) * 
      * forceScalingFactor(); otherwise forces are scaled by forceScalingFactor().
	  */
     bool coolTemperature() const { return m_coolTemperature; }
     void coolTemperature(bool b) { m_coolTemperature = b; }
 
     //CoolValue
     //! The value by which forces are decreased if coolTemperature() is true.
     double coolValue() const { return m_coolValue; }
     void coolValue(double x) { m_coolValue = (((x >0 )&&(x<=1) )? x : 0.99);}
 
 
     //InitialPlacementForces
     //! Defines how initial placement is done.
	 /**
	  * Possible values:
      *   - \a ipfUniformGrid: uniform on a grid
      *   - \a ipfRandomTime: random based on actual time
      *   - \a ipfRandomRandIterNr: random based on randIterNr()
	  */
     InitialPlacementForces initialPlacementForces() const { return 
							     m_initialPlacementForces; }
     void initialPlacementForces(InitialPlacementForces ipf) { 
       m_initialPlacementForces = ipf; }
  
    //%%%%%%%%%%%%%%%%%%% (e) options for postprocessing step %%%%%%%%%%%%%%%%%%%%%

     //ResizeDrawing
     /**
	  * If set to true, the resulting drawing is resized so that the average edge 
      * length is the desired edge length times resizingScalar().
	  */
     bool resizeDrawing() const { return m_resizeDrawing; }
     void resizeDrawing(bool b) { m_resizeDrawing = b; }
 
     //ResizingScalar
     //! A parameter to scale the drawing if resizeDrawing() is true.
     double resizingScalar() const { return m_resizingScalar; }
     void resizingScalar(double s) { m_resizingScalar = ((s > 0) ? s : 1);}
 
     //FineTuningIterations
     //! The number of iterations for fine tuning.
     int fineTuningIterations() const { return m_fineTuningIterations; }
     void fineTuningIterations(int n) { m_fineTuningIterations =((n >= 0) ? n : 0);}
 
     //FineTuneScalar
     //! A parameter for scaling the forces in the fine-tuning iterations.
     double fineTuneScalar() const { return m_fineTuneScalar; }
     void fineTuneScalar(double s) { m_fineTuneScalar = ((s >= 0) ? s : 1);}

     //AdjustPostRepStrengthDynamically
     /**
	  * If set to true, the strength of the repulsive force field is calcualted 
      * dynamically by a formula depending on the number of nodes; otherwise the 
      * strength are scaled by PostSpringStrength and PostStrengthOfRepForces.
	  */
     bool adjustPostRepStrengthDynamically() const { return
						    m_adjustPostRepStrengthDynamically; }
     void adjustPostRepStrengthDynamically(bool b) { m_adjustPostRepStrengthDynamically 
						       = b; }

     //PostSpringStrength
     //! The strength of the springs in the postprocessing step.
     double postSpringStrength() const { return m_postSpringStrength; }
     void postSpringStrength(double x) { m_postSpringStrength  = ((x > 0)? x : 1);}
 
     //PostStrengthOfRepForces
     //! The strength of the repulsive forces in the postprocessing step.
     double postStrengthOfRepForces() const { return m_postStrengthOfRepForces; }
     void postStrengthOfRepForces(double x) { m_postStrengthOfRepForces =
						((x > 0)? x : 1);}

     //%%%%%%%%%% (f) options for repulsive force approximation methods  %%%%%%%%%%%%%%%
  
     //FRGridQuotient
     /**
	  * The number k of rows and colums of the grid is sqrt(|V|) / frGridQuotient()  
      * (Note that in [FrRe] frGridQuotient() is 2.) 
	  */
     void frGridQuotient(int p) { m_frGridQuotient = ((0<=p) ? p : 2);}
     int  frGridQuotient() const {return m_frGridQuotient;}

     //NMTreeConstruction
     //! Defines how the reduced bucket quadtree is constructed.
	 /**
	  * Possible values:
      *   - \a rtcPathByPath: path by path construction
      *   - \a rtcSubtreeBySubtree: subtree by subtree construction
	  */
     ReducedTreeConstruction nmTreeConstruction() const { return m_NMTreeConstruction; }
     void nmTreeConstruction(ReducedTreeConstruction rtc) { m_NMTreeConstruction = rtc; }
 
     //NMSmallCell
     /**
	  * Defines how the smallest quadratic cell that surrounds the particles of a node 
      * in the reduced bucket quadtree is calcualted. Possible values:
      *   - \a scfIteratively: iteratively (in constant time)
      *   - \a scfAluru: by the formula by Aluru et al. (in constant time)
	  */
     SmallestCellFinding nmSmallCell() const { return m_NMSmallCell; }
     void nmSmallCell(SmallestCellFinding scf) { m_NMSmallCell = scf; }
  
     //NMParticlesInLeaves
     //! Maximal number of particles that are contained in a leave of the reduced bucket quadtree.
     int nmParticlesInLeaves() const { return m_NMParticlesInLeaves; }
     void nmParticlesInLeaves(int n) { m_NMParticlesInLeaves = ((n>= 1)? n : 1);}
       
     //NMPrecision
     //! The precision p for the p-term multipole expansions.
     int nmPrecision() const { return m_NMPrecision; }
     void nmPrecision(int p) { m_NMPrecision  = ((p >= 1 ) ? p : 1);}

  private:
      
     //high level options
     bool                  m_useHighLevelOptions;
     PageFormatType        m_pageFormat;
     double                m_unitEdgeLength;
     bool                  m_newInitialPlacement;
     QualityVsSpeed        m_qualityVersusSpeed;

     //low level options
     //general options
     int                   m_randSeed;    
     EdgeLengthMeasurement m_edgeLengthMeasurement;
     AllowedPositions      m_allowedPositions;
     int                   m_maxIntPosExponent;
 
     //options for divide et impera step
     double                m_pageRatio;
     int                   m_stepsForRotatingComponents;
     TipOver               m_tipOverCCs;
     double                m_minDistCC;
     PreSort               m_presortCCs;
 
     //options for multilevel step
     int                   m_minGraphSize;
     GalaxyChoice          m_galaxyChoice;
     int                   m_randomTries;
     MaxIterChange         m_maxIterChange;
     int                   m_maxIterFactor;
     InitialPlacementMult m_initialPlacementMult;
 
     //options for force calculation step
     ForceModel            m_forceModel;
     double                m_springStrength;
     double                m_repForcesStrength;
     RepulsiveForcesMethod m_repulsiveForcesCalculation;
     StopCriterion         m_stopCriterion;
     double                m_threshold;
     int                   m_fixedIterations;
     double                m_forceScalingFactor;
     bool                  m_coolTemperature;
     double                m_coolValue;
     InitialPlacementForces m_initialPlacementForces;
 
     //options for postprocessing step
     bool                  m_resizeDrawing;
     double                m_resizingScalar;
     int                   m_fineTuningIterations;
     double                m_fineTuneScalar;
     bool                  m_adjustPostRepStrengthDynamically;
     double                m_postSpringStrength;
     double                m_postStrengthOfRepForces;
 
     //options for repulsice force approximation methods
     
     int                   m_frGridQuotient;      
     ReducedTreeConstruction m_NMTreeConstruction;
     SmallestCellFinding   m_NMSmallCell;
     int                   m_NMParticlesInLeaves;
     int                   m_NMPrecision;

     //other variables
     double max_integer_position; //!< maximum value for an integer position  
     double cool_factor; //!< needed for scaling the forces if coolTemperature() is true
     double average_ideal_edgelength; //!< measured from center to center
     double boxlength; //!< holds the length of the quadratic comput. box
     int number_of_components; //!< the number of components of the graph
     DPoint down_left_corner; //!< holds down left corner of the comput. box
     NodeArray<double> radius; //!< holds the radius of the sur. circle for each node
     double time_total; //!< runtime (=CPU-time) of the algorithm in seconds
     
     FruchtermanReingold FR; //!< classes for repulsive force calculation
     NMM NM;

      //------------------- most important functions ----------------------------

      //Calls the divide (decomposition into conncted components) and impera (drawing
      //and packing of the componenets) step of FMMM.
      void call_DIVIDE_ET_IMPERA_step (Graph& G,NodeArray<NodeAttributes>& A,
				       EdgeArray<EdgeAttributes>& E);
          
      //Calls the multilevel step for subGraph G. 
      void call_MULTILEVEL_step_for_subGraph (Graph& G,NodeArray<NodeAttributes>
					      & A,EdgeArray<EdgeAttributes>& E, int
					      comp_index);

      //Calls the force calculation step for G,A,E. If act_level is 0 and resizeDrawing()
      //is true the drawing is resized. Furthermore, the maximum number of force calc.
      //steps is calculated depending on MaxIterChange, act_level, and max_level.
      void call_FORCE_CALCULATION_step (Graph& G,NodeArray<NodeAttributes>& A,
					EdgeArray<EdgeAttributes>& E,int act_level,
					int max_level);

      //Calls the postprocessing step. 
      void call_POSTPROCESSING_step(Graph& G, NodeArray<NodeAttributes>& A,EdgeArray
				    <EdgeAttributes>& E,NodeArray<DPoint>& F,NodeArray
				    <DPoint>& F_attr, NodeArray<DPoint>& F_rep,
				    NodeArray<DPoint>& last_node_movement);

      //---------------- functions for pre/pos-processing -----------------------------

      //All parameter options are set to the default values.
      void initialize_all_options();

      //Updates several low level parameter options due to the settings of the high level
      //parameter options. 
      void update_low_level_options_due_to_high_level_options_settings();

      //Imports for each node v of G its width, height and position(given from GA) in A.
      void import_NodeAttributes(const Graph& G, GraphAttributes& GA, NodeArray
                  <NodeAttributes>& A);

      //Imports for each edge e of G its desired length given via edgeLength. 
      void import_EdgeAttributes (const Graph& G, const EdgeArray<double>& edgeLength, 
				  EdgeArray <EdgeAttributes>& E);

      //Sets the individual ideal edglength for each edge e. 
      void init_ind_ideal_edgelength(const Graph& G,NodeArray<NodeAttributes>&A, 
				     EdgeArray <EdgeAttributes>& E);

      //The radii of the surrounding circles of the bounding boxes are computed.
      void set_radii(const Graph& G,NodeArray<NodeAttributes>& A);
 
      //Exports for each node v in G_reduced the position of the original_node in G
      //(given  via A_reduced) to GA.
      void export_NodeAttributes(Graph& G_reduced, NodeArray<NodeAttributes>& A_reduced,
				 GraphAttributes& GA);

      //Creates a simple and loopfree copy of G and stores the corresponding
      //Node/EdgeAttributes in A_reduced/E_reduced. The links to the copy_node and
      //original node are stored in A, A_reduced, too.
      void make_simple_loopfree(const Graph& G,NodeArray<NodeAttributes>& 
				A, EdgeArray<EdgeAttributes>E, Graph& G_reduced,
				NodeArray<NodeAttributes>& A_reduced,EdgeArray
				<EdgeAttributes>& E_reduced);
      
      //Deletes parallel edges of G_reduced; Saves for each set of parallel edges one
      //representative edge in S and saves in new_edgelength the new edgelength of 
      //this edge in G_reduced.
      void delete_parallel_edges(const Graph& G,EdgeArray<EdgeAttributes>& E,Graph& 
		       	     G_reduced,List<edge>& S,EdgeArray<double>& new_edgelength);

      //Sets for each edge e of G_reduced in S its edgelength to the new_edgelength[e] 
      //and stores this information in E_reduced.
      void update_edgelength(List<edge>& S,EdgeArray <double>& new_edgelength,EdgeArray
			     <EdgeAttributes>& E_reduced);

      //Returns the value for the strength of the rep. forces in the postprocessing 
      //step depending on n = G.numberOfNodes()
      double get_post_rep_force_strength(int n);

      //Makes the node positions integers. If allowedPositions() == apInteger the 
      //values are in a range depending on G.number_of_nodes() and the 
      //average_ideal_edgelength. If allowed_positions() == apExponent the values 
      //are integers in a bounded integer range.
      void make_positions_integer(Graph& G, NodeArray<NodeAttributes>& A);

      //Cretes a simple drawing of the attributed graph AG in postscript format and 
      //saves it in file ps_file. 
      void create_postscript_drawing(GraphAttributes& AG, char* ps_file);

      //------------------ functions for divide et impera step -----------------------

      //Constructs the list of connected components of G (=: connected subgraphs of G)
      //and the corresponding lists with the Node/EdgeAttributes (containing a pointer
      //to the original node in G for each node in a subgraph). 
      void create_maximum_connected_subGraphs(Graph& G,NodeArray<NodeAttributes>&A, 
					   EdgeArray<EdgeAttributes>&E,Graph G_sub[],
					   NodeArray<NodeAttributes> A_sub[],EdgeArray
					   <EdgeAttributes> E_sub[],NodeArray<int>& 
					   component);
   
      //The drawings of the subgraphs are packet such that they do not overlap and
      //fit into a small box with the desired aspect ratio.
      void pack_subGraph_drawings (NodeArray<NodeAttributes>& A,Graph G_sub[],
				   NodeArray<NodeAttributes> A_sub[]);

      //The bounding rectangles of all connected componenents of G are calculated
      //and stored in R.
      void  calculate_bounding_rectangles_of_components(List<Rectangle>& R,Graph  
				 G_sub[],NodeArray<NodeAttributes> A_sub[]);

      //The bounding rectangle of the componenet_index-th. component of G is returned.
      Rectangle calculate_bounding_rectangle(Graph& G,NodeArray<NodeAttributes>& A,
					     int componenet_index);

      //If number_of_components > 1, the subgraphs G_sub are rotated and skipped to 
      //find bounding rectangles with minimum area. The information is saved in R and 
      //the node positions in A_sub are updated. If number_of_components == 1 a rotation 
      //with minimal aspect ratio is found instead.
      void rotate_components_and_calculate_bounding_rectangles(List<Rectangle>&R,Graph
					              G_sub[],NodeArray<NodeAttributes>
					              A_sub[]);

      //Returns the area (aspect ratio area) of a rectangle with width w and height h
      //if comp_nr > 1 ( comp_nr == 1).
      double calculate_area(double width,double height,int comp_nr);

      //The positions of the nodes in the subgraphs are calculated by using the 
      //information stored in R and are exported to A. (The coordinates of components
      //which surrounding rectangles have been tipped over in the packing step are
      //tipped over here,too)
      void export_node_positions(NodeArray<NodeAttributes>& A,List<Rectangle>& R,Graph  
				 G_sub[],NodeArray<NodeAttributes> A_sub[]);

      //Frees dynamically allocated memory for the connected component subgraphs.
      void delete_all_subGraphs(Graph G_sub[],NodeArray<NodeAttributes> A_sub[],
				EdgeArray<EdgeAttributes> E_sub[]);

      //------------------  functions for multilevel step    --------------------------
 
      //Returns the maximum number of iterations for the force calc. step depending
      //on act_level, max_level, FixedIterations, MaxIterChange, MaxIterFactor,
      //and the number of nodes of the Graph in the actual mutilevel. 
      int get_max_mult_iter(int act_level,int max_level,int node_nr);

      //------------------  functions for force calculation ---------------------------
   
      //the forces are calculated here.
      void calculate_forces(Graph& G, NodeArray<NodeAttributes>& A,EdgeArray
			    <EdgeAttributes>& E,NodeArray<DPoint>& F,NodeArray
			    <DPoint>& F_attr, NodeArray<DPoint>& F_rep,NodeArray
			    <DPoint>& last_node_movement,int iter,int fine_tuning_step);
      
      //The length of the computational box in the first iteration is set 
      //(down left corner is at (0,0).
      void init_boxlength_and_cornercoordinate(Graph& G,NodeArray<NodeAttributes>& A);

      //The initial placements of the nodes are created by using initialPlacementForces()
      void create_initial_placement (Graph& G,NodeArray<NodeAttributes>& A);

      //Sets all entries of F to (0,0).
      void  init_F (Graph& G, NodeArray<DPoint>& F);


      //Make initialisations for the data structures that are used in the choosen
      //class for rep. force calculation.
      void make_initialisations_for_rep_calc_classes(Graph& G,NodeArray<NodeAttributes>
						     &A, NodeArray<DPoint>& F_rep);

      //Calculates repulsive forces for each node. 
      void calculate_repulsive_forces(Graph &G,NodeArray<NodeAttributes>& A,NodeArray
				      <DPoint>& F_rep);

      //Deallocates dynamically allocated memory of the choosen rep. calculation class.
      void deallocate_memory_for_rep_calc_classes();

      //Calculates attractive forces for each node.
      void calculate_attractive_forces(Graph& G,NodeArray<NodeAttributes> & A,
				       EdgeArray<EdgeAttributes>& E, NodeArray
                                       <DPoint>& F_attr);
  
      //Returns the attractive force scalar.
      double f_attr_scalar (double d,double ind_ideal_edge_length);

      //Add attractive and repulsive forces for each node.
      void add_attr_rep_forces(Graph& G,NodeArray<DPoint>& F_attr,NodeArray<DPoint>& 
			       F_rep,NodeArray<DPoint>& F,int iter,int 
			       fine_tuning_step);

      //Move the nodes.
      void move_nodes(Graph& G,NodeArray<NodeAttributes>& A,NodeArray<DPoint>& F);

      //Computes a new tight computational square-box! (Guaranteeing, that all 
      //midpoints are inside the square.)
      void update_boxlength_and_cornercoordinate(Graph& G,NodeArray<NodeAttributes>& A);

      //Describes the max. radius of a move in one timestep, depending on the number of 
      //iterations.
      double max_radius(int iter);
 
      //The average_ideal_edgelength for all edges is computed.
      void set_average_ideal_edgelength(Graph& G,EdgeArray<EdgeAttributes>& E);

      //Calculates the average force on each node in the actual iteration, which is 
      //needed if StopCriterion is scThreshold() or scFixedIterationsOrThreshold().
      double get_average_forcevector_length (Graph& G, NodeArray<DPoint>& F);

      //Depending on the direction of last_node_movement[v], the length of the next 
      //displacement of node v is restricted.
      void prevent_oscilations(Graph& G, NodeArray<DPoint>& F,NodeArray<DPoint>&
                               last_node_movement,int iter);

      //Calculates the angle between PQ and PS in [0,2pi).
      double angle(DPoint& P,DPoint& Q, DPoint& R);

      //last_node_movement is initialized to F (used after first iteration).
      void init_last_node_movement(Graph& G, NodeArray<DPoint>& F,NodeArray
                                   <DPoint>& last_node_movement);  

      //If resizeDrawing is true, the drawing is adapted to the ideal average 
      //edgelength by shrinking respectively expanding the drawing area.
      void adapt_drawing_to_ideal_average_edgelength(Graph& G,NodeArray<NodeAttributes>
						     & A,EdgeArray<EdgeAttributes>& E);
						  
      //The force is restricted to have values within the comp. box (needed for 
      //exception handling, if the force is too large for further calculations).
      void restrict_force_to_comp_box(DPoint& force);

      //------------------- functions for analytic information -------------------------

      //Sets time_total to zero.
      void init_time(){time_total = 0;}

      //Returns the difference of user-time of t_last and the user-time of actual_time. 
      //clock_t get_time(tms t_last);

}; //FMMMLayout
} //end namespace ogdf 
#endif

