/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2005/11/18 15:25:27 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class GEMLayout.
 *
 * Fast force-directed layout algorithm (GEMLayout)
 * based on Frick et al.'s algorithm.
 *
 * \author Christoph Buchheim
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_FAST_LAYOUT_H
#define OGDF_FAST_LAYOUT_H

#include <ogdf/module/LayoutModule.h>

namespace ogdf {

	class GraphCopy;
	class GraphCopyAttributes;

//---------------------------------------------------------
// GEMLayout
//
// Fast force-directed layout algorithm. See
// - A. Frick, A. Ludwig, H. Mehldau: "A Fast Adaptive 
//	 Layout Algorithm for Undirected Graphs"
//	
//---------------------------------------------------------

class GEMLayout : public LayoutModule {

	// algorithm parameters (see below)

	int m_numberOfRounds;		
	double m_minimalTemperature;
	double m_maximalTemperature;
	double m_initialTemperature;
	double m_gravitationalConstant;
	double m_desiredLength; 
	double m_maximalDisturbance;		
	double m_rotationAngle;
	double m_oscillationAngle;
	double m_rotationSensitivity;
	double m_oscillationSensitivity;
	int m_attractionFormula;
	double m_minDistCC;
	double m_pageRatio;

	// node data used by the algorithm

	NodeArray<double> m_impulseX;			
		// x-coordinate of the last impulse of the node

	NodeArray<double> m_impulseY;			
		// y-coordinate of the last impulse of the node

	NodeArray<double> m_localTemperature;
		// local temperature of the node

	NodeArray<double> m_skewGauge;
		// skew gauge of the node

	// other data used by the algorithm

	double m_barycenterX;				
		// weighted sum of x-coordinates of all nodes

	double m_barycenterY;				
		// weighted sum of y-coordinates of all nodes
	
	double m_newImpulseX;
		// x-coordinate of the new impulse of the current node
	
	double m_newImpulseY;				
		// y-coordinate of the new impulse of the current node
	
	double m_globalTemperature;			
		// average of all node temperatures
	
	double m_cos;
		// cosine of m_oscillationAngle / 2
	
	double m_sin;
		// sine of (pi + m_rotationAngle) / 2

public:

	// constructor: sets options to default values
	GEMLayout();

	// copy constructor
	GEMLayout(const GEMLayout &fl);

	// destructor
	~GEMLayout();

	// assignment operator
	GEMLayout &operator=(const GEMLayout &fl);

	// calls the algorithm for an attributed graph AG
	// returns layout information in AG
	void call(GraphAttributes &AG);

	// option that determines the maximal number of rounds per node
	// n >= 0
	int numberOfRounds() const { return m_numberOfRounds; }
	void numberOfRounds(int n) {
		if(n < 0) n = 0;
		m_numberOfRounds = n;
	}

	// option that determines the minimal temperature
	// x >= 0
	double minimalTemperature() const { return m_minimalTemperature; }
	void minimalTemperature(double x) { 
		if(DIsLess(x,0)) x = 0;
		m_minimalTemperature = x; 
	}

	// option that determines the initial temperature
	// x >= minimalTemperature 
	double initialTemperature() const { return m_initialTemperature; }
	void initialTemperature(double x) { 
		if(DIsLess(x,m_minimalTemperature)) x = m_minimalTemperature;
		m_initialTemperature = x; 
	}

	// option that determines the gravitational constant
	// x >= 0
	double gravitationalConstant() const { return m_gravitationalConstant; }
	void gravitationalConstant(double x) { 
		if(DIsLess(x,0)) x = 0;
		m_gravitationalConstant = x; 
	}

	// option that determines the desired edge length
	// x >= 0
	double desiredLength() const { return m_desiredLength; }
	void desiredLength(double x) {
		if(DIsLess(x,0)) x = 0;
		m_desiredLength = x; 
	}

	// option that determines the maximal disturbance
	// x >= 0
	double maximalDisturbance() const { return m_maximalDisturbance; }
	void maximalDisturbance(double x) {
		if(DIsLess(x,0)) x = 0;
		m_maximalDisturbance = x; 
	}

	// option that determines the opening angle for rotations
	// 0 <= x <= pi / 2
	double rotationAngle() const { return m_rotationAngle; }
	void rotationAngle(double x) { 
		if(DIsLess(x,0)) x = 0;
		if(DIsGreater(x,pi / 2.0)) x = pi / 2.0;
		m_rotationAngle = x; 
	}

	// option that determines the opening angle for oscillations
	// 0 <= x <= pi/2
	double oscillationAngle() const { return m_oscillationAngle; }
	void oscillationAngle(double x) { 
		if(DIsLess(x,0)) x = 0;
		if(DIsGreater(x,pi / 2.0)) x = pi / 2.0;
		m_oscillationAngle = x; 
	}

	// option that determines the rotation sensitivity
	// 0 <= x <= 1
	double rotationSensitivity() const { return m_rotationSensitivity; }
	void rotationSensitivity(double x) { 
		if(DIsLess(x,0)) x = 0;
		if(DIsGreater(x,1)) x = 1;
		m_rotationSensitivity = x; 
	}

	// option that determines the oscillation sensitivity
	// 0 <= x <= 1
	double oscillationSensitivity() const { return m_oscillationSensitivity; }
	void oscillationSensitivity(double x) { 
		if(DIsLess(x,0)) x = 0;
		if(DIsGreater(x,1)) x = 1;
		m_oscillationSensitivity = x; 
	}

	// option that determines the formula for attraction
	// n = 1: Fruchterman / Reingold
	// n = 2: GEM
	int attractionFormula() const { return m_attractionFormula; }
	void attractionFormula(int n) { 
		if(n == 1 || n == 2) m_attractionFormula = n;
	}

	// option that determines the distance between connected components
	// 0 <= x
    const double minDistCC() { return m_minDistCC; }
    void minDistCC(double x) { m_minDistCC = x; }

	// option that determines the page ratio used for the layout of connected
	// components
	// 0 < x
    const double pageRatio() { return m_pageRatio; }
    void pageRatio(double x) { m_pageRatio = x; }

private:

	// returns the length of the vector (x,y)
	inline double length(double x,double y = 0) const {
		return sqrt(x * x + y * y);
	}

	// returns the weight of node v according to its degree
	inline double weight(node v) const {
		return (double)(v->degree()) / 2.5 + 1.0;
	}

	// compute the new impulse for node v
	void computeImpulse(GraphCopy &GC, GraphCopyAttributes &AGC,node v);

	// update the node data for node v
	void updateNode(GraphCopy &GC, GraphCopyAttributes &AGC,node v);

	OGDF_NEW_DELETE
};

} // end namespace ogdf

#endif

