/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of Spring-Embedder (Fruchterman,Reingold)
 *        algorithm.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_SPRING_EMBEDDER_FR_H
#define OGDF_SPRING_EMBEDDER_FR_H


#include <ogdf/module/LayoutModule.h>
#include <ogdf/basic/Array2.h>


namespace ogdf {


	class GraphCopyAttributes;
	class GraphCopy;


//---------------------------------------------------------
// SpringEmbedderFR
// Sprin-Embedder algorithm by Fruchterman, Reingold
//---------------------------------------------------------
class SpringEmbedderFR : public LayoutModule
{
public:

	// constructor: sets options to default values
	SpringEmbedderFR();

	// destructor
	~SpringEmbedderFR() { }


	// calls the algorithm for attributed graph AG
	// returns layout information in AG
	void call(GraphAttributes &AG);


    int iterations() const {
		return m_iterations;
	}

    void iterations(int i) {
		if (i>0)
			m_iterations = i;
	}

    bool noise() const {
		return m_noise;
	}

    void noise(bool on) {
		m_noise = on;
	}
    
    const double minDistCC() { return m_minDistCC; }
    void minDistCC(double x) { m_minDistCC = x; }

    const double pageRatio() { return m_pageRatio; }
    void pageRatio(double x) { m_pageRatio = x; }

	// parameter for for scaling / translating the starting layout

	// type of scaling used by algorithm
	//   scInput: bounding box of input is used
	//   scUserBoundingBox: bounding box set by userBoundingBox() is used
	//   scScaleFunction: automatic scaling is used with parameter set by
	//                    scaleFunctionFactor() (larger factor, larger b-box
	enum Scaling { scInput, scUserBoundingBox, scScaleFunction };

	Scaling scaling() const {
		return m_scaling;
	}

	void scaling(Scaling sc) {
		m_scaling = sc;
	}

	double scaleFunctionFactor() const {
		return m_scaleFactor;
	}

	void scaleFunctionFactor(double f) {
		m_scaleFactor = f;
	}

	void userBoundingBox(double xmin, double ymin, double xmax, double ymax) {
		m_bbXmin = xmin;
		m_bbYmin = ymin;
		m_bbXmax = xmax;
		m_bbYmax = ymax;
	}

private:
    
    bool initialize(GraphCopy &G, GraphCopyAttributes &AG);

    void mainStep(GraphCopy &G, GraphCopyAttributes &AG);
    void cleanup() {
		delete m_A;
		m_A = 0;
	}

    
    NodeArray<ListIterator<node> > m_lit; 

    int m_cF;
    
    double m_width;
    double m_height; 

    double m_txNull;
    double m_tyNull;
    double m_tx;
    double m_ty;

    double m_k;
    double m_k2;
    double m_kk;
    int m_ki;
    
    int m_xA;
    int m_yA;

    Array2<List<node> > *m_A;


    double mylog2(int x) {
		double l = 0.0;
		while(x > 0) {
			l++;
			x >>= 1;
		}
		return l/2;
	}

    int    m_iterations;     // number of iterations
	double m_fineness;       // fineness of the grid
	double m_edgeLength;

    double m_xleft;          // coordinates of bounding box
    double m_xright;
    double m_ysmall;
    double m_ybig;

    bool m_noise;            // perform random perturbations

	Scaling m_scaling;       // scaling type
	double m_scaleFactor;    // factor used if scaling type is scScaleFunction
	// user defined bounding box used if scaling type is scUserBoundingBox
	double m_bbXmin;
	double m_bbYmin;
	double m_bbXmax;
	double m_bbYmax;

	double m_minDistCC;
	double m_pageRatio;
};


} // end namespace ogdf


#endif
