/** \file
 * \brief Computes a connected subgraph G' of G containing node n.
 * 
 * \author Thorsten Kerkhof (Thorsten.Kerkhof@udo.edu)
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_CONNECTED_SUBGRAPH_h
#define OGDF_CONNECTED_SUBGRAPH_h

#include <ogdf/basic/NodeArray.h>
#include <ogdf/basic/EdgeArray.h>

namespace ogdf {

template<class T>
class ConnectedSubgraph
{
public:
	//constructor
	ConnectedSubgraph() { }

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param return nSG is the corresponding node of nG in SG.
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 node& nSG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
	 * \param return nG_to_nSG is a mapping of nodes in G to nodes in SG.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG,
	                 NodeArray<node>& nG_to_nSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param return nSG is the corresponding node of nG in SG.
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
	 * \param edgeLengthG is saving for each edge of G its length.
	 * \param return edgeLengthSG is saving for each edge of SG its length.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 node& nSG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG,
	                 const EdgeArray<T>& edgeLengthG,
	                 EdgeArray<T>& edgeLengthSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
	 * \param edgeLengthG is saving for each edge of G its length.
	 * \param return edgeLengthSG is saving for each edge of SG its length.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG,
	                 const EdgeArray<T>& edgeLengthG,
	                 EdgeArray<T>& edgeLengthSG);

  /**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
   * \param return SG is the connected subgraph containing nG.
   * \param nSG_to_nG is mapping nodes in SG to nodes in G
   * \param eSG_to_eG is mapping edges in SG to edges in G
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
	 * \param edgeLengthG is saving for each edge of G its length.
	 * \param return edgeLengthSG is saving for each edge of SG its length.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
                   node& nSG,
                   NodeArray<node>& nSG_to_nG,
                   EdgeArray<edge>& eSG_to_eG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG,
	                 const EdgeArray<T>& edgeLengthG,
	                 EdgeArray<T>& edgeLengthSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
   * \param return SG is the connected subgraph containing nG.
   * \param nSG_to_nG is mapping nodes in SG to nodes in G
   * \param eSG_to_eG is mapping edges in SG to edges in G
	 * \param nG_to_nSG is mapping nodes in G to nodes in SG
   * \param eG_to_eSG is mapping edges in G to edges in SG
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param return nodeLengthSG is saving for each node of SG its length.
	 * \param edgeLengthG is saving for each edge of G its length.
	 * \param return edgeLengthSG is saving for each edge of SG its length.
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
                   node& nSG,
                   NodeArray<node>& nSG_to_nG,
                   EdgeArray<edge>& eSG_to_eG,
                   NodeArray<node>& nG_to_nSG,
                   EdgeArray<edge>& eG_to_eSG,
	                 const NodeArray<T>& nodeLengthG,
	                 NodeArray<T>& nodeLengthSG,
	                 const EdgeArray<T>& edgeLengthG,
	                 EdgeArray<T>& edgeLengthSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param nSG_to_nG is mapping nodes in SG to nodes in G
   * \param eSG_to_eG is mapping edges in SG to edges in G
   * \param nG_to_nSG is mapping nodes in G to nodes in SG
   * \param eG_to_eSG is mapping edges in G to edges in SG
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 NodeArray<node>& nSG_to_nG,
	                 EdgeArray<edge>& eSG_to_eG,
	                 NodeArray<node>& nG_to_nSG,
	                 EdgeArray<edge>& eG_to_eSG);

	/**
   * \brief Computes a connected subgraph SG of G containing node nG.
   * \param G is the original graph.
	 * \param return SG is the connected subgraph containing nG.
	 * \param nG is a node in G.
	 * \param nSG_to_nG is mapping nodes in SG to nodes in G
   */
	static void call(const Graph& G,
	                 Graph& SG,
	                 const node& nG,
	                 NodeArray<node>& nSG_to_nG);

private:
	/**
   * \brief Copies to SG node nG and recursively all adjacent edges
	 *  and nodes.
	 *
	 * \param SG is the connected subgraph.
	 * \param nodeVisited saves for all nodes in G, if they were already
	 *    treated.
	 * \param edgeVisited saves for all edges in G, if they were already
	 *    treated.
	 * \param nG is a node in G.
	 * \param nodeLengthG is saving for each node of G its length.
	 * \param nodeLengthSG is saving for each node of SG its length.
	 * \param edgeLengthG is saving for each edge of G its length.
	 * \param edgeLengthSG is saving for each edge of SG its length.
   * \param nSG_to_nG is mapping nodes in SG to nodes in G
   * \param eSG_to_eG is mapping edges in SG to edges in G
	 * \param nG_to_nSG is mapping nodes in G to nodes in SG
   * \param eG_to_eSG is mapping edges in G to edges in SG
   */
	static void recursion(Graph& SG,
	                      bool* nodeVisited,
	                      bool* edgeVisited,
	                      const node& nG,
	                      const NodeArray<T>& nodeLengthG,
	                      NodeArray<T>& nodeLengthSG,
	                      const EdgeArray<T>& edgeLengthG,
	                      EdgeArray<T>& edgeLengthSG,
	                      NodeArray<node>& nSG_to_nG,
	                      EdgeArray<edge>& eSG_to_eG,
	                      NodeArray<node>& nG_to_nSG,
	                      EdgeArray<edge>& eG_to_eSG);
};

} // end namespace ogdf

#endif
