/** \file
 * \brief Computes an embedding of a biconnected graph with maximum
 * external face. See paper "Graph Embedding with Minimum Depth and
 * Maximum External Face" by C. Gutwenger and P. Mutzel (2004) for
 * details.
 * The algorithm is combined with the algorithm for maximum external
 * layers which defines how to embed blocks into inner faces. See
 * diploma thesis "Algorithmen zur Bestimmung von guten
 * Graph-Einbettungen fr orthogonale Zeichnungen" (in german) of
 * Thorsten Kerkhof (2007) for details.
 * 
 * \author Thorsten Kerkhof (Thorsten.Kerkhof@udo.edu)
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_EMBEDDER_MAX_FACE_BICONNECTED_GRAPHS_Layers_H
#define OGDF_EMBEDDER_MAX_FACE_BICONNECTED_GRAPHS_Layers_H

#include <ogdf/decomposition/StaticSPQRTree.h>
#include <ogdf/basic/CombinatorialEmbedding.h>
#include <ogdf/planarity/PlanarModule.h>

//#define TK_DEBUG
//#define TK_DEBUG_FILES

namespace ogdf {

template<class T>
class EmbedderMaxFaceBiconnectedGraphsLayers
{
public:
	//constructor
	EmbedderMaxFaceBiconnectedGraphsLayers() { }

	/**
	 * \brief Embeds \a G by computing and extending a maximum face in \a G
	 *   containg \a n.
	 * \param G is the original graph.
	 * \param return adjExternal is an adjacency entry of the external face.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \param n is a node of the original graph. If n is given, a maximum face
	 *   containing n is computed, otherwise any maximum face.
	 */
	static void embed(Graph& G,
	                  adjEntry& adjExternal,
	                  const NodeArray<T>& nodeLength,
	                  const EdgeArray<T>& edgeLength,
	                  const node& n = 0);

	/**
	 * \brief Computes the component lengths of all virtual edges in spqrTree.
	 * \param G is the original graph.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \param spqrTree is the SPQR-tree of \a G.
	 * \param edgeLengthSkel is saving for each skeleton graph of the SPQR-tree
	 *   all edge lengths.
	 */
	static void compute(const Graph& G,
	                    const NodeArray<T>& nodeLength,
	                    const EdgeArray<T>& edgeLength,
	                    StaticSPQRTree& spqrTree,
	                    NodeArray< EdgeArray<T> >& edgeLengthSkel);
	
	/**
	 * \brief Returns the size of a maximum external face in \a G containing the node \a n.
	 * \param G is the original graph.
	 * \param n is a node of the original graph.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \return The size of a maximum external face in \a G containing the node \a n.
	 */
	static T computeSize(const Graph& G,
	                     const node& n,
	                     const NodeArray<T>& nodeLength,
	                     const EdgeArray<T>& edgeLength);

	/**
	 * \brief Returns the size of a maximum external face in \a G containing
	 *   the node \a n.
	 *
	 * \param G is the original graph.
	 * \param n is a node of the original graph.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \param spqrTree is the SPQR-tree of G.
	 * \return The size of a maximum external face in \a G containing the node \a n.
	 */
	static T computeSize(const Graph& G,
	                     const node& n,
	                     const NodeArray<T>& nodeLength,
	                     const EdgeArray<T>& edgeLength,
	                     StaticSPQRTree& spqrTree);

	/**
	 * \brief Returns the size of a maximum external face in \a G containing
	 *   the node \a n.
	 *
	 * \param G is the original graph.
	 * \param n is a node of the original graph.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \param spqrTree is the SPQR-tree of G.
	 * \param edgeLengthSkel is saving for each skeleton graph the length
	 *   of each edge.
	 * \return The size of a maximum external face in \a G containing the node \a n.
	 */
	static T computeSize(const Graph& G,
	                     const node& n,
	                     const NodeArray<T>& nodeLength,
	                     const EdgeArray<T>& edgeLength,
	                     StaticSPQRTree& spqrTree,
	                     const NodeArray< EdgeArray<T> >& edgeLengthSkel);

	/**
	 * \brief Returns the size of a maximum external face in \a G.
	 * \param G is the original graph.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \return The size of a maximum external face in \a G.
	 */
	static T computeSize(const Graph& G,
	                     const NodeArray<T>& nodeLength,
	                     const EdgeArray<T>& edgeLength);

	/**
	 * \brief Returns the size of a maximum external face in \a G.
	 *   The SPQR-tree is given. The computed component lengths are
	 *   computed and returned.
	 *
	 * \param G is the original graph.
	 * \param nodeLength is saving for each vertex in \a G its length.
	 * \param edgeLength is saving for each edge in \a G its length.
	 * \param spqrTree is the SPQR-tree of G.
	 * \param edgeLengthSkel is saving for each skeleton graph the length
	 *   of each edge.
 	 * \return The size of a maximum external face in \a G.
	 */
	static T computeSize(const Graph& G,
	                     const NodeArray<T>& nodeLength,
	                     const EdgeArray<T>& edgeLength,
	                     StaticSPQRTree& spqrTree,
	                     NodeArray< EdgeArray<T> >& edgeLengthSkel);

private:
	/**
	 * \brief Bottom up traversal of SPQR-tree computing the component length of
	 *   all non-reference edges.
	 * \param spqrTree is the SPQR-tree of \a G.
	 * \param mu is the SPQR-tree node treated in this function call.
	 * \param nodeLength is saving for each node of the original graph \a G its
	 *   length.
	 * \param edgeLength is saving for each skeleton graph the length of each
	 *   edge.
	 */
	static void bottomUpTraversal(StaticSPQRTree& spqrTree,
	                              const node& mu,
																const NodeArray<T>& nodeLength,
																NodeArray< EdgeArray<T> >& edgeLength);

	/**
	 * \brief Top down traversal of SPQR-tree computing the component length of
	 *   all reference edges.
	 * \param spqrTree is the SPQR-tree of \a G.
	 * \param mu is the SPQR-tree node treated in this function call.
	 * \param nodeLength is saving for each node of the original graph \a G its
	 *   length.
	 * \param edgeLength is saving for each skeleton graph the length of each
	 *   edge.
	 */
	static void topDownTraversal(StaticSPQRTree& spqrTree,
	                             const node& mu,
															 const NodeArray<T>& nodeLength,
															 NodeArray< EdgeArray<T> >& edgeLength);

	/**
	 * \brief Computes the size of a maximum face in the skeleton graph of \a mu
	 *   containing \a n.
	 * \param spqrTree is the SPQR-tree of \a G.
	 * \param mu is the SPQR-tree node treated in this function call.
	 * \param n is a node of the original graph \a G.
	 * \param nodeLength is saving for each node of the original graph \a G its
	 *   length.
	 * \param edgeLength is saving for each skeleton graph the length of each
	 *   edge.
	 */
	static T largestFaceContainingNode(const StaticSPQRTree& spqrTree,
	                                   const node& mu,
	                                   const node& n,
	                                   const NodeArray<T>& nodeLength,
	                                   const NodeArray< EdgeArray<T> >& edgeLength);

	/**
	 * \brief Computes the size of a maximum face in the skeleton graph of \a mu.
	 * \param spqrTree is the SPQR-tree of \a G.
	 * \param mu is the SPQR-tree node treated in this function call.
	 * \param nodeLength is saving for each node of the original graph \a G its
	 *   length.
	 * \param edgeLength is saving for each skeleton graph the length of each
	 *   edge.
	 */
	static T largestFaceInSkeleton(const StaticSPQRTree& spqrTree,
	                               const node& mu,
	                               const NodeArray<T>& nodeLength,
	                               const NodeArray< EdgeArray<T> >& edgeLength);

	/* \brief Computes recursively the thickness of the skeleton graph of all
	 *   nodes in the SPQR-tree.
	 *
	 * \param spqrTree The SPQR-tree of the treated graph.
	 * \param mu a node in the SPQR-tree.
	 * \param thickness saving the computed results - the thickness of each
	 *   skeleton graph.
	 * \param nodeLength is saving for each node of the original graph \a G its
	 *   length.
	 * \param edgeLength is saving the edge lengths of all edges in each skeleton
	 *   graph of all tree nodes.
	 */
	static void bottomUpThickness(const StaticSPQRTree& spqrTree,
	                              const node& mu,
	                              NodeArray<T>& thickness,
	                              const NodeArray<T>& nodeLength,
	                              const NodeArray< EdgeArray<T> >& edgeLength);

	/* \brief ExpandEdge embeds all edges in the skeleton graph \a S into an
	 *   existing embedding and calls recursively itself for all virtual edges
	 *   in S.
	 *
	 * \param spqrTree The SPQR-tree of the treated graph.
	 * \param treeNodeTreated is an array saving for each SPQR-tree node \a mu
	 *   whether it was already treated by any call of ExpandEdge or not. Every
	 *   \a mu should only be treated once.
	 * \param mu is a node in the SPQR-tree.
	 * \param leftNode is the node adjacent to referenceEdge, which should be "left"
	 *   in the embedding
	 * \param nodeLength is an array saving the lengths of the nodes of \a G.
	 * \param edgeLength is saving the edge lengths of all edges in each skeleton
	 *   graph of all tree nodes.
	 * \param thickness of each skeleton graph.
	 * \param newOrder is saving for each node \a n in \a G the new adjacency
	 *   list. This is an output parameter.
	 * \param adjBeforeNodeArraySource is saving for the source of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param adjBeforeNodeArrayTarget is saving for the target of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param delta_u the distance from the second adjacent face of the reference edge
	 *   except the external face to the external face of G.
	 * \param delta_d the distance from the external face to the external face of G.
	 * \param adjExternal is an adjacency entry in the external face.
	 * \param n is only set, if ExpandEdge is called for the first time, because
	 *   then there is no virtual edge which has to be expanded, but the max face
	 *   has to contain a certain node \a n.
	 */
	static void expandEdge(const StaticSPQRTree& spqrTree,
	                       NodeArray<bool>& treeNodeTreated,
	                       const node& mu,
	                       const node& leftNode,
	                       const NodeArray<T>& nodeLength,
	                       const NodeArray< EdgeArray<T> >& edgeLength,
	                       const NodeArray<T>& thickness,
	                       NodeArray< List<adjEntry> >& newOrder,
												 NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArraySource,
												 NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArrayTarget,
                         const T& delta_u,
                         const T& delta_d,
												 adjEntry& adjExternal,
												 const node& n = 0);

	/* \brief Embeds all edges in the skeleton graph \a S of an S-node of the
	 *   SPQR-tree into an existing embedding and calls recursively itself for
	 *   all virtual edges in S.
	 *
	 * \param spqrTree The SPQR-tree of the treated graph.
	 * \param treeNodeTreated is an array saving for each SPQR-tree node \a mu
	 *   whether it was already treated by any call of ExpandEdge or not. Every
	 *   \a mu should only be treated once.
	 * \param mu is a node in the SPQR-tree.
	 * \param leftNode is the node adjacent to referenceEdge, which should be "left"
	 *   in the embedding
	 * \param nodeLength is an array saving the lengths of the nodes of \a G.
	 * \param edgeLength is saving the edge lengths of all edges in each skeleton
	 *   graph of all tree nodes.
	 * \param thickness of each skeleton graph.
	 * \param newOrder is saving for each node \a n in \a G the new adjacency
	 *   list. This is an output parameter.
	 * \param adjBeforeNodeArraySource is saving for the source of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param adjBeforeNodeArrayTarget is saving for the target of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param delta_u the distance from the second adjacent face of the reference edge
	 *   except the external face to the external face of G.
	 * \param delta_d the distance from the external face to the external face of G.
	 * \param adjExternal is an adjacency entry in the external face.
	 */
	static void expandEdgeSNode(const StaticSPQRTree& spqrTree,
	                            NodeArray<bool>& treeNodeTreated,
	                            const node& mu,
	                            const node& leftNode,
	                            const NodeArray<T>& nodeLength,
	                            const NodeArray< EdgeArray<T> >& edgeLength,
	                            const NodeArray<T>& thickness,
	                            NodeArray< List<adjEntry> >& newOrder,
	                            NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArraySource,
	                            NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArrayTarget,
	                            const T& delta_u,
	                            const T& delta_d,
	                            adjEntry& adjExternal);

	/* \brief Embeds all edges in the skeleton graph \a S of an P-node of the
	 *   SPQR-tree into an existing embedding and calls recursively itself for
	 *   all virtual edges in S.
	 *
	 * \param spqrTree The SPQR-tree of the treated graph.
	 * \param treeNodeTreated is an array saving for each SPQR-tree node \a mu
	 *   whether it was already treated by any call of ExpandEdge or not. Every
	 *   \a mu should only be treated once.
	 * \param mu is a node in the SPQR-tree.
	 * \param leftNode is the node adjacent to referenceEdge, which should be "left"
	 *   in the embedding
	 * \param nodeLength is an array saving the lengths of the nodes of \a G.
	 * \param edgeLength is saving the edge lengths of all edges in each skeleton
	 *   graph of all tree nodes.
	 * \param thickness of each skeleton graph.
	 * \param newOrder is saving for each node \a n in \a G the new adjacency
	 *   list. This is an output parameter.
	 * \param adjBeforeNodeArraySource is saving for the source of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param adjBeforeNodeArrayTarget is saving for the target of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param delta_u the distance from the second adjacent face of the reference edge
	 *   except the external face to the external face of G.
	 * \param delta_d the distance from the external face to the external face of G.
	 * \param adjExternal is an adjacency entry in the external face.
	 */
	static void expandEdgePNode(const StaticSPQRTree& spqrTree,
	                            NodeArray<bool>& treeNodeTreated,
	                            const node& mu,
	                            const node& leftNode,
	                            const NodeArray<T>& nodeLength,
	                            const NodeArray< EdgeArray<T> >& edgeLength,
	                            const NodeArray<T>& thickness,
                              NodeArray< List<adjEntry> >& newOrder,
                              NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArraySource,
                              NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArrayTarget,
                              const T& delta_u,
                              const T& delta_d,
                              adjEntry& adjExternal);

	/* \brief Embeds all edges in the skeleton graph \a S of an R-node of the
	 *   SPQR-tree into an existing embedding and calls recursively itself for
	 *   all virtual edges in S.
	 *
	 * \param spqrTree The SPQR-tree of the treated graph.
	 * \param treeNodeTreated is an array saving for each SPQR-tree node \a mu
	 *   whether it was already treated by any call of ExpandEdge or not. Every
	 *   \a mu should only be treated once.
	 * \param mu is a node in the SPQR-tree.
	 * \param leftNode is the node adjacent to referenceEdge, which should be "left"
	 *   in the embedding
	 * \param nodeLength is an array saving the lengths of the nodes of \a G.
	 * \param edgeLength is saving the edge lengths of all edges in each skeleton
	 *   graph of all tree nodes.
	 * \param thickness of each skeleton graph.
	 * \param newOrder is saving for each node \a n in \a G the new adjacency
	 *   list. This is an output parameter.
	 * \param adjBeforeNodeArraySource is saving for the source of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param adjBeforeNodeArrayTarget is saving for the target of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param delta_u the distance from the second adjacent face of the reference edge
	 *   except the external face to the external face of G.
	 * \param delta_d the distance from the external face to the external face of G.
	 * \param adjExternal is an adjacency entry in the external face.
	 * \param n is only set, if ExpandEdge is called for the first time, because
	 *   then there is no virtual edge which has to be expanded, but the max face
	 *   has to contain a certain node \a n.
	 */
	static void expandEdgeRNode(const StaticSPQRTree& spqrTree,
	                            NodeArray<bool>& treeNodeTreated,
	                            const node& mu,
	                            const node& leftNode,
	                            const NodeArray<T>& nodeLength,
	                            const NodeArray< EdgeArray<T> >& edgeLength,
	                            const NodeArray<T>& thickness,
	                            NodeArray< List<adjEntry> >& newOrder,
	                            NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArraySource,
	                            NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArrayTarget,
	                            const T& delta_u,
	                            const T& delta_d,
	                            adjEntry& adjExternal,
	                            const node& n = 0);

	/* \brief Writes a given adjacency entry into the newOrder. If the edge
	 *   belonging to ae is a virtual edge, it is expanded.
	 *
	 * \param ae is the adjacency entry which has to be expanded.
	 * \param before is the adjacency entry of the node in \a G, before
	 *   which ae has to be inserted.
	 * \param spqrTree The SPQR-tree of the treated graph.
	 * \param treeNodeTreated is an array saving for each SPQR-tree node \a mu
	 *   whether it was already treated by any call of ExpandEdge or not. Every
	 *   \a mu should only be treated once.
	 * \param mu is a node in the SPQR-tree.
	 * \param leftNode is the node adjacent to referenceEdge, which should be "left"
	 *   in the embedding
	 * \param nodeLength is an array saving the lengths of the nodes of \a G.
	 * \param edgeLength is saving the edge lengths of all edges in each skeleton
	 *   graph of all tree nodes.
	 * \param thickness of each skeleton graph.
	 * \param newOrder is saving for each node \a n in \a G the new adjacency
	 *   list. This is an output parameter.
	 * \param adjBeforeNodeArraySource is saving for the source of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param adjBeforeNodeArrayTarget is saving for the target of the reference edge
	 *   of the skeleton of mu the adjacency entry, before which new entries have
	 *   to be inserted.
	 * \param delta_u the distance from the second adjacent face of the reference edge
	 *   except the external face to the external face of G.
	 * \param delta_d the distance from the external face to the external face of G.
	 * \param adjExternal is an adjacency entry in the external face.
	 */
	static void adjEntryForNode(adjEntry& ae,
                              ListIterator<adjEntry>& before,
                              const StaticSPQRTree& spqrTree,
                              NodeArray<bool>& treeNodeTreated,
                              const node& mu,
                              const node& leftNode,
                              const NodeArray<T>& nodeLength,
                              const NodeArray< EdgeArray<T> >& edgeLength,
                              const NodeArray<T>& thickness,
                              NodeArray< List<adjEntry> >& newOrder,
                              NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArraySource,
                              NodeArray< ListIterator<adjEntry> >& adjBeforeNodeArrayTarget,
                              const T& delta_u,
                              const T& delta_d,
                              adjEntry& adjExternal);

	/* \brief Single source shortest path.
	 *
	 * \param G directed graph
	 * \param s source node
	 * \param length length of an edge
	 * \param d contains shortest path distances after call
	 */
	static bool sssp(const Graph& G,
	                 const node& s,
	                 const EdgeArray<T>& length,
	                 NodeArray<T>& d);
};

} // end namespace ogdf

#endif
