/*
 * $Revision: 1.2 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2006/07/19 13:37:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of Hierarchy class.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_HIERARCHY_H
#define OGDF_HIERARCHY_H


#include <ogdf/basic/EdgeArray.h>
#include <ogdf/layered/Level.h>
#include <ogdf/basic/GraphCopy.h>


namespace ogdf {


//---------------------------------------------------------
// Hierarchy
// representation of proper hierarchies used by Sugiyama
//---------------------------------------------------------
class Hierarchy {
public:
	friend class Level;
	enum TraversingDir { downward, upward };

private:
	Array<Level *> m_pLevel; // array of all levels
	GraphCopy m_GC; // graph copy representing topology of proper hierarchy
	NodeArray<int> m_pos;  // position of a node in its level
	NodeArray<int> m_rank; // rank (level) of a node
	// (sorted) adjacent node on lower level
	NodeArray<Array<node> > m_lowerAdjNodes;
	// (sorted) adjacent node on upper level
	NodeArray<Array<node> > m_upperAdjNodes;

	NodeArray<int> m_nSet; //(only used by buildAdjNodes())

	// (only used by calculateCrossingsPlaneSweep()):
	NodeArray<ListIterator<node> > m_lastOcc;

	TraversingDir m_direction; // current direction of crossing minimization

public:
	// construction
	Hierarchy() { }
	Hierarchy(const Graph &G, const NodeArray<int> &rank);

	// destruction
	~Hierarchy();

	void createEmpty(const Graph &G);
	void initByNodes(const List<node> &nodes,
		EdgeArray<edge> &eCopy,
		const NodeArray<int> &rank);

	// conversion to const GraphCopy reference
	operator const GraphCopy &() const { return m_GC; }

	// returns the current direction of crossing minimization
	TraversingDir direction() const {
		return m_direction;
	}

	// sets the current direction of crossing minimization
	void direction (TraversingDir dir) {
		m_direction = dir;
	}

	// returns number of levels
	int size() const { return m_pLevel.size(); }
	// returns maximal array index of level (= size()-1)
	int high() const { return m_pLevel.high(); }

	// returns rank (level) of node v
	int rank(node v) const { return m_rank[v]; }

	// returns position of node v on its level
	int pos(node v) const { return m_pos[v]; }

	// returns adjacent nodes of v (according to direction())
	const Array<node> &adjNodes(node v) {
		return (m_direction == downward) ? m_lowerAdjNodes[v] :
		m_upperAdjNodes[v];
	}

	// returns the adjacent level of level i (according to direction())
	const Level &adjLevel(int i) const {
		return (m_direction == downward) ? *m_pLevel[i-1] : *m_pLevel[i+1];
	
	}

	bool isLongEdgeDummy(node v) const {
		return (m_GC.isDummy(v) && v->outdeg() == 1);
	}

	// returns level i
	const Level &operator[](int i) const { return *m_pLevel[i]; }
	// returns level i
	Level &operator[](int i) { return *m_pLevel[i]; }

	
	// computes number of crossings between level i and i+1
	int calculateCrossings(int i);
	// computes number of crossings
	int calculateCrossings();

	// old versions using plane-seep algorithm (only for testing purposes)
	int calculateCrossingsPlaneSweep(int i);
	int calculateCrossingsPlaneSweep();

	// computes simultaneous crossing number between level i and i+1
	int calculateCrossingsSimDraw(int i, const EdgeArray<unsigned int> *edgeSubGraph);
	int calculateCrossingsSimDraw(const EdgeArray<unsigned int> *edgeSubGraph);

	
	// stores position of nodes in oldPos
	void storePos (NodeArray<int> &oldPos);
	// restores position of nodes according to newPos
	void restorePos (const NodeArray<int> &newPos);

	// permutes order of nodes on each level
	void permute();
	bool transpose(node v);

	void print(ostream &os);

	void buildAdjNodes(int i);
	void buildAdjNodes();

	void check();

private:
	void doInit(const NodeArray<int> &rank);

	int transposePart(/*node v, node w,*/
		const Array<node> &adjV, const Array<node> &adjW);

	OGDF_MALLOC_NEW_DELETE
};


} // end namespace ogdf


#endif
