/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/15 10:08:46 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of Sugiyama algorithm.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_SUGIYAMA_LAYOUT_H
#define OGDF_SUGIYAMA_LAYOUT_H



#include <ogdf/module/LayoutModule.h>
#include <ogdf/module/RankingModule.h>
#include <ogdf/simultaneous/TwoLayerCrossMinSimDraw.h>
#include <ogdf/module/HierarchyLayoutModule.h>
#include <ogdf/module/HierarchyClusterLayoutModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/cluster/ClusterGraphAttributes.h>


namespace ogdf {

	class ExtendedNestingGraph;

//---------------------------------------------------------
// SugiyamaLayout
// Sugiyama drawing algorithm for hierarchical graphs
//---------------------------------------------------------
class SugiyamaLayout : public LayoutModule {

protected:

	ModuleOption<RankingModule>                m_ranking;
	ModuleOption<TwoLayerCrossMin>             m_crossMin;
	ModuleOption<TwoLayerCrossMinSimDraw>      m_crossMinSimDraw;
	ModuleOption<HierarchyLayoutModule>        m_layout;
	ModuleOption<HierarchyClusterLayoutModule> m_clusterLayout;

	int    m_fails;
	int    m_runs;
	bool   m_transpose;
	double m_minDistCC;
	double m_pageRatio;

	int m_nCrossings;
	RCCrossings m_nCrossingsCluster;
	Array<bool> m_levelChanged;

	// additional options for callUML()
	bool m_alignBaseClasses;
	bool m_alignSiblings;
	
	// simdraw
	EdgeArray<unsigned int> *m_subgraphs;

public:

	// constructor: sets options to default values
	SugiyamaLayout();

	// destructor
	~SugiyamaLayout() { }


	// calls the algorithm for attributed graph AG
	// returns layout information in AG
	void call(GraphAttributes &AG);

	void call(ClusterGraphAttributes &AG);

	// calls the algorithm for attributed graph AG and given level assignment
	// returns layout information in AG
	void call(GraphAttributes &AG, NodeArray<int> &rank);

	// special call for UML graphs
	void callUML(GraphAttributes &AG);

	// option that determines, how many times the total number of crossings
	// after a complete top down or bottom up traversal may not decrease before
	// one repetition is stopped.
	int fails() const { return m_fails; }
	void fails(int nFails) { m_fails = nFails; }

	// option that determines, how many times the crossing minimization is
	// repeated. Each repetition (except for the first) starts with randomly
	// permuted nodes on each layer. Deterministic behaviour can be achieved
	//by setting |runs| to $1$.
	int runs() const { return m_runs; }
	void runs(int nRuns) { m_runs = nRuns; }

	// option that if set to true performs an additional fine tuning step
	// after each traversal, which tries to reduce the total number of crossings
	// by switching adjacent vertices on the same layer.
	bool transpose() const { return m_transpose; }
	void transpose(bool bTranspose) { m_transpose = bTranspose; }

    const double minDistCC() { return m_minDistCC; }
    void minDistCC(double x) { m_minDistCC = x; }

    const double pageRatio() { return m_pageRatio; }
    void pageRatio(double x) { m_pageRatio = x; }

	// options for UML Hierarchical
	bool alignBaseClasses() const { return m_alignBaseClasses; }
	void alignBaseClasses(bool b) { m_alignBaseClasses = b; }

	bool alignSiblings() const { return m_alignSiblings; }
	void alignSiblings(bool b) { m_alignSiblings = b; }

	// simdraw-call data
	void setSubgraphs(EdgeArray<unsigned int> *esg) { m_subgraphs = esg; }
	bool useSubgraphs() const { return (m_subgraphs == 0) ? 0 : 1; }

	// module option for the computation of the node ranking (level assignment)
	void setRanking(RankingModule *pRanking) {
		m_ranking.set(pRanking);
	}

	// module option for the two-layer crossing minimization
	void setCrossMin(TwoLayerCrossMin *pCrossMin) {
		m_crossMin.set(pCrossMin);
	}

	// module option for the computation of the final layout
	void setLayout(HierarchyLayoutModule *pLayout) {
		m_layout.set(pLayout);
	}

	// module option for the computation of the final layout
	void setClusterLayout(HierarchyClusterLayoutModule *pLayout) {
		m_clusterLayout.set(pLayout);
	}

	// returns the number of crossings in the layout after the algorithm
	// has been aplied
	int numberOfCrossings() const { return m_nCrossings; }
	RCCrossings numberOfCrossingsCluster() const { return m_nCrossingsCluster; }

protected:

	void reduceCrossings(Hierarchy &H);
	void reduceCrossings(ExtendedNestingGraph &H);

private:
	void doCall(GraphAttributes &AG, bool umlCall);
	void doCall(GraphAttributes &AG, bool umlCall, NodeArray<int> &rank);

	int traverseTopDown (Hierarchy &H);
	int traverseBottomUp(Hierarchy &H);

	bool transposeLevel(int i, Hierarchy &H);
	void doTranspose(Hierarchy &H);
	void doTransposeRev(Hierarchy &H);

	RCCrossings traverseTopDown (ExtendedNestingGraph &H);
	RCCrossings traverseBottomUp(ExtendedNestingGraph &H);
};


}

#endif
