/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of compaction of turn-regular representations.
 *
 * This class implements constructive 1-dimensional compaction of
 * turn-regular representations based on min-cost flow on the dual
 * graph of a dissection graph special version to cope with expanded
 * nodes (expansion faces are turn-regular).
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_TR_COMPACTION_H
#define OGDF_TR_COMPACTION_H

#include <ogdf/orthogonal/OrthoRep.h>
#include <ogdf/planarity/PlanRepUML.h>
#include <ogdf/internal/orthogonal/RoutingChannel.h>
#include <ogdf/orthogonal/MinimumEdgeDistances.h>
#include <ogdf/basic/GridLayoutMapped.h>
#include <ogdf/internal/orthogonal/TRProcessor.h>


namespace ogdf {

	class Layout;


//---------------------------------------------------------
// TRCompaction
// represents compaction algorithm using min-cost flow on
// the dual of an oriented graph dissection
//---------------------------------------------------------
class TRCompaction
{
public:
	// construction, how to split to connect kitty corners, special costs
	TRCompaction(int splitnear = 1, //split to nearest/farest kitty corner 
		                            //in face cycle
                 int dissectdirection=1, //dissect 1=horizontally, 2=vertically, 3=random
                 int cagecost = 1, //cost of length unit for cage edges
                 int dissectcost = 1);

	// call of constructive heuristics for orthogonal representation
    //it is not (yet?) possible to improve the result using the same techniques
	void constructiveHeuristics(
		PlanRepUML &PG,
		OrthoRep &OR,
		const RoutingChannel<int> &rc,
		GridLayoutMapped &drawing);

	// options

	//set the direction of the first edge of the graph to a specific direction, mainly
	//for debugging purposes
	void setFirstDirection(OrthoDir d) {m_firstDir = d;}

	//number of heuristics used - which dissection and which computation method
	void setHeuristicsNr(int i) {m_heuristicsNr = i;} //todo soft checking of range
	int getHeuristicsNr() {return m_heuristicsNr;}

	//decide if splitting face to nearest/farest kitty corner
	void setSplitNear(bool b) { m_trProcessor->setSplitNear(b);} 
	//todo use ints and allow arbitrary strategies
	bool getSplitNear() {return m_trProcessor->getSplitNear();}

	//set the preferred dissection direction
	void setDisDir(int i) {m_dissectDir = i;} //todo soft check range
	int getDisDir() {return m_dissectDir;}

	//number of inserted edges in dissection (0 before)
	int getSplitCount() {return m_trProcessor->getSplitCount();}

protected:
	//preparation section

    //set orientation and construct saturators
	void buildup();

	//computation section

	//insert saturators
    void setSaturators();

private:

	//the TRProcesser checks and achieves turn-regularity if necessary
	TRProcessor* m_trProcessor;


	// options

	int m_heuristicsNr;

	int m_cageCost,         //cost for cage edges - can be fixed in UML Class Diagrams
        m_dissectCost,      //costs for dissection edges
        m_dissectDir; //1=horizontal, 2=vertical, 3=random
    bool m_splitnear;
	OrthoDir m_firstDir; //direction of "first" edge

};


} // end namespace ogdf


#endif
