/*
 * $Revision: 1.8 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2007/04/16 15:02:09 $ 
 ***************************************************************/
 
/** \file
 * \brief declaration of class FixedEmbeddingInserter
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/
 
#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_FIXED_EMBEDDING_INSERTER_H
#define OGDF_FIXED_EMBEDDING_INSERTER_H



#include <ogdf/module/EdgeInsertionModule.h>
#include <ogdf/basic/CombinatorialEmbedding.h>
#include <ogdf/basic/FaceArray.h>


template <class E> class FaceArray;


namespace ogdf {


class FaceSetSimple;


//! Edge insertion module that inserts each edge optimally into a given embedding.
class FixedEmbeddingInserter : public EdgeInsertionModule
{
public:
	//! construction
	FixedEmbeddingInserter();
	//! destruction
	virtual ~FixedEmbeddingInserter() { }

	//
	// options
	//

	//! sets remove-reinsert option (postprocessing)
	/** possible values: (see EdgeInsertionModule)
	 * rrNone, rrInserted, rrMostCrossed, rrAll
	 */
	void removeReinsert(RemoveReinsertType rrOption) {
		m_rrOption = rrOption;
	}

	//! returns remove-reinsert option
	RemoveReinsertType removeReinsert() const {
		return m_rrOption;
	}


	//! sets the portion of most crossed edges used if remove-reinsert option is set to rrMostCrossed
	/** this portion no. of edges * percentMostCrossed() / 100
	 */
	void percentMostCrossed(double percent) {
		m_percentMostCrossed = percent;
	}

	//! returns option percentMostCrossed
	double percentMostCrossed() const {
		return m_percentMostCrossed;
	}
	
	//! returns the number of runs performed by the postprocessing after algoithm has been called
	int runsPostprocessing() const {
		return m_runsPostprocessing;
	}


private:
	//! performs actual call
	ReturnType doCall(
		PlanRep &PG,                  // planarized representation
		const List<edge> &origEdges,     // original edge to be inserted
		bool forbidCrossinGens,          // frobid crossings between gen's
		const EdgeArray<int> *costOrig,  // pointer to array of cost of
				// original edges; if pointer is 0 all costs are 1
		const EdgeArray<bool> *forbiddenEdgeOrig = 0,
		const EdgeArray<unsigned int> *edgeSubGraph = 0); // pointer to array deciding
				// which original edges are forbidden to cross; if pointer is
				// is 0 no edges are explicitly forbidden to cross

	//! construct dual graph
	/** marks dual edges corresponding to generalization in m_primalIsGen
	 * assumes that m_pDual, m_primalAdj and m_pNodeOf are already constructed
	 */
	void constructDualForbidCrossingGens(const PlanRepUML &PG,
		const CombinatorialEmbedding &E);

	//! construct dual graph
	/** assumes that m_pDual, m_primalAdj and m_pNodeOf are already constructed
	 */
	void constructDual(const GraphCopy &GC,
		const CombinatorialEmbedding &E,
		const EdgeArray<bool> *forbiddenEdgeOrig);

	//! finds a shortest path in the dual graph augmented by s and t 
	/** 
	 * ([s and t] are represented by m_vS and m_vT); 
	 * returns list of crossed adjacency entries (corresponding 
	 * to used edges in the dual) in crossed.
	 */
	void findShortestPath(const CombinatorialEmbedding &E,
		node s,
		node t,
		Graph::EdgeType eType,
		SList<adjEntry> &crossed);

	//! finds a weighted shortest path in the dual graph augmented by s and t
	/**
	 * ([s and t] are represented by m_vS and m_vT) ... using edges weights 
	 * given by costOrig;
	 * returns list of crossed adjacency entries (corresponding to used 
	 * edges in the dual) in crossed.
	 */
	void findShortestPath(
		const PlanRep &PG,
		const CombinatorialEmbedding &E,
		const EdgeArray<int> &costOrig,
		node s,
		node t,
		Graph::EdgeType eType,
		SList<adjEntry> &crossed,
		const EdgeArray<unsigned int> *edgeSubGraph,
		int eSubGraph);

	void insertEdge(PlanRep &PG,
		CombinatorialEmbedding &E,
		edge eOrig,
		const SList<adjEntry> &crossed,
		bool forbidCrossingGens,
		const EdgeArray<bool> *forbiddenEdgeOrig);

	void removeEdge(
		PlanRep &PG,
		CombinatorialEmbedding &E,
		edge eOrig,
		bool forbidCrossingGens,
		const EdgeArray<bool> *forbiddenEdgeOrig);

	edge crossedEdge(adjEntry adj) const;
	int costCrossed(edge eOrig,
		const PlanRep &PG,
		const EdgeArray<int> &costOrig,
		const EdgeArray<unsigned int> *subgraphs) const;

	Graph m_dual;   //!< (extended) dual graph, constructed/destructed during call

	EdgeArray<adjEntry> m_primalAdj;   //!< adjacency entry in primal graph corresponding to edge in dual
	FaceArray<node>     m_nodeOf;      //!< node in dual corresponding to face in primal
	EdgeArray<bool>     m_primalIsGen; //!< true iff corresponding primal edge is a generalization
	FaceSetSimple       *m_delFaces;
	FaceSetPure         *m_newFaces;

	node m_vS, m_vT;  //!< nodes in extended dual representing s and t


	// options
	RemoveReinsertType m_rrOption;
	double m_percentMostCrossed;

	// results
	int m_runsPostprocessing;
};

} // end namespace ogdf

#endif
