/*
 * $Revision: 1.6 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/21 09:18:24 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of class UMLPlanarizationLayout.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_UML_PLANARIZATION_LAYOUT_H
#define OGDF_UML_PLANARIZATION_LAYOUT_H



#include <ogdf/module/UMLLayoutModule.h>
#include <ogdf/module/PlanarSubgraphModule.h>
#include <ogdf/module/EdgeInsertionModule.h>
#include <ogdf/module/UMLPlanarLayoutModule.h>
#include <ogdf/module/CCLayoutPackModule.h>
#include <ogdf/basic/ModuleOption.h>
#include <ogdf/module/EmbedderModule.h>



namespace ogdf {


	//---------------------------------------------------------
	// UMLPlanarizationLayout
	// planarization approach for drawing UML diagrams
	//---------------------------------------------------------
	class UMLPlanarizationLayout : public UMLLayoutModule
	{
	public:
		// construction / destruction
		UMLPlanarizationLayout();
		virtual ~UMLPlanarizationLayout() { }

		//sets the default parameter setting (used e.g. in the constructor)
		void setDefaultSettings();

		// call UML layout algorithm. Input graph is given by umlGraph, layout
		// information are in attributes of umlGraph
		// must be overridden by derived classes
		virtual void call(UMLGraph &umlGraph);
		//simple call function that does not care about UML semantics, cliques etc.
		virtual void simpleCall(UMLGraph &umlGraph);

		// call for simultaneous drawing
		virtual void callSimDraw(UMLGraph &umlGraph);

		//call algorithm and fix the embedding information from the input
		//instead of new computation
		virtual void callFixEmbed(UMLGraph &umlGraph);

		//call with information about objects that should be
		//fixed as much as possible in the old/new drawing
		//for incremental drawing: takes a fixed part of the input
		//graph (indicated by fixedNodes(Edges)==true), embeds it using
		//the input layout, then inserts the remaining part into this embedding
		virtual void callIncremental(UMLGraph &umlgraph,
			NodeArray<bool> &fixedNodes, const EdgeArray<bool> &fixedEdges);

		// returns current page ratio (= desired width / height of layout)
		double pageRatio() const {
			return m_pageRatio;
		}

		// sets page ratio
		void pageRatio(double ratio) {
			m_pageRatio = ratio;
		}

		// module option for the computation of the planar subgraph
		void setSubgraph(PlanarSubgraphModule *pSubgraph) {
			m_subgraph.set(pSubgraph);
		}

		// module option for the edge insertion module
		void setInserter(EdgeInsertionModule *pInserter) {
			m_inserter.set(pInserter);
		}

		// module option for the planar layout algorithm
		void setPlanarLayouter(UMLPlanarLayoutModule *pPlanarLayouter) {
			m_planarLayouter.set(pPlanarLayouter);
		}

		// module option for the arrangement of connected components
		void setPacker(CCLayoutPackModule *pPacker) {
			m_packer.set(pPacker);
		}

		// module option for embedding the graph in simple call
		void setEmbedder(EmbedderModule *pEmbedder) {
			m_embedder.set(pEmbedder);
		}

		// returns number of crossings in layout produced in last call
		int numberOfCrossings() const {
			return m_nCrossings;
		}

		//exit if precondition violated
		void assureDrawability(UMLGraph& PG);
		//return if no precondition violated
		bool checkDrawability(UMLGraph& PG);

		//----------------------------------------------------------------------------
		//global parameters

		//draw hierarchy nodes corresponding to their level
		void alignSons(bool b) 
		{
			int opts = m_planarLayouter.get().getOptions();

			if (b) m_planarLayouter.get().setOptions(opts | umlOpAlign); 
			else  m_planarLayouter.get().setOptions(opts & ~umlOpAlign);
		}

		//set the option field for the orthogonal layouter
		void setLayouterOptions(int ops)
		{m_planarLayouter.get().setOptions(ops);}

		//search for cliques/dense subgraphs and apply
		//special treatment to avoid long running time
		//and ugly layout
		void preProcessCliques(bool b) {m_processCliques = b;}

		//insert generalization mergers where possible, reinterpret
		//generalizations as associations otherwise, for incremental layout
		void prepareIncrementalMergers(UMLGraph &uml);

		//set minimal size of cliques to search for, doesn't make sense for i < 3
		void setCliqueSize(int i) { m_cliqueSize = max(i, 3); }

	protected:
		//sorts the additional nodes for piecewise insertion
		void sortIncrementalNodes(List<node> &addNodes, const NodeArray<bool> &fixedNodes);
		void getFixationDistance(node startNode, HashingArray<int, int> &distance,
			const NodeArray<bool> &fixedNodes);
		//reembeds already planarized PG in case of errors
		void reembed(PlanRepUML &PG, int ccNumber, bool l_align = false, 
			bool l_gensExist = false);

		virtual void preProcess(UMLGraph &UG);
		virtual void postProcess(UMLGraph& UG); //redo changes at original

		//collect and store nodes around center in correct order
		void fillAdjNodes(List<node>& adjNodes, PlanRepUML& PG, node centerNode, 
			NodeArray<bool>& isClique, Layout& drawing);

	private:
		// options
		ModuleOption<PlanarSubgraphModule>  m_subgraph;
		ModuleOption<EdgeInsertionModule>   m_inserter;
		ModuleOption<UMLPlanarLayoutModule> m_planarLayouter;
		ModuleOption<CCLayoutPackModule>    m_packer;
		ModuleOption<EmbedderModule>        m_embedder;

		double m_pageRatio;

		// information
		int m_nCrossings;

		//option: rearrange labels
		bool m_arrangeLabels;

		//option: preprocess cliques, not applicable if uml layout
		bool m_processCliques;
		int m_cliqueSize; //minimum size of cliques to search for

		//temporary changes to avoid errors
		List<edge> m_fakedGens; //made to associations
		bool m_fakeTree;

		face findBestExternalFace(
			const PlanRep &PG,
			const CombinatorialEmbedding &E);
	};


	//--------------------------------------------------------
	//incremental part

	//simply sorts by decreasing int values
	class AddNodeComparer : public Comparer<node>
	{
		HashingArray<int, int> *m_indToDeg;
	public:
		AddNodeComparer(HashingArray<int, int> &ha) : m_indToDeg(&ha)
		{}
		int compare(const node &v1, const node &v2)
		{
			if ((*m_indToDeg)[v1->index()] < (*m_indToDeg)[v2->index()]) return 1;
			else if ((*m_indToDeg)[v1->index()] > (*m_indToDeg)[v2->index()]) return -1;
			else return 0;
		}//compare
	};//AddNodeComparer


} // end namespace ogdf


#endif
