/*
 * $Revision: 1.8 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2007/04/16 15:02:09 $ 
 ***************************************************************/

/** \file
 * \brief Declaration of class VariablEmbeddingInserter.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_VARIABLE_EMBEDDING_INSERTER_H
#define OGDF_VARIABLE_EMBEDDING_INSERTER_H


#include <ogdf/module/EdgeInsertionModule.h>
#include <ogdf/basic/CombinatorialEmbedding.h>
#include <ogdf/basic/FaceArray.h>


namespace ogdf {

class PlanarSPQRTree;
class ExpandedGraph;
class PlaneGraphCopy;


class BiconnectedComponent;


//---------------------------------------------------------
// VariableEmbeddingInserter
// edge insertion module that inserts each edge optimally
// into a given embedding.
//---------------------------------------------------------
class VariableEmbeddingInserter : public EdgeInsertionModule
{
public:
	// construction
	VariableEmbeddingInserter();
	// destruction
	virtual ~VariableEmbeddingInserter() { }

	//
	// options
	//

	// sets remove-reinsert option (postprocessing)
	// possible values: (see EdgeInsertionModule)
	//    rrNone, rrInserted, rrMostCrossed, rrAll
	void removeReinsert(RemoveReinsertType rrOption) {
		m_rrOption = rrOption;
	}

	// returns remove-reinsert option
	RemoveReinsertType removeReinsert() const {
		return m_rrOption;
	}


	// sets the portion of most crossed edges used if remove-reinsert option
	// is set to rrMostCrossed
	// this portion no. of edges * percentMostCrossed() / 100
	void percentMostCrossed(double percent) {
		m_percentMostCrossed = percent;
	}

	// returns option percentMostCrossed
	double percentMostCrossed() const {
		return m_percentMostCrossed;
	}


	// returns the number of runs performed by the postprocessing
	// after algoithm has been called
	int runsPostprocessing() const {
		return m_runsPostprocessing;
	}

private:
	// performs actual call
	ReturnType doCall(
		PlanRep &PG,                  // planarized representation
		const List<edge> &origEdges,     // original edge to be inserted
		bool forbidCrossingGens,         // frobid crossings between gen's
		const EdgeArray<int> *costOrig,  // pointer to array of cost of
	                          // original edges; if pointer is0 all costs are 1
		const EdgeArray<bool> *forbiddenEdgeOrig, // pointer to array deciding
				// which original edges are forbidden to cross; if pointer is
				// is 0 no edges are explicitly forbidden to cross
		const EdgeArray<unsigned int> *edgeSubGraph); // pointer to array of SubGraphInformation
	// used in Simultaneous Drawing

	edge crossedEdge(adjEntry adj) const;
	int costCrossed(edge eOrig) const;
		
	bool                   m_forbidCrossingGens;
	const EdgeArray<int>  *m_costOrig;
	const EdgeArray<bool> *m_forbiddenEdgeOrig;
	const EdgeArray<unsigned int>  *m_edgeSubgraph;
	Graph::EdgeType        m_typeOfCurrentEdge;

	void insert(node s, node t, SList<adjEntry> &eip);
	void blockInsert(const BiconnectedComponent &G,
		node s,
		node t,
		List<adjEntry> &L);

	bool dfsVertex(node v, int parent);
	bool dfsComp(int i, node parent, node &repT);

	PlanRep *m_pPG;
	node   m_s, m_t;
	edge   m_st;
	SList<adjEntry> *m_pEip;

	static int m_bigM;  // used for SimDraw

	NodeArray<SList<int> > m_compV;
	Array<SList<node> >    m_nodeB;
	Array<SList<edge> >    m_edgeB;
	NodeArray<node>        m_GtoBC;

	bool pathSearch(node v, edge parent, List<edge> &path);
	void buildSubpath(node v,
		edge eIn,
		edge eOut,
		List<adjEntry> &L,
		ExpandedGraph &Exp,
		node s,
		node t);
	edge insertEdge(node v, node w, Graph &Exp,
		NodeArray<node> &GtoExp, List<node> &nodesG);

	//const Graph    *m_tree;
	node            m_v1, m_v2;


	// options
	RemoveReinsertType m_rrOption;
	double m_percentMostCrossed;

	// results
	int m_runsPostprocessing;
};

} // end namespace ogdf

#endif
