/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/12 16:02:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of linear time layout algorithm for free
 *        trees (class RadialTreeLayout).
 *
 * Based on chapter 3.1.1 Radial Drawings of Graph Drawing by
 * Di Battista, Eades, Tamassia, Tollis.
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_RADIAL_TREE_LAYOUT_H
#define OGDF_RADIAL_TREE_LAYOUT_H

#include <ogdf/module/LayoutModule.h>
#include <ogdf/basic/SList.h>


namespace ogdf {

//---------------------------------------------------------
// RadialTreeLayout
//
// Linear time layout algorithm for radial drawings of free trees.
//	
//---------------------------------------------------------

class RadialTreeLayout : public LayoutModule
{
public:
	enum RootSelectionType { rootIsSource, rootIsSink, rootIsCenter };

private:
    double m_levelDistance;          // minimal distance between levels
	double m_connectedComponentDistance; // minimal distance between trees

	RootSelectionType m_selectRoot;  // how to determine the root

	node            m_root;          // root of the tree

	int             m_numLevels;     // number of levels (root is on level 0)
	NodeArray<int>  m_level;         // level of node
	NodeArray<node> m_parent;        // parent node, 0 if root
	NodeArray<double> m_leaves;      // weighted number of leaves in subtree
	Array<SListPure<node> > m_nodes; // nodes at level

	NodeArray<double> m_angle;       // angle of node center (for placement)
	NodeArray<double> m_wedge;       // wedge reserved for subtree

	NodeArray<double> m_diameter;    // diameter of a circle bounding a node
	Array<double>     m_width;       // width of a circle

	Array<double>   m_radius;        // width of level
	double          m_outerRadius;   // radius of circle bounding the drawing
	
	struct Group
	{
		RadialTreeLayout *m_data;

		bool            m_leafGroup;
		SListPure<node> m_nodes;
		double          m_sumD;
		double          m_sumW;
		double          m_leftAdd;
		double          m_rightAdd;

		Group(RadialTreeLayout *data, node v) {
			m_data = data;
			m_leafGroup = (v->degree() == 1);
			m_nodes.pushBack(v);
			m_sumD = m_data->diameter()[v] + m_data->levelDistance();
			m_sumW = m_data->leaves()[v];
			m_leftAdd = m_rightAdd = 0.0;
		}

		bool isSameType(node v) const {
			return (m_leafGroup == (v->degree() == 1));
		}

		void append(node v) {
			m_nodes.pushBack(v);
			m_sumD += m_data->diameter()[v] + m_data->levelDistance();
			m_sumW += m_data->leaves()[v];
		}

		double add() const { return m_leftAdd + m_rightAdd; }
		node leftVertex () const { return m_nodes.front(); }
		node rightVertex() const { return m_nodes.back (); }
	};

	class Grouping : public List<Group>
	{
	public:
		void computeAdd(double &D, double &W);
	};

	NodeArray<Grouping> m_grouping;

public:
	// constructor: sets options to default values
	RadialTreeLayout();

	// copy constructor
	RadialTreeLayout(const RadialTreeLayout &tl);

	// destructor
	~RadialTreeLayout();

	// assignment operator
	RadialTreeLayout &operator=(const RadialTreeLayout &tl);

	// calls the algorithm for attributed graph AG
	// returns layout information in AG
	//
	// precondition: the graph is a tree
	//
	// the order of children is given by the adjacency lists;
	void call(GraphAttributes &AG);


	// option that determines the minimal vertical distance 
	// required between levels
	double levelDistance() const { return m_levelDistance; }
	void levelDistance(double x) { m_levelDistance = x; }

	// option that determines the minimal horizontal distance 
	// required between trees in the forest
	double connectedComponentDistance() const { return m_connectedComponentDistance; }
	void connectedComponentDistance(double x) { m_connectedComponentDistance = x; }

	// option that determines if the root is on the top or on the bottom
	RootSelectionType rootSelection() const { return m_selectRoot; }
	void rootSelection(RootSelectionType x) { m_selectRoot = x; }

	const NodeArray<double> &diameter() const { return m_diameter; }
	const NodeArray<double> &leaves()   const { return m_leaves;   }

private:
	void FindRoot(const Graph &G);
	void ComputeLevels(const Graph &G);
	void ComputeDiameters(GraphAttributes &AG);
	void ComputeAngles(const Graph &G);
	void ComputeCoordinates(GraphAttributes &AG);
	void ComputeGrouping(int i);

	OGDF_NEW_DELETE
};

} // end namespace ogdf

#endif

