/*
 * $Revision: 1.5 $
 * 
 * last checkin:
 *   $Author: gutweng $ 
 *   $Date: 2007/03/14 13:54:40 $ 
 ***************************************************************/
 
/** \file
 * \brief Declaration of linear time layout algorithm for trees
 *        (TreeLayout) based on Walker's algorithm.
 * 
 * \author Christoph Buchheim
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#ifdef _MSC_VER
#pragma once
#endif

#ifndef OGDF_TREE_LAYOUT_H
#define OGDF_TREE_LAYOUT_H

#include <ogdf/module/LayoutModule.h>
#include <ogdf/basic/SList.h>

namespace ogdf {

//---------------------------------------------------------
// TreeLayout
//
// Linear time layout algorithm for trees. See
// - J. Q. Walker II: "A Node-Positioning Algorithm for
//   General Trees", Software-Practice and Experience,
//   vol. 20(7), 685-705, 1990
// - C. Buchheim, M. Juenger, S. Leipert: tba.
//	
//---------------------------------------------------------

class TreeLayout : public LayoutModule {
public:
	enum RootSelectionType { rootIsSource, rootIsSink, rootByCoord };

private:
	double m_siblingDistance;        // minimal distance between siblings
	double m_subtreeDistance;        // minimal distance between subtrees
    double m_levelDistance;          // minimal distance between levels
	double m_treeDistance;           // minimal distance between trees

	bool m_orthogonalLayout;         // orthogonal layout required?
	Orientation m_orientation;       // orientation of tree layout
	RootSelectionType m_selectRoot;  // how to determine the root

	NodeArray<int> m_number;         // consecutive numbers for children 

	NodeArray<node> m_parent;        // parent node, 0 if root
	NodeArray<node> m_leftSibling;   // left sibling, 0 if none
	NodeArray<node> m_firstChild;    // leftmost child, 0 if leaf
	NodeArray<node> m_lastChild;	 // rightmost child, 0 if leaf
	NodeArray<node> m_thread;        // thread, 0 if none
	NodeArray<node> m_ancestor;      // actual highest ancestor

	NodeArray<double> m_preliminary; // preliminary x-coordinate
	NodeArray<double> m_modifier;    // modifier of x-coordinate
	NodeArray<double> m_change;      // change of shift applied to subtrees
	NodeArray<double> m_shift;       // shift applied to subtrees

	SListPure<edge> m_reversedEdges;  // list of temporarily removed edges
	Graph           *m_pGraph;
public:
	// constructor: sets options to default values
	TreeLayout();

	// copy constructor
	TreeLayout(const TreeLayout &tl);

	// destructor
	~TreeLayout();

	// assignment operator
	TreeLayout &operator=(const TreeLayout &tl);

	// calls the algorithm for attributed graph AG
	// returns layout information in AG
	//
	// precondition: the graph is a tree
	//
	// the order of children is given by the adjacency lists;
	// the successor of the unique in-edge of a non-root node
	// leads to its leftmost child; the leftmost child of the root
	// is given by its first adjacency entry
	void call(GraphAttributes &AG);

	// calls the algorithm for attributed graph AG
	// sorts the adjacency entries according to the positions of adjacent
	// vertices in AG
	// G is the graph associated with AG
	void callSortByPositions(GraphAttributes &AG, Graph &tree);


	// option that determines the minimal horizontal distance 
	// required between siblings
	const double siblingDistance() { return m_siblingDistance; }
	void siblingDistance(double x) { m_siblingDistance = x; }

	// option that determines the minimal horizontal distance 
	// required between subtrees
	double subtreeDistance() const { return m_subtreeDistance; }
	void subtreeDistance(double x) { m_subtreeDistance = x; }

	// option that determines the minimal vertical distance 
	// required between levels
	double levelDistance() const { return m_levelDistance; }
	void levelDistance(double x) { m_levelDistance = x; }

	// option that determines the minimal horizontal distance 
	// required between trees in the forest
	double treeDistance() const { return m_treeDistance; }
	void treeDistance(double x) { m_treeDistance = x; }

	// option that determines whether orthogonal layout 
	// is required
	bool orthogonalLayout() const { return m_orthogonalLayout; }
	void orthogonalLayout(bool x) { m_orthogonalLayout = x; }

	// option that determines if the root is on the top or on the bottom
	Orientation orientation() const { return m_orientation; }
	void orientation(Orientation x) { m_orientation = x; }

	// option that determines if the root is on the top or on the bottom
	RootSelectionType rootSelection() const { return m_selectRoot; }
	void rootSelection(RootSelectionType x) { m_selectRoot = x; }

private:
	class AdjComparer;

	void adjustEdgeDirections(Graph &G, node v, node parent);
	void setRoot(GraphAttributes &AG, Graph &tree);
	void undoReverseEdges(GraphAttributes &AG);

	// initialize all node arrays and
	// compute the tree structure from the adjacency lists
	//
	// returns the root node
	void initializeTreeStructure(const Graph &tree, List<node> &roots);

	// delete all node arrays
	void deleteTreeStructure();

	// returns whether node v is a leaf
	int isLeaf(node v) const;

	// returns the successor of node v on the left/right contour
	// returns 0 if there is none
	node nextOnLeftContour(node v) const;
	node nextOnRightContour(node v) const;

	// recursive bottom up traversal of the tree for computing
	// preliminary x-coordinates
	void firstWalk(node subtree,const GraphAttributes &AG,bool upDown);

	// space out the small subtrees on the left hand side of subtree
	// defaultAncestor is used for all nodes with obsolete m_ancestor
	void apportion(node subtree,
		           node &defaultAncestor,
				   const GraphAttributes &AG,
				   bool upDown);

	// recursive top down traversal of the tree for computing final
	// x-coordinates
	void secondWalkX(node subtree,
		            double modifierSum,
					GraphAttributes &AG);
	void secondWalkY(node subtree,
		            double modifierSum,
					GraphAttributes &AG);

	// compute y-coordinates and edge shapes 
	void computeYCoordinatesAndEdgeShapes(node root,GraphAttributes &AG);
	void computeXCoordinatesAndEdgeShapes(node root,GraphAttributes &AG);

	void findMinX(GraphAttributes &AG, node root, double &minX);
	void findMinY(GraphAttributes &AG, node root, double &minY);
	void findMaxX(GraphAttributes &AG, node root, double &maxX);
	void findMaxY(GraphAttributes &AG, node root, double &maxY);
	void shiftTreeX(GraphAttributes &AG, node root, double shift);
	void shiftTreeY(GraphAttributes &AG, node root, double shift);

	OGDF_NEW_DELETE
};

} // end namespace ogdf

#endif

