/*
 * $Revision: 1.2 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 10:02:21 $ 
 ***************************************************************/
 
/** \file
 * \brief Implementation of heuristics for two-layer crossing
 * minimization (BarycenterHeuristic, MedianHeuristic)
 * 
 * \author Carsten Gutwenger
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#include <ogdf/layered/BarycenterHeuristic.h>
#include <ogdf/layered/MedianHeuristic.h>


namespace ogdf {


//---------------------------------------------------------
// BarycenterHeuristic
// implements the barycenter heuristic for 2-layer 
// crossing minimization
//---------------------------------------------------------

void BarycenterHeuristic::call (Level &L)
{
	const Hierarchy& H = L.hierarchy();

	for (int i = 0; i <= L.high(); ++i) {
		node v = L[i];
		long sumpos = 0L;
    
		const Array<node> &adjNodes = L.adjNodes(v);
		for(int j = 0; j <= adjNodes.high(); ++j)
			sumpos += H.pos(adjNodes[j]);

		m_weight[v] = (adjNodes.high() < 0) ? 0.0 :
			double(sumpos) / double(adjNodes.size());
	}
	
	L.sort(m_weight);
}


//---------------------------------------------------------
// MedianHeuristic
// implements the median heuristic for 2-layer 
// crossing minimization
//---------------------------------------------------------

void MedianHeuristic::call (Level &L)
{
	const Hierarchy& H = L.hierarchy();

	for (int i = 0; i <= L.high(); ++i) {
		node v = L[i];
    
		const Array<node> &adjNodes = L.adjNodes(v);
		const int high = adjNodes.high();

		if (high < 0) m_weight[v] = 0;
		else if (high & 1)
			m_weight[v] = H.pos(adjNodes[high/2]) + H.pos(adjNodes[1+high/2]);
		else
			m_weight[v] = 2*H.pos(adjNodes[high/2]);
	}

	L.sort(m_weight,0,2*H.adjLevel(L.index()).high());
}


} // namespace ogdf
