/*
 * $Revision: 1.4 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2006/03/02 14:40:59 $ 
 ***************************************************************/
 
/** \file
 * \brief Computes the Orthogonal Representation of a Planar
 * Representation of a UML Graph
 * 
 * \author Sebastian Leipert
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#include <ogdf/orthogonal/OrthoFormerUML.h>
#include <ogdf/basic/FaceArray.h>
#include <limits.h>
#include <ogdf/graphalg/MinCostFlowReinelt.h>


namespace ogdf {


void OrthoFormerUML::call(PlanRepUML &PG,
						  CombinatorialEmbedding &E,
						  OrthoRep &OR,
						  int startBoundBendsPerEdge)
{

	if (PG.numberOfEdges() == 0)
		return;


	OR.init(E);
	FaceArray<node> F(E);

	OGDF_ASSERT(PG.representsCombEmbedding()) 
	OGDF_ASSERT(F.valid())


	// the min-cost flow we use
	MinCostFlowReinelt flowModule;
	const int infinity = flowModule.infinity();


	//
	// VARIABLES
	//

	// NETWORK VARIABLES
	
	Graph network;
	EdgeArray<int>  lowerBound(network,0); // lower bound for flow
	EdgeArray<int>  upperBound(network,0); // upper bound for flow
	EdgeArray<int>  cost(network,0);       // cost of an edge
	NodeArray<int>  supply(network,0);     // supply of every node


	// FORWARD TRANSFORMATION

	// stores for edges of the network the corresponding adjEntries 
	// nodes, and faces of PG
	EdgeArray<adjEntry> adjCor(network,0);    
	EdgeArray<node>		nodeCor(network,0);    
	EdgeArray<face>		faceCor(network,0);    



	//BACKWARD TRANSFORMATION

	// Contains for every node of PG the corresponding node
	// in the network
	NodeArray<node>		backNodeCor(PG,0);
	// Contains for every adjEntry of PG the corresponding edge
	// in the network
	AdjEntryArray<edge>	backAdjCor(PG,0);

	

	// USEFULL TRANSFORMATION
	
	// Contains for adjacebcy Entry of PG the face it belongs to in PG
	AdjEntryArray<face>  adjF(PG,0);


	// STANDARD VARIABLES

	node v;
	adjEntry adj;
	edge e;



	// GENERATE ALL NODES OF THE NETWORK

	forall_nodes(v,PG)
	{
		if (v->degree() < 4)
			backNodeCor[v] = network.newNode();
		else if (v->degree() == 4)
			// no node generated. Set angles in orthogonal representation.
		{
			forall_adj(adj,v)
				OR.angle(adj) = 1;
		}
		else
			OGDF_ASSERT(false); // No node with degree more than 4 allowed.
	}


	face f;
	for (f = E.firstFace(); f; f = f->succ())
		F[f] = network.newNode();

	node s = network.newNode();
	node t = network.newNode();


	#ifdef OGDF_DEBUG
	if (int(ogdf::debugLevel) >= int(dlHeavyChecks)) {
		forall_nodes(v,PG) {
			cout << " v = " << v << " corresponds to " <<
			backNodeCor[v] << endl;
		}
		for (f = E.firstFace(); f; f = f->succ()) {
			cout << " face = " << f->index() << " corresponds to " << F[f];
			if (f == E.externalFace())
				cout<<" (Outer Face)";
			cout<<endl;
		}
		cout << " s = " << s << endl << " t = " << t << endl;
	}
	#endif




	// GENERATE ALL EDGES OF THE NETWORK.


	// OPTIMIZATION POTENTIAL
	// Do not insert edges with upper bound 0 into the network.

	// Locate for every adjacency entry its adjacent faces.
	for (f = E.firstFace(); f; f = f->succ())
	{
		forall_face_adj(adj,f)
			adjF[adj] = f;			
	}
	


	#ifdef OGDF_DEBUG
	if(int(ogdf::debugLevel)>=int(dlHeavyChecks)) {
		for(f = E.firstFace(); f; f = f->succ()) {
			cout << "Face " << f->index() << " : ";
			forall_face_adj(adj,f)
				cout << adj << "; ";
			cout<<endl;
		}
	}
	#endif



	// Insert for every edge the (two) network arcs
	// entering the face nodes
	forall_edges(e,PG)
	{
		OGDF_ASSERT(adjF[e->adjSource()] && adjF[e->adjTarget()]) 
		if (F[adjF[e->adjSource()]] != F[adjF[e->adjTarget()]]) 
		{
			// not a selfloop.
			edge newE = network.newEdge(F[adjF[e->adjSource()]],F[adjF[e->adjTarget()]]); 
			adjCor[newE] = e->adjSource();
			if (PG.typeOf(e) == Graph::generalization)
				upperBound[newE] = 0;
			else 
                upperBound[newE] = infinity;
			cost[newE] = 1;
			backAdjCor[e->adjSource()] = newE;

			newE = network.newEdge(F[adjF[e->adjTarget()]],F[adjF[e->adjSource()]]); 
			adjCor[newE] = e->adjTarget();
			if (PG.typeOf(e) == Graph::generalization)
				upperBound[newE] = 0;
			else
				upperBound[newE] = infinity;
			cost[newE] = 1;
			backAdjCor[e->adjTarget()] = newE;
		}
	}







	// Insert for every node v with degree < 3 an edge to 
	// every node of its adjacent faces and insert the 
	// edge (s,v);
	// Observe that two generalizations are not allowed to bend on 
	// a node. There must be a 180 degree angel between them.


	forall_nodes(v,PG)
	{
		if (v->degree() < 4)
		{
			e = network.newEdge(s,backNodeCor[v]);
			upperBound[e] = 4 - v->degree();
			supply[s] += upperBound[e];

			// Locate popssible adjacent generalizations
			adjEntry gen1 = 0;
			adjEntry gen2 = 0;

			if (PG.typeOf(v) != Graph::generalizationMerger 
				&& PG.typeOf(v) != Graph::generalizationExpander)
			{
				forall_adj(adj,v)
				{
					if (PG.typeOf(adj->theEdge()) == Graph::generalization)
					{
						if (!gen1)
							gen1 = adj;
						else
							gen2 = adj;
					}
				}
			}

			
			forall_adj(adj,v)
			{
				edge e2 = network.newEdge(backNodeCor[v],F[adjF[adj]]);
				upperBound[e2] = infinity;
				nodeCor[e2] = v;
				adjCor[e2] = adj;
				faceCor[e2] = adjF[adj];
				if (gen2 == adj && gen1 == adj->cyclicSucc())
				{
					upperBound[e2] = 1;
					lowerBound[e2] = 1;
				}
				else if (gen1 == adj && gen2 == adj->cyclicSucc())
				{
					upperBound[e2] = 1;
					lowerBound[e2] = 1;
				}
			}
		}
	}

	// Insert for every face f with size < 4 an edge (s,f) 
	// and with size > 4 an edge (f,t)
	for (f = E.firstFace(); f; f = f->succ())
	{
		if (f == E.externalFace())
		{
			e = network.newEdge(F[f],t);
			upperBound[e] = 4 + f->size();
			supply[t] += upperBound[e];
		}
		else if (f->size() < 4)
		{

			e = network.newEdge(s,F[f]);
			upperBound[e] = 4 - f->size();
			supply[s] += upperBound[e];
		}
		else if (f->size() > 4)
		{
			e = network.newEdge(F[f],t);
			lowerBound[e] = 0;
			upperBound[e] = f->size() - 4;
			supply[t] += upperBound[e];
		}
	}




	// Reset upper and lower Bounds for network arcs that
	// correspond to edges of generalizationmerger faces 
	// and edges of expanded nodes.

	forall_nodes(v,PG)
	{
		if (PG.expandAdj(v))
		{
			adj = PG.expandAdj(v);
			// Get the corresponding face in the original embedding.
			f = adjF[adj];

			if (PG.typeOf(v) == Graph::generalizationMerger)
			{
				// Set upperBound to 0  for all edges.
				forall_face_adj(adj,f)
				{
					upperBound[backAdjCor[adj]] = 0;
					upperBound[backAdjCor[adj->twin()]] = 0;

					// Node w is in network
					node w = backNodeCor[adj->twin()->theNode()];
					forall_adj_edges(e,w)
					{
						if (e->target() == F[f])
						{
							lowerBound[e] = 1;
							upperBound[e] = 1;
						}
					}
					
				}
				// Set the upper and lower bound for the first edge of
				// the mergeexpander face to guaranty a 90 degree bend.
				upperBound[backAdjCor[PG.expandAdj(v)]] = 1;
				lowerBound[backAdjCor[PG.expandAdj(v)]]= 1;


				// Set the upper and lower bound for the first node in 
				// clockwise order of the mergeexpander face to
				// guaranty a 90 degree angle at the node in the interior 
				// and a 180 degree angle between at generalization in the
				// exterior. 
				node secFace;

				if (F[f] == backAdjCor[PG.expandAdj(v)]->target())
					secFace = backAdjCor[PG.expandAdj(v)]->source();
				else if (F[f] == backAdjCor[PG.expandAdj(v)]->source())
					secFace = backAdjCor[PG.expandAdj(v)]->target();
				else 
					OGDF_ASSERT(false); // Edges in network mixed up.

				node w = backNodeCor[PG.expandAdj(v)->twin()->theNode()];

				forall_adj(adj,w)
				{
					if (adj->theEdge()->target() == F[f])
					{
						lowerBound[adj->theEdge()] = 0;
						upperBound[adj->theEdge()] = 0;
						break;
					}
				}
				e = adj->cyclicSucc()->theEdge();

				if (e->target() == secFace)
				{
					lowerBound[e] = 1;
					upperBound[e] = 1;
				}

				// Set the upper and lower bound for the last edge of
				// the mergeexpander face to guaranty a 90 degree bend.
				upperBound[backAdjCor[PG.expandAdj(v)->faceCyclePred()]] = 1;
				lowerBound[backAdjCor[PG.expandAdj(v)->faceCyclePred()]] = 1;


				// Set the upper and lower bound for the last node in 
				// clockwise order of the mergeexpander face to
				// guaranty a 90 degree angle at the node in the interior 
				// and a 180 degree angle between at generalization in the
				// exterior. 
				if (F[f] == backAdjCor[PG.expandAdj(v)->faceCyclePred()]->target())
					secFace = backAdjCor[PG.expandAdj(v)->faceCyclePred()]->source();
				else if (F[f] == backAdjCor[PG.expandAdj(v)->faceCyclePred()]->source())
					secFace = backAdjCor[PG.expandAdj(v)->faceCyclePred()]->target();
				else 
					OGDF_ASSERT(false); // Edges in network mixed up.

				w = backNodeCor[PG.expandAdj(v)->faceCyclePred()->theNode()];

				forall_adj(adj,w)
				{
					if (adj->theEdge()->target() == F[f])
					{
						lowerBound[adj->theEdge()] = 0;
						upperBound[adj->theEdge()] = 0;
						break;
					}
				}
				e = adj->cyclicPred()->theEdge();

				if (e->target() == secFace)
				{
					lowerBound[e] = 1;
					upperBound[e] = 1;
				}


			}



			else if (PG.typeOf(v) == Graph::highDegreeExpander )
			{
				// Set upperBound to 1 for all edges, allowing maximal one
				// 90 degree bend.
				// Set upperBound to 0 for the corresponding entering edge
				// allowing no 270 degree bend.
				// Set upperbound to 1 for every edge corresponding to the
				// angle of a vertex. This permitts 270 degree angles in
				// the face

				adjEntry splitter = 0;
				forall_face_adj(adj,f)
				{
                    //should be: no 270 degrees
					upperBound[backAdjCor[adj->twin()]] = 0;
					if (PG.faceSplitter(adj->theEdge()))
					{
						// No bends allowed  on the face splitter
						upperBound[backAdjCor[adj]] = 0;
						splitter = adj;
						continue;
					}

                    else {
                    //should be: only one bend
	                    if (m_distributeEdges) 
		                {
			              //check the special case degree 4 with 2 generalizations following each other
				          if (backNodeCor[adj->twin()->theNode()])  // (v->degree() > 4)
					          upperBound[backAdjCor[adj]] = 1;
						}
						
						//align multi edges on the same side
	                    if (m_multiAlign)
                        {
							adjEntry srcadj = adj->cyclicPred();
							adjEntry tgtadj = adj->twin()->cyclicSucc();

							//check if the nodes are expanded
							node vt1, vt2;
							if (PG.expandedNode(srcadj->twin()->theNode())) 
								vt1 = PG.expandedNode(srcadj->twin()->theNode());
							else
								vt1 = srcadj->twin()->theNode();

							if (PG.expandedNode(tgtadj->twin()->theNode())) 
								vt2 = PG.expandedNode(tgtadj->twin()->theNode());
							else
								vt2 = tgtadj->twin()->theNode();

							if (vt1 == vt2) 
                            {
                                lowerBound[backAdjCor[adj]] =
									upperBound[backAdjCor[adj]] = 0;
                            } // multi edge
                        }
                    } // else no face splitter

					// Node w is in network
					node w = backNodeCor[adj->twin()->theNode()];

					if (w)  //otherwise the corrsponding node is of degree 4
							// and thus w is nil in the network.	
					{
                        //should be: inner face angles set to 180
						forall_adj_edges(e,w)
						{
							if (e->target() == F[f])
							{
								upperBound[e] = 1;//sl: 1
								lowerBound[e] = 1;
							}
						}
					}
				}


				// In case a face splitter was used, we need to update the 
				// second face of the cage.
				if (splitter)
				{

					adj = splitter->twin();
					// Get the corresponding face in the original embedding.
					face f2 = adjF[adj];

					forall_face_adj(adj,f2)
					{
						if (adj == splitter->twin())
							continue;

						upperBound[backAdjCor[adj->twin()]] = 0;

						// Node w is in network
						node w = backNodeCor[adj->twin()->theNode()];
						if (w)  //otherwise the corrsponding node is of degree 4
								// and thus w is nil in the network.	
						{
							forall_adj_edges(e,w)
							{
								if (e->target() == F[f2])
								{
									upperBound[e] = 1;
									lowerBound[e] = 1;
								}
							}
						}
					}
				}
			}
		}
	}




	bool isFlow = false;
	SList<edge> capacityBoundedEdges;
	EdgeArray<int> flow(network,0);
	// Set upper bound depending on startboundbendsperedge if given. 
	// Otherwise to infinity. Do not set it to INT_MAX. 
	// This causes problems with MinCostFlowReinelt.
	forall_edges(e,network)
		if (upperBound[e] == infinity)
			capacityBoundedEdges.pushBack(e);

		
	OGDF_ASSERT(supply[t] == supply[s])
	supply[t] = - supply[t];



	int currentUpperBound;
	if (startBoundBendsPerEdge > 0)	
		currentUpperBound = startBoundBendsPerEdge;
	else
		currentUpperBound = supply[s];


	while (!isFlow)
	{

		SListIterator<edge> it;
		for (it = capacityBoundedEdges.begin(); it.valid(); it++)
			upperBound[(*it)] = currentUpperBound;



		#ifdef OGDF_DEBUG
		if (int(ogdf::debugLevel) >= int(dlHeavyChecks)) {
			forall_edges(e,network)
				cout << "e = " << e << " lower = "
					<< lowerBound[e] << " upper = " << upperBound[e]
					<< " cost = " << cost[e] << endl;
			forall_nodes(v,network)
				cout << "v " << v << " supply = " << supply[v] << endl;
		}
		#endif



		isFlow = flowModule.call(network,lowerBound,upperBound,cost,supply,flow);

		#ifdef OGDF_DEBUG
		if (int(ogdf::debugLevel) >= int(dlHeavyChecks)) {
			forall_edges(e,network) {
				cout << "e = " << e << " flow = " << flow[e];
				if(nodeCor[e] == 0 && adjCor[e])
					cout << " real edge = " << adjCor[e]->theEdge();
				cout << endl;
			}
			forall_edges(e,network) {
				if(nodeCor[e] == 0 && adjCor[e] != 0 && flow[e] > 0) {
				cout << "Bends " << flow[e] << " on edge "
					<< adjCor[e]->theEdge()
					<< " between faces " << adjF[adjCor[e]]->index() << " - "
					<< adjF[adjCor[e]->twin()]->index() << endl;
				}
			}
			forall_edges(e,network) {
				if(nodeCor[e] !=0 && faceCor[e] != 0) {
				cout << "Angle " << (flow[e]+1)*90 << "\tdegree   on node "
					<< nodeCor[e] << " at face " << faceCor[e]->index()
					<< "\tbetween edge " << adjCor[e]->faceCyclePred()
					<< "\tand " << adjCor[e] << endl;
				}
			}
		}
		if (startBoundBendsPerEdge> 0) {
			cout << "Minimizing edge bends for upper bound " << currentUpperBound;
			if(isFlow)
				cout << " ... Successful";
			cout<<endl;
		}
		#endif

		OGDF_ASSERT(startBoundBendsPerEdge >= 1 || isFlow);


		currentUpperBound++;

	}// while (!isflow)



	int totalNumBends = 0;

	forall_edges(e,network)
	{
		if (nodeCor[e] == 0 && adjCor[e] != 0 && flow[e] > 0)
		{
			OR.bend(adjCor[e]).set('0',flow[e]);
			OR.bend(adjCor[e]->twin()).set('1',flow[e]);
			totalNumBends += flow[e];
		}
		else if (nodeCor[e] != 0 && faceCor[e] != 0)
			OR.angle(adjCor[e]) = (flow[e]+1);
	}


	#ifdef OGDF_DEBUG
	if (int(ogdf::debugLevel) >= int(dlHeavyChecks))
		cout << endl << endl << "Total Number of Bends : "
			<< totalNumBends << endl << endl;
	#endif


	String error;
	OGDF_ASSERT(OR.check(error))


}


} // end namespace ogdf

