/*
 * $Revision: 1.3 $
 * 
 * last checkin:
 *   $Author: chimani $ 
 *   $Date: 2005/11/15 10:02:26 $ 
 ***************************************************************/
 
/** \file
 * \brief implementation of class ClusterPlanarizationLayout 
 * applies planarization approach for drawing Cluster diagrams
 * 
 * \author Karsten Klein
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/


#include <ogdf/cluster/ClusterPlanarizationLayout.h>
#include <ogdf/cluster/CPlanarSubClusteredGraph.h>

#include <ogdf/planarity/MaximalPlanarSubgraphSimple.h>
#include <ogdf/planarity/FixedEmbeddingInserter.h>
#include <ogdf/packing/TileToRowsCCPacker.h>
#include <ogdf/planarity/PlanarModule.h>

#include <ogdf/basic/HashingArray.h>
//#include <ogdf/basic/precondition.h>

#include <ogdf/cluster/CPlanarEdgeInserter.h>
#include <ogdf/basic/extended_graph_alg.h>

//#define foutput

namespace ogdf {


ClusterPlanarizationLayout::ClusterPlanarizationLayout()
{
	m_pageRatio = 1.0;

	m_subgraph.set(new MaximalPlanarSubgraphSimple);
	m_inserter.set(new FixedEmbeddingInserter);
	m_planarLayouter.set(new ClusterOrthoLayout);
	m_packer.set(new TileToRowsCCPacker);

    m_syncTrees = false; //draw hierarchies in same direction?

    m_arrangeLabels = false;
    
    m_fakeTree = true; //change edge type from gen to ass if non-tree hierarchy
}//constructor


//the call function that lets ClusterPlanarizationLayout compute a layout 
//for the input
//void ClusterPlanarizationLayout::call(Graph& G, UMLGraph& umlGraph, ClusterGraph& cGraph)
void ClusterPlanarizationLayout::call(Graph& G, 
									  ClusterGraphAttributes& acGraph, 
									  ClusterGraph& cGraph,
									  bool simpleCConnect) //default true
{
	m_nCrossings = 0;

	bool subGraph = false; // c-planar subgraph computed?


	//check some simple cases 
	if (G.numberOfNodes() == 0) return;

//*************************************************************
//we set pointers and arrays to the working graph, which can be
//the original or, in the case of non-c-planar input, a copy

	Graph* workGraph = &G;
	ClusterGraph* workCG = &cGraph;
	ClusterGraphAttributes* workACG = & acGraph;

	//potential copy of original if non c-planar
	Graph GW;
	//list of non c-planarity causing edges
	List<edge> leftEdges;

	//list of nodepairs to be connected (deleted edges)
	List<NodePair> leftWNodes;

	//store some information
	//original to copy
	NodeArray<node> resultNode(G);
	EdgeArray<edge> resultEdge(G);
	ClusterArray<cluster> resultCluster(cGraph);
	//copy to original
	NodeArray<node> orNode(G);
	EdgeArray<edge> orEdge(G);
	ClusterArray<cluster> orCluster(cGraph);

	//cluster IDs may differ after copying
	ClusterArray<int> originalClId(cGraph);
	//maybe clusterof(ornode(v)) will do

	node workv;
	forall_nodes(workv, G)
	{
		resultNode[workv] = workv; //will be set to copy if non-c-planar
		orNode[workv] = workv;
	}
	edge worke;
	forall_edges(worke, G)
	{
		resultEdge[worke] = worke; //will be set to copy if non-c-planar
		orEdge[worke] = worke;
	}
	cluster workc;
	forall_clusters(workc, cGraph)
	{
		resultCluster[workc] = workc; //will be set to copy if non-c-planar
		orCluster[workc] = workc;
		originalClId[workc] = workc->index();
	}

    //check necessary preconditions
	//as long as we don't  have separate cluster structures for the connected
	//components, we make the cluster graph temporarily connected
    assureDrawability(acGraph); //TODO!!


	//***********************************************
	//check if instance is clusterplanar and embed it 
	CconnectClusterPlanarEmbed CCPE; //cccp

	bool cplanar = CCPE.embed(cGraph, G);

	List<edge> connectEdges;

#ifdef foutput
	cGraph.writeGraphWinGML("Boundaries.gml");
#endif

	//if the graph is not c-planar, we have to check the reason and to 
	//correct the problem by planarising or inserting connection edges
	if (!cplanar) 
	{
		#ifdef foutput
			cout<<"Graph is not c-planar\n";
		#endif

		bool connect = false;
		
		if ( (CCPE.errCode() == CconnectClusterPlanarEmbed::nonConnected) ||
				(CCPE.errCode() == CconnectClusterPlanarEmbed::nonCConnected) )
		{
			#ifdef foutput
				cout << "because Graph is not c-connected\n";
			#endif

			//we insert edge to make the input c-connected
			makeCConnected(cGraph, G, connectEdges, simpleCConnect);
			
			//save edgearray info for inserted edges
			ListConstIterator<edge> itE = connectEdges.begin();
			while (itE.valid())
			{
				resultEdge[*itE] = (*itE);
				orEdge[*itE]     = (*itE);
				itE++;
			}//while
			
			connect = true;

			#ifdef foutput
				cGraph.writeGraphWinGML("C:\\connected.gml");
			#endif
			
			CCPE.embed(cGraph, G);

			if ( (CCPE.errCode() == CconnectClusterPlanarEmbed::nonConnected) ||
				(CCPE.errCode() == CconnectClusterPlanarEmbed::nonCConnected) )
			{
				cerr << "no correct connection made\n"<<flush;
				THROW(AlgorithmFailureException);
			}
		}//if not cconnected
		if ( (CCPE.errCode() == CconnectClusterPlanarEmbed::nonPlanar) ||
				(CCPE.errCode() == CconnectClusterPlanarEmbed::nonCPlanar) )
		{
			#ifdef foutput
				cout<<"Graph is cconnected, still "<< 
					((CCPE.errCode() == CconnectClusterPlanarEmbed::nonPlanar) ?
					"non-planar\n" : "non-cplanar\n" ) <<flush;
			#endif

			subGraph = true;
			EdgeArray<bool> inSubGraph(G, false);
			
			CPlanarSubClusteredGraph cps;
			cps.call(cGraph, inSubGraph, leftEdges);
#ifdef foutput
			cout << "  Anzahl Restkanten: " << leftEdges.size() << "\n" <<flush;
#endif

			//**************************************************************
			//now we delete the copies of all edges not in subgraph and embed
			//the subgraph (use a new copy)

			//construct copy
			
			//ClusterGraph CGW(cGraph, GW, resultCluster, resultNode, resultEdge);

			workGraph = &GW;
			workCG = new ClusterGraph(cGraph, GW, resultCluster, resultNode, resultEdge);
				//&CGW;
			
			//****************************
			//reinit original arrays
			orNode.init(GW,0);
			orEdge.init(GW,0);
			orCluster.init(*workCG,0);
			originalClId.init(*workCG, -1);

			//set array entries to the appropriate values
			forall_nodes(workv, G)
			{
				orNode[resultNode[workv]] = workv; 
			}
			forall_edges(worke, G)
			{
				orEdge[resultEdge[worke]] = worke;
			}
			forall_clusters(workc, cGraph)
			{
				orCluster[resultCluster[workc]] = workc; 
				originalClId[resultCluster[workc]] = workc->index();
			}

			//****************************
			//create new ACG and copy values (width, height, type)
			
			workACG = new ClusterGraphAttributes(*workCG, workACG->attributes());
			forall_nodes(workv, GW)
			{
				//should set sam attributes in construction!!!
				if (acGraph.attributes() & GraphAttributes::nodeType)
					workACG->type(workv)	= acGraph.type(orNode[workv]);
				workACG->height(workv)	= acGraph.height(orNode[workv]);
				workACG->width(workv)	= acGraph.width(orNode[workv]);
				//not necessary
				//workACG->x(workv) = acGraph.x(orNode[workv]);
				//workACG->y(workv) = acGraph.y(orNode[workv]);
			}
			if (acGraph.attributes() & GraphAttributes::edgeType)
				forall_edges(worke, GW)
				{
					workACG->type(worke)	= acGraph.type(orEdge[worke]);
					//all other attributes are not needed or will be set
				}

			//delete edges
			ListConstIterator<edge> itE = leftEdges.begin();

			while (itE.valid())
			{
				edge e = resultEdge[*itE];
				NodePair np;
				np.m_src = e->source();
				np.m_tgt = e->target();
				
				leftWNodes.pushBack(np);
				//hier muss man noch die Typen sichern
				#ifdef foutput
					cout<<"Deleting edge "<< e <<"\n"<<flush;
				#endif
				GW.delEdge(e);
				
				itE++;
			}//while

			CconnectClusterPlanarEmbed CCP; //cccp

#ifdef foutput
			workCG->writeGraphWinGML("C:\\temp\\ogdl\\subcluster.gml");
#endif

			bool subPlanar = CCP.embed(*workCG, GW);
			//bool subPlanar = CCP.embed(*workCG, *workGraph);
			OGDF_ASSERT(subPlanar);
//********************************************
			//spaeter unten bzw. in ortholayout
			//ClusterGraphAttributes AGW(CGW);
			//ClusterPlanRep CPRW(AGW, CGW);
			//CPRW.initCC(0);
			//CPRW.setOriginalEmbedding();
			/*
			CPlanarEdgeInserter CEI;
			CombinatorialEmbedding CE(CPRW);
			CEI.call(CPRW,CE,GW,leftWNodes);
			CGW.writeGraphWinGML("C:\\temp\\ogdl\\subclusterInsert.gml");
			CPRW.writeGML("C:\\temp\\ogdl\\CPRclusterInsert.gml");
			*/
//********************************************
			//return;//debug
		}//if not planar
		else 
		{
			if (!connect)
			THROW_PARAM(PreconditionViolatedException, pvcClusterPlanar);
		}

	}//if

	//hier kann man aus workgraph die connectedges (bzw. ihre Kopien)
	//entfernen, falls man mehrere CC verarbeiten kann
	//Hier Problem: erst nach Modellboundary zusammenhaengend, danach
	//kann man aber nicht mehr loeschen
	/*
	while (!connectEdges.empty())
	{
		//hier muessten aber die Eintraege in der Einbettung, ausgehende
		//Kanten, aktualisiert werden
		workGraph->delEdge(resultEdge[connectEdges.popFrontRet()]);
	}*/


	//now CCPE should give us the external face
	
	//ClusterPlanRep CP(acGraph, cGraph);
	ClusterPlanRep CP(*workACG, *workCG);

    OGDF_ASSERT(CP.representsCombEmbedding())
	
	const int numCC = CP.numberOfCCs(); //equal to one
	//preliminary
	OGDF_ASSERT(numCC == 1);

	// (width,height) of the layout of each connected component
	Array<DPoint> boundingBox(numCC);

	for (int ikl = 0; ikl < numCC; ikl++)
	{
	
			CP.initCC(ikl);
			CP.setOriginalEmbedding();

			OGDF_ASSERT(CP.representsCombEmbedding())

			Layout drawing(CP);

			//ClusterOrthoLayout COL;

			//m_planarLayouter.get().setOptions(4);//progressive
				//(getOptions() & ~umlOpAlign));

			adjEntry ae = 0;

			//internally compute adjEntry for outer face

			//edges that are reinserted in workGraph (in the same
			//order as leftWNodes)
			List<edge> newEdges;
			UMLPlanarLayoutModule& UCOL = m_planarLayouter.get();
			ClusterOrthoLayout& COL = (ClusterOrthoLayout&)(UCOL);
			//(m_planarLayouter.get()).call(CP, ae, drawing, leftWNodes, newEdges, *workGraph);
			COL.call(CP, ae, drawing, leftWNodes, newEdges, *workGraph);

			OGDF_ASSERT(leftWNodes.size()==newEdges.size())
			OGDF_ASSERT(leftEdges.size()==newEdges.size())
			ListConstIterator<edge> itE = newEdges.begin();
			ListConstIterator<edge> itEor = leftEdges.begin();
			while (itE.valid())
			{
				orEdge[*itE] = *itEor;
				itE++;
				itEor++;
			}

			//hash index over cluster ids
			HashingArray<int, ClusterPosition> CA;

			computeClusterPositions(CP, drawing, CA);

			// copy layout into acGraph
			// Later, we move nodes and edges in each connected component, such
			// that no two overlap.
		    const List<node> &origInCC = CP.nodesInCC(ikl);
		    ListConstIterator<node> itV;

			for(itV = origInCC.begin(); itV.valid(); ++itV) 
			{
				node vG = *itV;

				acGraph.x(orNode[vG]) = drawing.x(CP.copy(vG));
				acGraph.y(orNode[vG]) = drawing.y(CP.copy(vG));

				adjEntry adj;
				forall_adj(adj,vG) 
				{
					if ((adj->index() & 1) == 0) continue;
					edge eG = adj->theEdge();

					edge orE = orEdge[eG];
					if (orE)
						drawing.computePolylineClear(CP,eG,acGraph.bends(orE));
				}//foralladj

			}//for

			//even assignment for all nodes is not enough, we need all clusters
			cluster c;
			forall_clusters(c, *workCG)
			{
				int clNumber = c->index();
				int orNumber = originalClId[c];

				//THIS IS WRONG: only if not readgml graph
				//root is zero in all CGs
				//if ((clNumber == 0) && (orNumber > 0)) throw AlgorithmFailureException();
				//=> if ((c == workCG->rootCluster()) && ())
				//not for root cluster
				//if (clNumber > 0)
				if (c != workCG->rootCluster())
				{
					OGDF_ASSERT(CA.isDefined(clNumber));
					acGraph.clusterHeight(orNumber) = CA[clNumber].m_height;
					acGraph.clusterWidth(orNumber) = CA[clNumber].m_width;
					acGraph.clusterYPos(orNumber) = CA[clNumber].m_miny;
					acGraph.clusterXPos(orNumber) = CA[clNumber].m_minx;
				}//if real cluster
			}//forallclusters
#ifdef foutput
			String fileName;
			fileName.sprintf("C:\\ClusterDrawing%d.gml",ikl);
			CP.writeGML(fileName, drawing);
#endif
			//cGraph.writeGraphWinGML(

			// the width/height of the layout has been computed by the planar
			// layout algorithm; required as input to packing algorithm
			//boundingBox[ikl] = COL.getBoundingBox();
			boundingBox[ikl] = m_planarLayouter.get().getBoundingBox();

	}//for connected components

	
//ab hier noch die Kopien bereitstellen, damit z.B. geaenderte clusterID
//keine Probleme machen

    postProcess(acGraph);
	//
	// arrange layouts of connected components
	//

	Array<DPoint> offset(numCC);

	m_packer.get().call(boundingBox,offset,m_pageRatio);

	// The arrangement is given by offset to the origin of the coordinate
	// system. We still have to shift each node, edge and cluster by the offset
	// of its connected component.

	for(int i = 0; i < numCC; ++i)
	{
		const List<node> &nodes = CP.nodesInCC(i);

		const double dx = offset[i].m_x;
		const double dy = offset[i].m_y;

		HashingArray<int, bool> shifted(false);
		
		// iterate over all nodes in ith CC
		ListConstIterator<node> it;
		for(it = nodes.begin(); it.valid(); ++it)
		{
			node v = *it;

			acGraph.x(orNode[v]) += dx;
			acGraph.y(orNode[v]) += dy;

			// update cluster positions accordingly
            int clNumber = cGraph.clusterOf(orNode[v])->index();

			if ((clNumber > 0) && !shifted[clNumber])
			{
				acGraph.clusterYPos(clNumber) += dy;
				acGraph.clusterXPos(clNumber) += dx;
				shifted[clNumber] = true;
			}//if real cluster

			adjEntry adj;
			forall_adj(adj,v) {
				if ((adj->index() & 1) == 0) continue;
				edge e = adj->theEdge();

				//edge eOr = orEdge[e];
				if (orEdge[e])
				{
					DPolyline &dpl = acGraph.bends(orEdge[e]);
					ListIterator<DPoint> it;
					for(it = dpl.begin(); it.valid(); ++it) {
						(*it).m_x += dx;
						(*it).m_y += dy;
					}//for
				}//if
			}//foralladj
		}//for nodes
	}//for numcc


	while (!connectEdges.empty())
			{
				//hier muessten aber die Eintraege in der Einbettung, ausgehende
				//Kanten, aktualisiert werden
				//workGraph->delEdge(resultEdge[connectEdges.popFrontRet()]);
				G.delEdge(connectEdges.popFrontRet());
			}

	if (subGraph) 
	{
		originalClId.init();
		orCluster.init();
		orNode.init();
		orEdge.init();
		delete workCG;
		delete workACG;
	}//if subgraph created

	acGraph.removeUnnecessaryBendsHV();

//  }//try
	/*catch (...)
	{
		//this may cause problems if error between subgraph init and new
		if (subGraph) {delete workCG; delete workACG;}
		throw AlgorithmFailureException();
	}//catch*/
}//call


void ClusterPlanarizationLayout::assureDrawability(ClusterGraphAttributes &UG)
{
  //preliminary, self loops are killed by the interface
  //should be done here later

  //cant handle self loops yet
  edge e;

  const Graph& G = UG.constGraph();

  //check for selfloops and handle them 
  forall_edges(e, G)
  {
    //OGDF_ASSERT(e->target() != e->source());
     if (e->target() == e->source())  
     {
       //temporarily, loops are handled in visiointerface, so throw
       THROW_PARAM(PreconditionViolatedException, pvcSelfLoop);
       
       //is always an error
       if (UG.type(e) == Graph::generalization) THROW_PARAM(PreconditionViolatedException, pvcSelfLoop);
       else 
       {
         //repair
       }
           
/*---------------------*/
        }
      }//foralledges

  //check for generalization - nontrees
  //if m_fakeTree is set, change type of "back" edges to association
  /*
  m_fakedGens.clear();//?
  if (!dfsGenTree(UG, m_fakedGens, m_fakeTree)) throw PreconditionViolatedException(pvcTreeHierarchies);
  else
  {

	  ListConstIterator<edge> itE = m_fakedGens.begin();
	  while (itE.valid())
	  {
		  UG.type(*itE) = Graph::association;
	      itE++;
	  }
  }
  */
}//assureDrawability

//used to check in advance the drawability or to continue the algorithm
//in case of minor failures
bool ClusterPlanarizationLayout::checkDrawability(ClusterGraphAttributes &UG)
{
   bool retval = true;
   const Graph& G = UG.constGraph();
  //cant handle self loops yet
  edge e;
  forall_edges(e, G)
      retval = (e->target() != e->source()) && retval;
  return retval;
}//checkDrawability

void ClusterPlanarizationLayout::postProcess(ClusterGraphAttributes& UG)
{
  //reset the type of faked associations to generalization
  if (m_fakeTree)
  {
    ListIterator<edge> itE = m_fakedGens.begin();
    while (itE.valid())
    {
      UG.type(*itE) = Graph::generalization;
      itE++;
    }//while
  }//if
    
}//postProcess


} // end namespace ogdf
