/*
* $Revision: 1.10 $
* 
* last checkin:
*   $Author: gutweng $ 
*   $Date: 2007/03/21 11:25:47 $ 
***************************************************************/

/** \file
* \brief implementation of class UMLPlanarizationLayout.
* 
* applies planarization approach for drawing UML diagrams
* by calling a planar layouter for every planarized connected 
* component
* Static and incremental calls available
* Replaces cliques (if m_processCliques is set) to speed up 
* the computation, this does only work in non-UML mode
* 
* \author Carsten Gutwenger
* 
* \par License:
* This file is part of the Open Graph Drawing Framework (OGDF).
* Copyright (C) 2005
* 
* \par
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* Version 2 as published by the Free Software Foundation.
* 
* \par
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* \par
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the Free
* Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301, USA.
* 
* \see  http://www.gnu.org/copyleft/gpl.html
***************************************************************/


#include <ogdf/planarity/UMLPlanarizationLayout.h>
#include <ogdf/planarity/MaximalPlanarSubgraphSimple.h>
#include <ogdf/planarity/FixedEmbeddingInserter.h>
#include <ogdf/orthogonal/UMLOrthoLayout.h>
#include <ogdf/packing/TileToRowsCCPacker.h>
#include <ogdf/planarity/PlanarModule.h>
#include <ogdf/basic/TopologyModule.h>
#include <ogdf/basic/precondition.h>
#include <ogdf/planarity/PlanRepInc.h>
#include <ogdf/basic/Queue.h>
#include <ogdf/planarity/SimpleIncNodeInserter.h>
#include <ogdf/basic/simple_graph_alg.h>
#include <ogdf/graphalg/CliqueFinder.h>
#include <ogdf/planarity/SimpleEmbedder.h>

//#define foutput

namespace ogdf {

	//Constructor:
	//set default values for parameters and set planar layouter,
	//planarization modules
	UMLPlanarizationLayout::UMLPlanarizationLayout()
	{
		//modules
		m_subgraph.set(new MaximalPlanarSubgraphSimple);
		m_inserter.set(new FixedEmbeddingInserter);
		m_planarLayouter.set(new UMLOrthoLayout);
		m_packer.set(new TileToRowsCCPacker);
		m_embedder.set(new SimpleEmbedder);

		//parameters
		setDefaultSettings();
	}

	void UMLPlanarizationLayout::setDefaultSettings()
	{
		//parameters
		m_pageRatio = 1.0;
		m_processCliques = false; //do not search for cliques, only if not uml
		m_cliqueSize = 10;
		m_arrangeLabels = false;
		m_fakeTree = true; //change edge type from generalization to association 
		//if non-tree hierarchy
	}

	//-----------------------------------------------------------------------------
	//incremental call: takes a fixed part of the input
	//graph (indicated by fixedNodes/Edges==true), embeds it using
	//the input layout, then inserts the remaining part into this embedding
	//currently, only the subgraph induced by the fixed nodes is fixed
	void UMLPlanarizationLayout::callIncremental(UMLGraph &umlGraph,
	                                             NodeArray<bool> &fixedNodes,
	                                             const EdgeArray<bool> &fixedEdges)
	{
		try {

			if(((const Graph &)umlGraph).empty())
				return;
			//-------------------
			//check preconditions
			//(change edge types in umlGraph at non-tree hierarchies
			//or replace cliques by nodes)
			preProcess(umlGraph);

			m_nCrossings = 0; //number of inserted crossings	

			//use the options set at the planar layouter
			int l_layoutOptions = m_planarLayouter.get().getOptions();
			bool l_align = ((l_layoutOptions & umlOpAlign)>0);

			//check: only use alignment mode if there are generalizations
			bool l_gensExist = false; //set this for all CC's, start with first gen

			//-------------------------------------------------------
			//first, we sort all edges around each node corresponding
			//to the given layout in umlGraph, then we insert the mergers
			//May be inserted in original or in copy 
			bool umlMerge = false; //Standard: insertion in copy
			//if (umlMerge)
			//{
			//	umlGraph.sortEdgesFromLayout();
			//	prepareIncrementalMergers(umlGraph);
			//}
			//-----------------------------------------------------------
			//second, we embed the fixed part corresponding to the layout
			//given in umlgraph
			//TODO: check if we still need the global lists, do this
			//for all CCs
			//we create lists of the additional elements, maybe this can 
			//later be done implicitly within some other loop

			//----------------------------------------------------------
			// first phase: Compute planarized representation from input
			//----------------------------------------------------------

			//------------------------------------------------
			//now we derive a partial planar representation of 
			//the input graph using its layout information
			PlanRepInc PG(umlGraph, fixedNodes);

			const int numCC = PG.numberOfCCs();

			//TODO: adjust for CC number before/after insertion
			// (width,height) of the layout of each connected component
			Array<DPoint> boundingBox(numCC);

			//------------------------------------------
			//now planarize CCs and apply drawing module
			int i;
			for(i = 0; i < numCC; ++i)
			{
				// we treat connected component i where PG is set
				// to a copy of this CC consisting only of fixed nodes
				node minActive = PG.initMinActiveCC(i);
				//if a single node was made active, we update its status
				if (minActive != 0)
				{
					fixedNodes[minActive] = true;
				}

#ifdef OGDF_DEBUG
				edge e;
				forall_edges(e, PG)
				{
					edge eOrig = PG.original(e);
					if (eOrig)
					{
						OGDF_ASSERT(PG.chain(eOrig).size() <= 1)
					}
				}//foralledges
#endif

				int nOrigVerticesPG = PG.numberOfNodes();
				//TODO: check if copying is really necessary
				//we want to sort the additional nodes and therefore
				//copy the list 
				List<node> addNodes;
				ListConstIterator<node> it = PG.nodesInCC().begin();
				while (it.valid())
				{
					if (!fixedNodes[(*it)])
						addNodes.pushBack((*it));
					it++;
				}//while


				//--------------------------------------------
				//now we insert the additional nodes and edges
				//sort the additional nodes
				//simple strategy: sort by their #connections to the fixed part
				//we can therefore only count if we have fixed nodes
				if ((addNodes.size() > 1) && (PG.nodesInCC().size() != addNodes.size()))
					sortIncrementalNodes(addNodes, fixedNodes);

				//TODO: guarantee that the CC is non-empty and connected
				//insert a first part otherwise
				//DONE: unconnected parts are connected in a chain

				//attention: the following list pop relies on the property of 
				//the sorting algorithm, that the first node has a connection
				//to a fixed node or otherwise none of the addnodes has a 
				//connection to fixednodes (its a CC on its own)
				//in the worst case, we have to introduce a tree, which equals 
				//the steinertree problem, which is NP-hard. As we do not want 
				//to preinsert nodes that do not have layout information (they 
				//do have, but we do not consider it (e.g. for randomly 
				//inserted nodes)), we avoid the problem by inserting artificial 
				//edges connecting all CCs, building a tree of CCs

				//now we derive the embedding given in umlgraph for the fixed part
				//we work on a copy of umlgraph, because we have to add/delete nodes
				//and edges and create a PlanRep on an intermediate representation
				adjEntry adjExternal = 0;

				//--------------------------------------------
				//here lies the main difference to the static call
				//we first set the embedding corresponding to the 
				//input and then planarize the given layout
				TopologyModule TM;

				bool embedded = true;
				try { //TODO:should be catched within setEmbeddingFromGraph
					//do not yet compute external face, embed and planarize
					//with layout given in umlGraph
					embedded = TM.setEmbeddingFromGraph(PG, umlGraph, 
						adjExternal, false, umlMerge);
				}//try
				catch(...)
				{
					embedded = false;
				}//catch

				//returns true if connnectivity edges introduced
				//TODO: hierhin oder eins nach unten?
				bool unConnected = PG.makeTreeConnected(adjExternal);

				//if embedding could not be set correctly, use standard embedding
				if (!embedded)
				{
					reembed(PG, i, l_align, l_gensExist);
				}

				//--------------------------------------------
				//now we compute a combinatorial embedding on
				//the partial PlanRep that is used for node insertion
				//and the external face
				bool singleNode = (PG.numberOfNodes() == 1);
				if ((PG.numberOfEdges() > 0) || singleNode)
				{
					CombinatorialEmbedding E(PG);

					//if we have edges, but no external face, find one
					//we have also to select one later if there are no edges yet
					if((adjExternal == 0) && PG.numberOfEdges() > 0) 
					{
						//face fExternal = E.maximalFace();
						face fExternal = findBestExternalFace(PG,E);
						adjExternal = fExternal->firstAdj();
						//while (PG.sinkConnect(adjExternal->theEdge())) 
						//	adjExternal = adjExternal->faceCycleSucc();
					}
					if ( (adjExternal != 0) && (PG.numberOfEdges() > 0) )
						E.setExternalFace(E.rightFace(adjExternal));

					//-------------------------------------------------
					//we insert additional nodes into the given PlanRep
					SimpleIncNodeInserter inserter(PG);
					ListIterator<node> itAdd = addNodes.begin();
					while (itAdd.valid())
					{
						edge eDebug = (*itAdd)->firstAdj()->theEdge();
						OGDF_ASSERT(PG.chain(eDebug).size() <= 1)
							//we can check here if PG CC connected and speed
							//up insertion by not updating CC part information
							inserter.insertCopyNode((*itAdd), E, umlGraph.type((*itAdd)));

						//select an arbitrary external face if there was only one fixed node
						if (singleNode && (PG.numberOfEdges() > 0))
						{
							adjExternal = PG.firstEdge()->adjSource();
							E.setExternalFace(E.rightFace(adjExternal));
						}
						else
						{
							adjExternal = E.externalFace()->firstAdj();
							int eNum = max(10, PG.numberOfEdges()+1);
							int count = 0;
							while ((adjExternal->theNode() == adjExternal->twin()->theNode()) &&
								(count < eNum))
							{
								adjExternal = adjExternal->faceCycleSucc();
								count++;
							}
							OGDF_ASSERT(count < eNum)
						}

						itAdd++;
					}//while

					if (!umlMerge)
						PG.setupIncremental(i, E);
					OGDF_ASSERT(E.consistencyCheck())

						//we now have a complete representation of the 
						//original CC

						m_nCrossings += PG.numberOfNodes() - nOrigVerticesPG;

					//********************
					//copied from fixembed

					//*********************************************************
					// third phase: Compute layout of planarized representation
					//*********************************************************
					Layout drawing(PG);

					//distinguish between CC's with/without generalizations
					//this changes the input layout modules options!
					if (l_gensExist)
						m_planarLayouter.get().setOptions(l_layoutOptions);
					else m_planarLayouter.get().setOptions((l_layoutOptions & ~umlOpAlign));


					//***************************************
					//call the Layouter for the CC's UMLGraph
					m_planarLayouter.get().call(PG,adjExternal,drawing);

					// copy layout into umlGraph
					// Later, we move nodes and edges in each connected component, such
					// that no two overlap.
					const List<node> &origInCC = PG.nodesInCC(i);
					ListConstIterator<node> itV;

					//set position for original nodes and set bends for
					//all edges
					for(itV = origInCC.begin(); itV.valid(); ++itV) 
					{
						node vG = *itV;

						umlGraph.x(vG) = drawing.x(PG.copy(vG));
						umlGraph.y(vG) = drawing.y(PG.copy(vG));

						adjEntry adj;
						forall_adj(adj,vG) 
						{
							if ((adj->index() & 1) == 0) continue;
							edge eG = adj->theEdge();

							drawing.computePolylineClear(PG,eG,umlGraph.bends(eG));
						}//foralladj
					}//for orig nodes


					if (!umlMerge)
					{
						//insert bend point for incremental mergers
						const SList<node>& mergers = PG.incrementalMergers(i);
						SListConstIterator<node> itMerger = mergers.begin();
						while (itMerger.valid())
						{
							node vMerger = (*itMerger);
							//due to the fact that the merger may be expanded, we are
							//forced to run through the face
							adjEntry adjMerger = PG.expandAdj(vMerger);
							//first save a pointer to the bends of the generalization
							//leading to the target
							DPolyline dpUp;

							//check if there is an expansion face
							if (adjMerger != 0)
							{
								//if there are bends on the edge, copy them
								//TODO: check where to derive the bends from
								//if eUp is nil
								adjEntry adjUp = adjMerger->cyclicPred();
								OGDF_ASSERT(PG.isGeneralization(adjUp->theEdge()))
									edge eUp = PG.original(adjUp->theEdge());
								//There is never an original here in current incremental algo
								//OGDF_ASSERT(eUp)
								if (eUp) dpUp = umlGraph.bends(eUp);
								//we run through the expansion face to the connected edges
								//this edge is to dummy corner
								adjEntry runAdj = adjMerger->faceCycleSucc();
								while (runAdj != adjMerger)
								{
									node vConnect = runAdj->theNode();
									//because of the node collapse using the original
									//edges instead of the merger copy edges (should be
									//fixed for incremental mode) the  degree is 4
									//FIXED!?
									//if (vConnect->degree() != 4) 
									//while?
									if (vConnect->degree() != 3) 
									{
										runAdj = runAdj->faceCycleSucc();
										continue;
									}
									edge eCopy = runAdj->cyclicPred()->theEdge();
									OGDF_ASSERT(eCopy->target() == runAdj->theNode())
										OGDF_ASSERT(PG.isGeneralization(eCopy))
										OGDF_ASSERT(PG.original(eCopy))

										DPolyline &eBends = umlGraph.bends(PG.original(eCopy));

									eBends.pushBack(
										DPoint(drawing.x(vMerger), drawing.y(vMerger)));

									if (eUp)
									{
										ListConstIterator<DPoint> itDp;
										for(itDp = dpUp.begin(); itDp.valid(); ++itDp)
											eBends.pushBack(*itDp);
									}
									else 
										eBends.pushBack(DPoint(drawing.x(adjUp->twin()->theNode()),
										drawing.y(adjUp->twin()->theNode())));

									runAdj = runAdj->faceCycleSucc();

								}

							}
							else //currently all nodes are expanded, but this is not guaranteed
							{
								//first save the bends
								adjEntry adjUp = 0;
								forall_adj(adjMerger, vMerger)
								{
									//upgoing edge
									if (adjMerger->theEdge()->source() == vMerger)
									{
										adjUp = adjMerger;
										OGDF_ASSERT(PG.isGeneralization(adjMerger->theEdge()))
											edge eUp = PG.original(adjMerger->theEdge());
										//check if this is
										//a) the merger up edge and not a connectivity edge
										//b) what to do if there is no original of outgoing edge
										if (eUp) 
											dpUp = umlGraph.bends(eUp);
										break;
									}//if

								}//foralladj
								forall_adj(adjMerger, vMerger)
								{
									if (adjMerger->theEdge()->target() == vMerger)
									{
										edge eOrig = PG.original(adjMerger->theEdge());
										if (eOrig)
										{
											//incoming merger edges always have an original here!
											umlGraph.bends(eOrig).pushBack(DPoint(drawing.x(vMerger), 
												drawing.y(vMerger)));

											//was there an original edge?
											if (dpUp.size()>0)
											{
												ListConstIterator<DPoint> itDp;
												for(itDp = dpUp.begin(); itDp.valid(); ++itDp)
													umlGraph.bends(eOrig).pushBack(*itDp);
											}//if
											else
											{
												if (adjUp)
													umlGraph.bends(eOrig).pushBack(DPoint(drawing.x(adjUp->twin()->theNode()),
													drawing.y(adjUp->twin()->theNode())));
											}//else
										}//if

									}
								}//forall adj
							}//else
							itMerger++;
						}//while merger nodes
					}//if !umlMerge

					//umlGraph.writeGML("C:\\FullLayout2Inc.gml");

					// the width/height of the layout has been computed by the planar
					// layout algorithm; required as input to packing algorithm
					boundingBox[i] = m_planarLayouter.get().getBoundingBox();

					//*******************

				}//if #edges > 0

				else 
				{
					//TODO: what if there are no edges but the insertion edges
					//DONE: we make the CC treeConnected
					//Nonetheless we have to compute a layout here
					OGDF_ASSERT(PG.numberOfNodes() < 2)
				}		



				//TODO: set m_crossings here
				//m_nCrossings += PG.numberOfNodes() - numOrigNodes;

			}//for connected components

			//*******************
			//TODO: check shifting to new place

			//
			// arrange layouts of connected components
			//

			Array<DPoint> offset(numCC);

			m_packer.get().call(boundingBox,offset,m_pageRatio);

			// The arrangement is given by offset to the origin of the coordinate
			// system. We still have to shift each node and edge by the offset
			// of its connected component.

			for(i = 0; i < numCC; ++i)
			{
				const List<node> &nodes = PG.nodesInCC(i);

				const double dx = offset[i].m_x;
				const double dy = offset[i].m_y;

				// iterate over all nodes in ith CC
				ListConstIterator<node> it;
				for(it = nodes.begin(); it.valid(); ++it)
				{
					node v = *it;

					umlGraph.x(v) += dx;
					umlGraph.y(v) += dy;

					adjEntry adj;
					forall_adj(adj,v) {
						if ((adj->index() & 1) == 0) continue;
						edge e = adj->theEdge();

						DPolyline &dpl = umlGraph.bends(e);
						ListIterator<DPoint> it;
						for(it = dpl.begin(); it.valid(); ++it) 
						{
							(*it).m_x += dx;
							(*it).m_y += dy;
						}//for bends
					}//foralladj
				}//fornodes
			}//for cc's

			if (umlMerge) umlGraph.undoGenMergers();

			umlGraph.removeUnnecessaryBendsHV();
			//********************
			//new position after adding clique process, check if correct
			postProcess(umlGraph);
		}//try
		catch (...)
		{
			call(umlGraph);
			return;
		}

	}//callIncremental


	//Insertion order of added nodes: 
	//sorting strategy: we count all adjacent nodes that are in 
	//fixedNodes and sort the nodes correspondingly
	//In addition, we guarantee that no node is inserted before at least one
	//of its neighbours is inserted

	//compute how far away from the fixed part the added nodes lay
	//uses BFS
	void UMLPlanarizationLayout::getFixationDistance(node startNode, 
		HashingArray<int, int> &distance,
		const NodeArray<bool> &fixedNodes)
	{
		//we only change the distance for nodes with distance <=0 because they are not 
		//connected to the fixed part
		//we only care about nodes which are members of array distance
		HashingArray<int, bool> indexMark(false);
		//the BFS queue
		QueuePure<node> nodeQ;

		nodeQ.append(startNode);
		indexMark[startNode->index()] = true;
		while (!nodeQ.empty())
		{
			adjEntry adjE;
			node topNode = nodeQ.pop();
			//hier aufpassen: geht nur, wenn kein fixedNode eine Distance hat
			//alternativ: alle auf null
			//zur sicherheit: fixed uebergeben und vergleichen
			bool fixedBase = fixedNodes[topNode];

			forall_adj(adjE, topNode)
			{
				node testNode = adjE->twin()->theNode();
				int ind = testNode->index();

				if (!indexMark[ind])
				{
					indexMark[ind] = true;
					nodeQ.append(testNode);
				}

				//we have a distance value, i.e. ind is an additional node
				if (!fixedNodes[testNode])
				{
					if (distance[ind] <= 0)
					{
						//should never occur (check: nodes not in CC?)
						if (fixedBase) 
						{
							distance[ind] = max(-1, distance[ind]);
							OGDF_ASSERT(false)
						}
						else
						{
							if (distance[ind] == 0) distance[ind] = min(-1, distance[topNode->index()] - 1);
							else distance[ind] = min(-1, max(distance[ind], distance[topNode->index()] - 1));
						}//else
					}//if node without contact to fixed nodes
				}//if distance (is an addnode)
			}//foralladj
		}//while

	}//getFixationDistance

	//Attention: changing this behavior makes it necessary to check 
	//the call procedure for the case where one of the addnodes is
	//made fix to allow insertion of an edge
	void UMLPlanarizationLayout::sortIncrementalNodes(List<node> &addNodes,
		const NodeArray<bool> &fixedNodes)
	{
		//if there are no fixed nodes, we can not sort
		//todo: do some other sorting

		//we count all adjacent fixed nodes
		//store degree by node index
		HashingArray<int, int> indexToDegree(0);
		ListIterator<node> it = addNodes.begin();
		adjEntry adjE;
		node someFixedNode = 0;

		while (it.valid())
		{
			if ((*it)->degree() < 1) 
			{
				indexToDegree[(*it)->index()] = 0;
				it++;
				continue;
			}
			int vDegree = 0;
			adjE = (*it)->firstAdj();
			do {
				if (fixedNodes[adjE->twin()->theNode()])
				{
					vDegree++;
					someFixedNode = adjE->twin()->theNode();
				}
				adjE = adjE->cyclicSucc();
			} while (adjE != (*it)->firstAdj());

			indexToDegree[(*it)->index()] = vDegree;

			it++;
		}//while
		//for all nodes that are not connected to the fixed part we have to guarantee
		//that they are not inserted before one of their neighbours is inserted
		//therefore we set negative values for all nodes corresponding to their distance 
		//to the fixed part
		OGDF_ASSERT(someFixedNode != 0)
			if (someFixedNode == 0) throw AlgorithmFailureException();
		//we start the BFS at some fixed node
		getFixationDistance(someFixedNode,indexToDegree, fixedNodes);


		//we sort the nodes in decreasing vDegree value order
		AddNodeComparer comp(indexToDegree);
		addNodes.quicksort(comp);

	}//sortincrementalnodes

	void UMLPlanarizationLayout::reembed(PlanRepUML &PG, int ccNumber, bool l_align,
		bool l_gensExist)
	{
		//TODO: update by reinitialization?
		//PG.initActiveCC(i);
		//first we remove all inserted crossings
		node v;
		List<node> crossings;
		forall_nodes(v, PG)
		{
			if (PG.isCrossingType(v))
			{
				crossings.pushBack(v);
			}
		}//forallnodes
		ListIterator<node> it = crossings.begin();
		while (it.valid())
		{
			PG.removeCrossing((*it));
			it++;
		}//while
		//***************************************
		// first phase: Compute a planar subgraph
		//***************************************

		// The planar subgraph should contain as many generalizations
		// as possible, hence we put all generalizations into the list
		// preferedEdges.
		List<edge> preferedEdges;
		edge e;
		EdgeArray<int> costOrig(PG.original(), 1);
		forall_edges(e,PG) 
		{
			if (PG.typeOf(e) == Graph::generalization)
			{
				if (l_align) l_gensExist = true;
				preferedEdges.pushBack(e);
				edge ori = PG.original(e);
				//high cost to allow alignment without crossings
				if ( (l_align) &&
					((ori && (PG.typeOf(e->target()) == Graph::generalizationMerger))
					|| (PG.alignUpward(e->adjSource())) 
					)
					)
					costOrig[ori] = 10;

			}//generalization
		}//foralledges

		List<edge> deletedEdges;
		m_subgraph.get().callAndDelete(PG, preferedEdges, deletedEdges);


		//**************************************
		// second phase: Re-insert deleted edges
		//**************************************

		m_inserter.get().callForbidCrossingGens(PG, costOrig, deletedEdges);

		PlanarModule pm;
		OGDF_ASSERT(pm.planarityTest(PG));


		//
		// determine embedding of PG
		//

		// We currently compute any embedding and choose the maximal face
		// as external face

		// if we use FixedEmbeddingInserter, we have to re-use the computed
		// embedding, otherwise crossing nodes can turn into "touching points"
		// of edges (alternatively, we could compute a new embedding and
		// finally "remove" such unnecessary crossings).
		if(!PG.representsCombEmbedding())
			pm.planarEmbed(PG);
		adjEntry adjExternal = 0;

		if(PG.numberOfEdges() > 0) 
		{
			CombinatorialEmbedding E(PG);
			//face fExternal = E.maximalFace();
			face fExternal = findBestExternalFace(PG,E);
			adjExternal = fExternal->firstAdj();
			//while (PG.sinkConnect(adjExternal->theEdge())) 
			//	adjExternal = adjExternal->faceCycleSucc();
		}
	}//reembed


	//---------------------------------------------------------
	//compute a layout with the given embedding, take special
	//care of  crossings (assumes that all crossings are non-ambiguous
	//and can be derived from node/bend positions)
	//---------------------------------------------------------
	void UMLPlanarizationLayout::callFixEmbed(UMLGraph &umlGraph)
	{
		m_nCrossings = 0;

		if(((const Graph &)umlGraph).empty())
			return;

		//check necessary preconditions
		preProcess(umlGraph);

		int l_layoutOptions = m_planarLayouter.get().getOptions();
		bool l_align = ((l_layoutOptions & umlOpAlign)>0);

		//--------------------------------------------------------
		//first, we sort all edges around each node corresponding
		//to the given layout in umlGraph
		//then we insert the mergers
		//try to rebuild this: insert mergers only in copy
		bool umlMerge = false; //Standard: false
		//if (umlMerge)
		//{
		//	umlGraph.sortEdgesFromLayout();
		//	prepareIncrementalMergers(umlGraph);
		//}

		int i;

		//*********************************************************
		// first phase: Compute planarized representation from input
		//*********************************************************

		//now we derive a planar representation of the input 
		//graph using its layout information
		PlanRepUML PG(umlGraph);

		const int numCC = PG.numberOfCCs();

		// (width,height) of the layout of each connected component
		Array<DPoint> boundingBox(numCC);


		//******************************************
		//now planarize CCs and apply drawing module
		for(i = 0; i < numCC; ++i)
		{
			// we treat connected component i
			// PG is set to a copy of this CC

			PG.initCC(i);

			int nOrigVerticesPG = PG.numberOfNodes();

			//alignment: check wether gens exist, special treatment is necessary
			bool l_gensExist = false; //set this for all CC's, start with first gen,
			//this setting can be mixed among CC's without problems

			//*********************************************************
			// we don't need to compute a planar subgraph, because we 
			// use the given embedding
			//*********************************************************

			adjEntry adjExternal = 0;

			//here lies the main difference to the other call
			//we first planarize by using the given layout and then 
			//set the embedding corresponding to the input
			bool embedded;
			TopologyModule TM;
			try {
				embedded = TM.setEmbeddingFromGraph(PG, umlGraph, adjExternal, umlMerge);
			}//try
			catch (...)
			{
				//TODO: check for graph changes that are not undone
				embedded = false;
			}


			//-------------------------------------------------
			//if not embedded correctly
			if (!embedded)
			{
				reembed(PG, i, l_align, l_gensExist);
			}//if !embedded

			//-------------------------------------------------

			CombinatorialEmbedding E(PG);
			
			if (!umlMerge)
				PG.setupIncremental(i, E);

			//
			// determine embedding of PG
			//

			// We currently compute any embedding and choose the maximal face
			// as external face

			// if we use FixedEmbeddingInserter, we have to re-use the computed
			// embedding, otherwise crossing nodes can turn into "touching points"
			// of edges (alternatively, we could compute a new embedding and
			// finally "remove" such unnecessary crossings).

			if((adjExternal == 0) && PG.numberOfEdges() > 0) 
			{
				//face fExternal = E.maximalFace();
				face fExternal = findBestExternalFace(PG,E);
				adjExternal = fExternal->firstAdj();
				//while (PG.sinkConnect(adjExternal->theEdge())) 
				//	adjExternal = adjExternal->faceCycleSucc();
			}

			m_nCrossings += PG.numberOfNodes() - nOrigVerticesPG;


			//*********************************************************
			// third phase: Compute layout of planarized representation
			//*********************************************************

			Layout drawing(PG);

			//distinguish between CC's with/without generalizations
			//this changes the input layout modules options!
			if (l_gensExist)
				m_planarLayouter.get().setOptions(l_layoutOptions);
			else m_planarLayouter.get().setOptions((l_layoutOptions & ~umlOpAlign));

			//***************************************
			//call the Layouter for the CC's UMLGraph
			m_planarLayouter.get().call(PG,adjExternal,drawing);

			// copy layout into umlGraph
			// Later, we move nodes and edges in each connected component, such
			// that no two overlap.
			const List<node> &origInCC = PG.nodesInCC(i);
			ListConstIterator<node> itV;

			//set position for original nodes and set bends for
			//all edges
			for(itV = origInCC.begin(); itV.valid(); ++itV) 
			{
				node vG = *itV;

				umlGraph.x(vG) = drawing.x(PG.copy(vG));
				umlGraph.y(vG) = drawing.y(PG.copy(vG));

				adjEntry adj;
				forall_adj(adj,vG) 
				{
					if ((adj->index() & 1) == 0) continue;
					edge eG = adj->theEdge();

					drawing.computePolylineClear(PG,eG,umlGraph.bends(eG));
				}//foralladj
			}//for orig nodes

			if (!umlMerge)
			{
				//insert bend point for incremental mergers
				const SList<node>& mergers = PG.incrementalMergers(i);
				SListConstIterator<node> itMerger = mergers.begin();
				while (itMerger.valid())
				{
					node vMerger = (*itMerger);
					//due to the fact that the merger may be expanded, we are
					//forced to run through the face
					adjEntry adjMerger = PG.expandAdj(vMerger);
					//check if there is an expansion face
					if (adjMerger != 0)
					{
						//we run through the expansion face to the connected edges
						//this edge is to dummy corner
						adjEntry runAdj = adjMerger->faceCycleSucc();
						while (runAdj != adjMerger)
						{
							node vConnect = runAdj->theNode();
							//because of the node collapse using the original
							//edges instead of the merger copy edges (should be
							//fixed for incremental mode) the  degree is 4
							//if (vConnect->degree() != 3) 
							if (vConnect->degree() != 4) 
							{
								runAdj = runAdj->faceCycleSucc();
								continue;
							}
							edge eCopy = runAdj->cyclicPred()->theEdge();
							OGDF_ASSERT(eCopy->target() == runAdj->theNode())
								OGDF_ASSERT(PG.isGeneralization(eCopy))
								OGDF_ASSERT(PG.original(eCopy))
								umlGraph.bends(PG.original(eCopy)).pushBack(
								DPoint(drawing.x(vMerger), drawing.y(vMerger)));
							runAdj = runAdj->faceCycleSucc();

						}

					}
					else //currently all nodes are expanded, but this is not guaranteed
					{
						forall_adj(adjMerger, vMerger)
						{
							if (adjMerger->theEdge()->target() == vMerger)
							{
								edge eOrig = PG.original(adjMerger->theEdge());
								if (eOrig)
									//incoming merger edges always have an original here!
									umlGraph.bends(eOrig).pushBack(DPoint(drawing.x(vMerger), 
									drawing.y(vMerger)));

							}
						}//forall adj
					}
					itMerger++;
				}//while merger nodes
			}

			// the width/height of the layout has been computed by the planar
			// layout algorithm; required as input to packing algorithm
			boundingBox[i] = m_planarLayouter.get().getBoundingBox();
		}//for cc's


		postProcess(umlGraph);
		//
		// arrange layouts of connected components
		//

		Array<DPoint> offset(numCC);

		m_packer.get().call(boundingBox,offset,m_pageRatio);

		// The arrangement is given by offset to the origin of the coordinate
		// system. We still have to shift each node and edge by the offset
		// of its connected component.

		for(i = 0; i < numCC; ++i)
		{
			const List<node> &nodes = PG.nodesInCC(i);

			const double dx = offset[i].m_x;
			const double dy = offset[i].m_y;

			// iterate over all nodes in ith CC
			ListConstIterator<node> it;
			for(it = nodes.begin(); it.valid(); ++it)
			{
				node v = *it;

				umlGraph.x(v) += dx;
				umlGraph.y(v) += dy;

				adjEntry adj;
				forall_adj(adj,v) {
					if ((adj->index() & 1) == 0) continue;
					edge e = adj->theEdge();

					DPolyline &dpl = umlGraph.bends(e);
					ListIterator<DPoint> it;
					for(it = dpl.begin(); it.valid(); ++it) 
					{
						(*it).m_x += dx;
						(*it).m_y += dy;
					}//for bends
				}//foralladj
			}//fornodes
		}//for cc's

		umlGraph.undoGenMergers();

		umlGraph.removeUnnecessaryBendsHV();

	}//callfixembed



	//-----------------------------------------------------------------------------
	//static call function: compute an UML layout for graph umlGraph
	//-----------------------------------------------------------------------------
	void UMLPlanarizationLayout::call(UMLGraph &umlGraph)
	{
		m_nCrossings = 0;

		if(((const Graph &)umlGraph).empty())
			return;

		//check necessary preconditions
		preProcess(umlGraph);

		//---------------------------------------------------
		// preprocessing: insert a merger for generalizations
		umlGraph.insertGenMergers();

		PlanRepUML PG(umlGraph);

		const int numCC = PG.numberOfCCs();

		// (width,height) of the layout of each connected component
		Array<DPoint> boundingBox(numCC);

		int i;

		//alignment section (should not be here, because planarlayout should
		//not know about the meaning of layouter options and should not cope 
		//with them), move later
		//we have to distinguish between cc's with and without generalizations 
		//if the alignment option is set 
		int l_layoutOptions = m_planarLayouter.get().getOptions();
		bool l_align = ((l_layoutOptions & umlOpAlign)>0);
		//end alignment section

		//------------------------------------------
		//now planarize CCs and apply drawing module
		for(i = 0; i < numCC; ++i)
		{
			// we treat connected component i
			// PG is set to a copy of this CC

			PG.initCC(i);

			int nOrigVerticesPG = PG.numberOfNodes();

			//alignment: check wether gens exist, special treatment is necessary
			bool l_gensExist = false; //set this for all CC's, start with first gen,
			//this setting can be mixed among CC's without problems

			//---------------------------------------
			// first phase: Compute a planar subgraph
			//---------------------------------------

			// The planar subgraph should contain as many generalizations
			// as possible, hence we put all generalizations into the list
			// preferredEdges.
			// In the case of clique processing we have to prevent the 
			// clique replacement (star) edges from being crossed.
			// We do not allow that clique replacement is done in the case
			// of UML diagrams, therefore we can temporarily set the type of
			// all deleted and replacement edges to generalization to
			// avoid crossings
			EdgeArray<Graph::EdgeType> savedType(PG);
			EdgeArray<Graph::EdgeType> savedOrigType(PG.original()); //for deleted copies

			List<edge> preferedEdges;
			edge e;
			EdgeArray<int> costOrig(PG.original(), 1);
			//edgearray for reinserter call: which edge may never be crossed?
			EdgeArray<bool> noCrossingEdge(PG.original(), false);
			forall_edges(e,PG) 
			{
				edge ori = PG.original(e);
				if (m_processCliques)
				{
					savedType[e] = PG.typeOf(e);
					if (ori)
					{
						if (umlGraph.isReplacement(ori))
						{
							preferedEdges.pushBack(e);
							costOrig[ori] = 10;
							PG.setGeneralization(e);
							noCrossingEdge[ori] = true;
							continue;
						}//if clique replacement

					}//if
				}//if cliques

				if (PG.typeOf(e) == Graph::generalization)
				{
					if (l_align) l_gensExist = true;
#ifdef OGDF_DEBUG
					if (ori)
					{
						OGDF_ASSERT(!(noCrossingEdge[ori]))
					}
#endif
					preferedEdges.pushBack(e);

					//high cost to allow alignment without crossings
					if ( (l_align) &&
						((ori && (PG.typeOf(e->target()) == Graph::generalizationMerger))
						|| (PG.alignUpward(e->adjSource())) 
					 )
					 )
					 costOrig[ori] = 10;

				}//generalization
			}//foralledges

			List<edge> deletedEdges;
			m_subgraph.get().callAndDelete(PG, preferedEdges, deletedEdges);

			if (m_processCliques)
			{
				ListIterator<edge> itEdge = deletedEdges.begin();
				while (itEdge.valid())
				{
					savedOrigType[*itEdge] = PG.typeOrig(*itEdge);
					umlGraph.type(*itEdge) = Graph::generalization;

					OGDF_ASSERT(!(umlGraph.isReplacement(*itEdge)))


						itEdge++;
				}//while
			}//if cliques

			//**************************************
			// second phase: Re-insert deleted edges
			//**************************************

			if (m_processCliques)
			{
				m_inserter.get().call(PG, costOrig, noCrossingEdge, deletedEdges);
			}
			else
				m_inserter.get().callForbidCrossingGens(PG, costOrig, deletedEdges);

			PlanarModule pm;
			OGDF_ASSERT(pm.planarityTest(PG));

			//reset the changed edge types
			if (m_processCliques)
			{
				forall_edges(e, PG)
				{
					//if replacement
					edge ori = PG.original(e);
					if (ori)
						if (umlGraph.isReplacement(ori))
						{
							OGDF_ASSERT(PG.chain(ori).size() == 1) 
								PG.setType(e, savedType[e]);
							umlGraph.type(ori) = Graph::association;
						}
				}//foralledges
				//set the type of the reinserted edges in original and copy
				ListIterator<edge> itEdge = deletedEdges.begin();
				while (itEdge.valid())
				{
					umlGraph.type(*itEdge) = savedOrigType[*itEdge];
					const List<edge> &le = PG.chain(*itEdge);
					ListConstIterator<edge> it2 = le.begin();
					while (it2.valid())
					{
						PG.setType(*it2, savedOrigType[*itEdge]);
						it2++;
					}//while
					itEdge++;
				}//while
			}//if cliques

			//-----------------------------------------------------------------
			//additional check for clique processing
			//guarantee that there is no crossing in replacement, otherwise
			//we can not cluster a star
#ifdef OGDF_DEBUG
#ifndef foutput
			String fileName;
#endif
			fileName.sprintf("planar2.gml");

			if (m_processCliques)
			{
				SListConstIterator<node> itDV = umlGraph.centerNodes().begin();
				while (itDV.valid())
				{
					node cent = PG.copy(*itDV);
					OGDF_ASSERT(cent->degree() == (*itDV)->degree())
						adjEntry adj;
					forall_adj(adj, cent)
					{
						OGDF_ASSERT(PG.original(adj->twin()->theNode()))
							OGDF_ASSERT(!(PG.isGeneralization(adj->theEdge())))
							PG.setBrother(adj->theEdge());//only output coloring

					}//foralladj
					itDV++;
				}//while
				Layout fakeDrawing(PG);
				PG.writeGML(fileName, fakeDrawing);
			}//if clique replacement
#endif
			//----------------------------------------------------------


			//
			// determine embedding of PG
			//

			// We currently compute any embedding and choose the maximal face
			// as external face

			// if we use FixedEmbeddingInserter, we have to re-use the computed
			// embedding, otherwise crossing nodes can turn into "touching points"
			// of edges (alternatively, we could compute a new embedding and
			// finally "remove" such unnecessary crossings).
			if(!PG.representsCombEmbedding())
				pm.planarEmbed(PG);
			adjEntry adjExternal = 0;

			if(PG.numberOfEdges() > 0) 
			{
				CombinatorialEmbedding E(PG);
				//face fExternal = E.maximalFace();
				face fExternal = findBestExternalFace(PG,E);
				adjExternal = fExternal->firstAdj();
				//while (PG.sinkConnect(adjExternal->theEdge())) 
				//	adjExternal = adjExternal->faceCycleSucc();
			}

			m_nCrossings += PG.numberOfNodes() - nOrigVerticesPG;


			//*********************************************************
			// third phase: Compute layout of planarized representation
			//*********************************************************


			if (m_processCliques)
			{
				//insert boundaries around clique representation node
				//and compute a representation layout for the cliques
				//(is needed to guarantee the size of the replacement 
				//boundary)
				//conserve external face information
				SListPure<node> centerNodes = umlGraph.centerNodes();
				SListIterator<node> itNode = centerNodes.begin();
				while (itNode.valid())
				{
					node vCopy = PG.copy(*itNode);
					PG.insertBoundary(*itNode, adjExternal);//vCopy);
					//OGDF_ASSERT(PG.boundaryAdj(*itNode))//vCopy))
					itNode++;
				}//while

			}//if cliques

			Layout drawing(PG);

			//distinguish between CC's with/without generalizations
			//this changes the input layout modules options!
			if (l_gensExist)
				m_planarLayouter.get().setOptions(l_layoutOptions);
			else m_planarLayouter.get().setOptions((l_layoutOptions & ~umlOpAlign));

			//***************************************
			//call the Layouter for the CC's UMLGraph
			m_planarLayouter.get().call(PG,adjExternal,drawing);

			//--------------------------------------
			//we now have to reposition clique nodes
			if (m_processCliques)
			{
				//--------------------------------------------------------
				//first, we derive the current size of the boundary around
				//the center nodes, then we position the clique nodes 
				//in a circular fashion in the boundary

				//the node array is only used for the simple anchor move strategy at the
				//end of this if and can later be removed
				NodeArray<bool> isClique(PG, false);

				SListPure<node> centerNodes = umlGraph.centerNodes();
				SListIterator<node> itNode = centerNodes.begin();
				while (itNode.valid())
				{
					node centerNode = (*itNode);
					//OGDF_ASSERT(PG.boundaryAdj(centerNode))//vCopy))
					adjEntry adjBoundary = PG.boundaryAdj(centerNode);//vCopy);
					//-----------------------------------------------------
					//derive the boundary size
					//if the boundary does not exist (connected component is clique), we 
					//only run over the nodes adjacent to centerNode
					double maxx, maxy, minx, miny;
					bool first = true;
					if (adjBoundary)
					{
						adjEntry adjRunner = adjBoundary;
						//explore the dimension and position of the boundary rectangle
						//TODO: guarantee (edge types?) that we run around a boundary
						do {

							if (first)
							{
								maxx = minx = drawing.x(adjRunner->theNode());
								maxy = miny = drawing.y(adjRunner->theNode());
								first = false;
							}
							else
							{
								double vx = drawing.x(adjRunner->theNode());
								double vy = drawing.y(adjRunner->theNode());
								if (vx < minx) minx = vx;
								if (vx > maxx) maxx = vx;
								if (vy < miny) miny = vy;
								if (vy > maxy) maxy = vy;
							}
							//are we at a bend or a crossing?
							OGDF_ASSERT((adjRunner->twin()->theNode()->degree() == 2) ||
								(adjRunner->twin()->theNode()->degree() == 4))
								//bend
								if (adjRunner->twin()->theNode()->degree() < 4) 
									adjRunner = adjRunner->faceCycleSucc();
								else adjRunner = adjRunner->faceCycleSucc()->cyclicPred();
						} while (adjRunner != adjBoundary);
					}//if boundary exists
					else 
					{
						forall_adj(adjBoundary, centerNode)
						{
							node w = adjBoundary->twin()->theNode();
							if (first)
							{
								minx = maxx = drawing.x(PG.copy(w));
								miny = maxy = drawing.y(PG.copy(w));
								first = false;
							}
							else
							{
								double vx = drawing.x(PG.copy(w)); 
								double vy = drawing.y(PG.copy(w));
								if (vx < minx) minx = vx;
								if (vx > maxx) maxx = vx;
								if (vy < miny) miny = vy;
								if (vy > maxy) maxy = vy;
							}//else
						}//foralladj
					}//else
					//-----------------------------------------------------------

					//we now have to arrange the nodes on a circle with positions
					//that respect the position within the given rectangle, i.e.
					//the node with highest position should be on top etc. . This
					//helps in avoiding unnecessary crossings of outgoing edges
					//with the clique circle and unnecessary long edges to the anchors
					//Note that the ordering around centerNode in umlGraph is different
					//to the ordering in the drawing (defined on PG)
					//recompute size of clique and the node positions
					//test
					//---------------------------------------------------------
					//derive the ordering of the nodes around centerNode in the 
					//planarized copy
					List<node> adjNodes;
					fillAdjNodes(adjNodes, PG, centerNode, isClique, drawing);

					//-----------------------------
					//compute clique node positions
					umlGraph.computeCliquePosition(adjNodes, centerNode, 
						min(maxx-minx,maxy-miny));//, centerNode->firstAdj());
					//testend

					double centralX = (maxx-minx)/2.0+minx;
					double centralY = (maxy-miny)/2.0+miny;
					double circleX  = umlGraph.cliqueRect(centerNode).width()/2.0;
					double circleY  = umlGraph.cliqueRect(centerNode).height()/2.0;

					//now we have the position and size of the rectangle around 
					//the clique

					//assign shifted coordinates to drawing
					forall_adj(adjBoundary, centerNode)
					{
						node w = adjBoundary->twin()->theNode();
						drawing.x(PG.copy(w)) = centralX-circleX+umlGraph.cliquePos(w).m_x;
						drawing.y(PG.copy(w)) = centralY-circleY+umlGraph.cliquePos(w).m_y;
					}//foralladj
					//cout<<"\n";

					itNode++;
				}//while
				
				//simple strategy to move anchor positions too (they are not needed: 
				// move to same position)
				node w;
				forall_nodes(w, PG)
				{
					//forall clique nodes shift the anchor points
					if (isClique[w])
					{
						adjEntry adRun = w->firstAdj();
						do 
						{
							node wOpp = adRun->twin()->theNode();
							drawing.x(wOpp) = drawing.x(w);
							drawing.y(wOpp) = drawing.y(w);
							adRun = adRun->cyclicSucc();
						} while (adRun != w->firstAdj());

					}
				}

			}//if cliques

			// copy layout into umlGraph
			// Later, we move nodes and edges in each connected component, such
			// that no two overlap.
			const List<node> &origInCC = PG.nodesInCC(i);
			ListConstIterator<node> itV;

			for(itV = origInCC.begin(); itV.valid(); ++itV) {
				node vG = *itV;

				umlGraph.x(vG) = drawing.x(PG.copy(vG));
				umlGraph.y(vG) = drawing.y(PG.copy(vG));

				adjEntry adj;
				forall_adj(adj,vG) {
					if ((adj->index() & 1) == 0) continue;
					edge eG = adj->theEdge();

					drawing.computePolylineClear(PG,eG,umlGraph.bends(eG));
				}
			}

			// the width/height of the layout has been computed by the planar
			// layout algorithm; required as input to packing algorithm
			boundingBox[i] = m_planarLayouter.get().getBoundingBox();
		}//for cc's

		//
		// arrange layouts of connected components
		//

		Array<DPoint> offset(numCC);

		m_packer.get().call(boundingBox,offset,m_pageRatio);

		// The arrangement is given by offset to the origin of the coordinate
		// system. We still have to shift each node and edge by the offset
		// of its connected component.

		for(i = 0; i < numCC; ++i)
		{
			const List<node> &nodes = PG.nodesInCC(i);

			const double dx = offset[i].m_x;
			const double dy = offset[i].m_y;

			// iterate over all nodes in ith CC
			ListConstIterator<node> it;
			for(it = nodes.begin(); it.valid(); ++it)
			{
				node v = *it;

				umlGraph.x(v) += dx;
				umlGraph.y(v) += dy;

				adjEntry adj;
				forall_adj(adj,v) {
					if ((adj->index() & 1) == 0) continue;
					edge e = adj->theEdge();

					DPolyline &dpl = umlGraph.bends(e);
					ListIterator<DPoint> it;
					for(it = dpl.begin(); it.valid(); ++it) {
						(*it).m_x += dx;
						(*it).m_y += dy;
					}
				}
			}
		}

		umlGraph.undoGenMergers();

		umlGraph.removeUnnecessaryBendsHV();

		//new position after adding of cliqueprocess, check if correct
		postProcess(umlGraph);

	}//call



	//-----------------------------------------------------------------------------
	//static call function: compute a layout for graph umlGraph without 
	//special UML or interactive features, clique processing etc.
	//-----------------------------------------------------------------------------
	void UMLPlanarizationLayout::simpleCall(UMLGraph &umlGraph)
	{
		//this simple call method does not care about any special treatments
		//of subgraphs, layout informations etc., therefore we save the
		//option status and set them back later on
		bool l_saveCliqueHandling = m_processCliques;
		m_processCliques = false;

		m_nCrossings = 0;

		if(((const Graph &)umlGraph).empty())
			return;

		//check necessary preconditions
		preProcess(umlGraph);

		//---------------------------------------------------
		// preprocessing: insert a merger for generalizations
		umlGraph.insertGenMergers();

		PlanRepUML PG(umlGraph);

		const int numCC = PG.numberOfCCs();

		// (width,height) of the layout of each connected component
		Array<DPoint> boundingBox(numCC);

		int i;

		//------------------------------------------
		//now planarize CCs and apply drawing module
		for(i = 0; i < numCC; ++i)
		{
			// we treat connected component i
			// PG is set to a copy of this CC

			PG.initCC(i);

			int nOrigVerticesPG = PG.numberOfNodes();

			//---------------------------------------
			// first phase: Compute a planar subgraph
			//---------------------------------------

			// The planar subgraph should contain as many generalizations
			// as possible, hence we put all generalizations into the list
			// preferredEdges.

			List<edge> preferedEdges;
			edge e;
			EdgeArray<int> costOrig(PG.original(), 1);
			//edgearray for reinserter call: which edge may never be crossed?
			EdgeArray<bool> noCrossingEdge(PG.original(), false);
			forall_edges(e,PG) 
			{
				if (PG.typeOf(e) == Graph::generalization)
				{
					preferedEdges.pushBack(e);	
				}//generalization
			}//foralledges

			List<edge> deletedEdges;
			m_subgraph.get().callAndDelete(PG, preferedEdges, deletedEdges);

			//**************************************
			// second phase: Re-insert deleted edges
			//**************************************

			m_inserter.get().callForbidCrossingGens(PG, costOrig, deletedEdges);

			//calls embedder module:
			adjEntry adjExternal = 0;
			m_embedder.get().call(PG, adjExternal);

			m_nCrossings += PG.numberOfNodes() - nOrigVerticesPG;


			//*********************************************************
			// third phase: Compute layout of planarized representation
			//*********************************************************

			Layout drawing(PG);

			//***************************************
			//call the Layouter for the CC's UMLGraph
			m_planarLayouter.get().call(PG,adjExternal,drawing);

			// copy layout into umlGraph
			// Later, we move nodes and edges in each connected component, such
			// that no two overlap.
			const List<node> &origInCC = PG.nodesInCC(i);
			ListConstIterator<node> itV;

			for(itV = origInCC.begin(); itV.valid(); ++itV) {
				node vG = *itV;

				umlGraph.x(vG) = drawing.x(PG.copy(vG));
				umlGraph.y(vG) = drawing.y(PG.copy(vG));

				adjEntry adj;
				forall_adj(adj,vG) {
					if ((adj->index() & 1) == 0) continue;
					edge eG = adj->theEdge();

					drawing.computePolylineClear(PG,eG,umlGraph.bends(eG));
				}
			}

			// the width/height of the layout has been computed by the planar
			// layout algorithm; required as input to packing algorithm
			boundingBox[i] = m_planarLayouter.get().getBoundingBox();
		}//for cc's

		//
		// arrange layouts of connected components
		//

		Array<DPoint> offset(numCC);

		m_packer.get().call(boundingBox,offset,m_pageRatio);

		// The arrangement is given by offset to the origin of the coordinate
		// system. We still have to shift each node and edge by the offset
		// of its connected component.

		for(i = 0; i < numCC; ++i)
		{
			const List<node> &nodes = PG.nodesInCC(i);

			const double dx = offset[i].m_x;
			const double dy = offset[i].m_y;

			// iterate over all nodes in ith CC
			ListConstIterator<node> it;
			for(it = nodes.begin(); it.valid(); ++it)
			{
				node v = *it;

				umlGraph.x(v) += dx;
				umlGraph.y(v) += dy;

				adjEntry adj;
				forall_adj(adj,v) {
					if ((adj->index() & 1) == 0) continue;
					edge e = adj->theEdge();

					DPolyline &dpl = umlGraph.bends(e);
					ListIterator<DPoint> it;
					for(it = dpl.begin(); it.valid(); ++it) {
						(*it).m_x += dx;
						(*it).m_y += dy;
					}
				}
			}
		}

		umlGraph.undoGenMergers();

		umlGraph.removeUnnecessaryBendsHV();

		postProcess(umlGraph);

		m_processCliques = l_saveCliqueHandling;

	}//simplecall


	//-----------------------------------------------------------------------------
	//call function for simultaneous drawing: compute a layout for graph umlGraph
        // without special UML or interactive features, clique processing etc.
	//-----------------------------------------------------------------------------
	void UMLPlanarizationLayout::callSimDraw(UMLGraph &umlGraph)
	{
		//this simple call method does not care about any special treatments
		//of subgraphs, layout informations etc., therefore we save the
		//option status and set them back later on
		bool l_saveCliqueHandling = m_processCliques;
		m_processCliques = false;

		m_nCrossings = 0;

		const Graph &G = umlGraph.constGraph();
		if(G.empty())
		  return;

		PlanRepUML PG(umlGraph);

		const int numCC = PG.numberOfCCs();

		// (width,height) of the layout of each connected component
		Array<DPoint> boundingBox(numCC);

		int i;

		//------------------------------------------
		//now planarize CCs and apply drawing module
		for(i = 0; i < numCC; ++i)
		{
			// we treat connected component i
			// PG is set to a copy of this CC

			PG.initCC(i);

			int nOrigVerticesPG = PG.numberOfNodes();

			edge e;
			EdgeArray<int> costOrig(PG.original(), 1);
			EdgeArray<unsigned int> esgOrig(PG.original(), 0);
			forall_edges(e, G)
			    esgOrig[e] = umlGraph.subGraphBits(e);

			//---------------------------------------
			// first phase: Compute a planar subgraph
			//---------------------------------------

			List<edge> deletedEdges;
			m_subgraph.get().callAndDelete(PG, deletedEdges);

			//**************************************
			// second phase: Re-insert deleted edges
			//**************************************

			m_inserter.get().call(PG, costOrig, deletedEdges, esgOrig);

			//calls embedder module:
			adjEntry adjExternal = 0;
			m_embedder.get().call(PG, adjExternal);

			m_nCrossings += PG.numberOfNodes() - nOrigVerticesPG;


			//*********************************************************
			// third phase: Compute layout of planarized representation
			//*********************************************************

			Layout drawing(PG);

			//***************************************
			//call the Layouter for the CC's UMLGraph
			m_planarLayouter.get().call(PG,adjExternal,drawing);

			// copy layout into umlGraph
			// Later, we move nodes and edges in each connected component, such
			// that no two overlap.
			const List<node> &origInCC = PG.nodesInCC(i);
			ListConstIterator<node> itV;

			for(itV = origInCC.begin(); itV.valid(); ++itV) {
				node vG = *itV;

				umlGraph.x(vG) = drawing.x(PG.copy(vG));
				umlGraph.y(vG) = drawing.y(PG.copy(vG));

				adjEntry adj;
				forall_adj(adj,vG) {
					if ((adj->index() & 1) == 0) continue;
					edge eG = adj->theEdge();

					drawing.computePolylineClear(PG,eG,umlGraph.bends(eG));
				}
			}


			// the width/height of the layout has been computed by the planar
			// layout algorithm; required as input to packing algorithm
			boundingBox[i] = m_planarLayouter.get().getBoundingBox();
		}//for cc's

		//
		// arrange layouts of connected components
		//

		Array<DPoint> offset(numCC);

		m_packer.get().call(boundingBox,offset,m_pageRatio);

		// The arrangement is given by offset to the origin of the coordinate
		// system. We still have to shift each node and edge by the offset
		// of its connected component.

		for(i = 0; i < numCC; ++i)
		{
			const List<node> &nodes = PG.nodesInCC(i);

			const double dx = offset[i].m_x;
			const double dy = offset[i].m_y;

			// iterate over all nodes in ith CC
			ListConstIterator<node> it;
			for(it = nodes.begin(); it.valid(); ++it)
			{
				node v = *it;

				umlGraph.x(v) += dx;
				umlGraph.y(v) += dy;

				adjEntry adj;
				forall_adj(adj,v) {
					if ((adj->index() & 1) == 0) continue;
					edge e = adj->theEdge();

					DPolyline &dpl = umlGraph.bends(e);
					ListIterator<DPoint> it;
					for(it = dpl.begin(); it.valid(); ++it) {
						(*it).m_x += dx;
						(*it).m_y += dy;
					}
				}
			}
		}

		umlGraph.removeUnnecessaryBendsHV();
		m_processCliques = l_saveCliqueHandling;

	}//callSimDraw


	//#################################################################

	//additional help functions

	void UMLPlanarizationLayout::assureDrawability(UMLGraph &UG)
	{
		//preliminary 
		//self loops are killed by the caller (e.g., the plugin interface)
		//should be done here later

		//cant handle self loops yet
		edge e;

		const Graph& G = UG;

		//check for selfloops and handle them 
		forall_edges(e, G)
		{
			if (e->target() == e->source())  
			{
				//temporarily, loops are handled in visiointerface, so throw
				THROW_PARAM(PreconditionViolatedException, pvcSelfLoop);

				//is always an error
				if (UG.type(e) == Graph::generalization)
					THROW_PARAM(PreconditionViolatedException, pvcSelfLoop);
			}
		}//foralledges

		//check for generalization - nontrees
		//if m_fakeTree is set, change type of "back" edges to association
		m_fakedGens.clear();//?
		if (!dfsGenTree(UG, m_fakedGens, m_fakeTree)) THROW_PARAM(PreconditionViolatedException, pvcTreeHierarchies);
		else
		{
			ListConstIterator<edge> itE = m_fakedGens.begin();
			while (itE.valid())
			{
				UG.type(*itE) = Graph::association;
				itE++;
			}
		}
	}//assureDrawability

	//used to check in advance the drawability or to continue the algorithm
	//in case of minor failures
	bool UMLPlanarizationLayout::checkDrawability(UMLGraph &UG)
	{
		bool retval = true;
		const Graph& G = UG;
		//cant handle self loops yet
		edge e;
		forall_edges(e, G)
			retval = (e->target() != e->source()) && retval;
		return retval;
	}//checkDrawability


	void UMLPlanarizationLayout::preProcess(UMLGraph &UG)
	{

		assureDrawability(UG);

		if (m_processCliques)
		{
			UG.setDefaultCliqueCenterSize(m_planarLayouter.get().separation());
			const Graph& G = (const Graph &)UG; 
			CliqueFinder cf(G);
			cf.setMinSize(m_cliqueSize);

			List< List<node> > cliques;
			cf.call(cliques);
			//now replace all found cliques by stars
			UG.replaceByStar(cliques);

		}//if cliques
		//TODO: sollte kein else sein, aber man muss abfangen,
		//ob eine Kante geloescht wurde (und die Info nachher wieder bereitstellen)
		else
		{
			const SListPure<UMLGraph::AssociationClass*> &acList = UG.assClassList();
			SListConstIterator<UMLGraph::AssociationClass*> it = acList.begin();
			while (it.valid())
			{
				UG.modelAssociationClass((*it));
				it++;
			}//while

		}//else
	}//preprocess


	void UMLPlanarizationLayout::postProcess(UMLGraph& UG)
	{
		//reset the type of faked associations to generalization
		if (m_fakeTree)
		{
			ListIterator<edge> itE = m_fakedGens.begin();
			while (itE.valid())
			{
				UG.type(*itE) = Graph::generalization;
				itE++;
			}//while
		}//if

		UG.undoAssociationClasses();
		if (m_processCliques)
		{
			UG.undoStars();
		}//if

	}//postProcess


	//TODO: check if any inconsitencies can be introduced by
	//running through all nodes changing gens to associations
	//and inserting mergers on existing gens

	void UMLPlanarizationLayout::prepareIncrementalMergers(UMLGraph &uml)
	{
		//we can't draw multiple hierarchies hanging at one class
		//object, therefore we reduce the number in the given layout
		//to 1 by interpreting only the edges in the largest sequence
		//of generalizations as gens, all other edges are associations
		const Graph& G = uml;

		node v;
		forall_nodes(v, G)
		{
			if (v->degree() < 2) continue;
			if (uml.type(v) == Graph::generalizationMerger) continue;

			int maxSeq = 0;   //stores current best sequence size
			int maxSeqRun = 0; //stores current sequence size 

			adjEntry maxSeqAdj = 0;
			adjEntry runSeqAdj = 0;
			adjEntry ad1 = v->firstAdj();
			//We have to avoid the case where we start within a sequence
			//we run back til the first non-input generalization is detected
			//if there is none, the following is also correct
			adjEntry stopAdj = ad1;
			edge e = ad1->theEdge();
			while ((ad1->cyclicPred() != stopAdj) &&
				( (e->target() == v) &&
				(uml.type(e) == Graph::generalization) ))
			{
				ad1 = ad1->cyclicPred();
				e = ad1->theEdge();
			}//while search start
			adjEntry ad = ad1->cyclicSucc();
			while (ad != ad1)
			{
				edge e = ad->theEdge();
				if ( (e->target() == v) &&
					(uml.type(e) == Graph::generalization) )
				{
					if (maxSeqRun == 0)
					{
						//initialize once
						runSeqAdj = ad;
						maxSeqAdj = ad;
					}
					maxSeqRun++;

				}
				else
				{
					//we never stop here if there is only one
					//gen sequence, but then we don't need to
					//(we don't use maxSeq elsewhere)
					//we change edge in both cases here to avoid 
					//running over all edges again (#gens may be
					//significantly lower then #ass)
					adjEntry changeAdj = 0;
					if (maxSeqRun > maxSeq)
					{
						//change edge type for old favorites
						if (maxSeqAdj != runSeqAdj)
							changeAdj = maxSeqAdj;

						maxSeq = maxSeqRun;
						maxSeqAdj = runSeqAdj;
					}
					else
					{
						//change edge types for weaker sequence
						if (maxSeqRun != 0)
							changeAdj = runSeqAdj;
					}//else, no new sequence

					//Change types for a sequence
					//invariant: on every pass of the loop, if a sequence
					//end is detected, one of the two sequences is deleted
					//(types changed) => only one sequence when we stop
					if (changeAdj != 0)
					{
						adjEntry runGenAdj = changeAdj;
						//no infinite loop because new sequence found
						edge e = runGenAdj->theEdge();
						while ((e->target() == v) &&
							(uml.type(e) == Graph::generalization))
						{
							uml.type(e) = Graph::association;
							runGenAdj = runGenAdj->cyclicSucc();
							e = runGenAdj->theEdge();
						}//while
						//changeAdj = 0;
					}
					maxSeqRun = 0;
				}//else

				ad = ad->cyclicSucc();
			}//while

			//now we insert mergers for all edges in best sequence
			//do not use maxSeq to count, may be 0 if only incoming gens

			if (maxSeqAdj != 0)
			{
				SList<edge> inGens;

				edge e = maxSeqAdj->theEdge();
				adjEntry runAdj = maxSeqAdj;
				while ((e->target() == v) &&
					(uml.type(e) == Graph::generalization))
				{
					inGens.pushBack(e);
					runAdj = runAdj->cyclicSucc();
					e = runAdj->theEdge();
					//maybe only one sequence around v
					if (runAdj == maxSeqAdj)
						break;
				}//while generalizations

				//insert the merger for v
				node newMerger = uml.doInsertMergers(v, inGens);
				//update the layout information for the merger
				//TODO: what is the best strategy for the merger position?
				//should lie on one of the generalizations, epsilon from v 
				if (newMerger != 0)
				{
					uml.x(newMerger) = uml.x(v);
					uml.y(newMerger) = uml.y(v);
				}
			}//if sequence of generalizations

		}//forallnodes

		uml.adjustHierarchyParents();

	}//prepareIncrementalMergers


	// find best suited external face according to certain criteria
	face UMLPlanarizationLayout::findBestExternalFace(
		const PlanRep &PG,
		const CombinatorialEmbedding &E)
	{
		FaceArray<int> weight(E);

		face f;
		forall_faces(f,E)
			weight[f] = f->size();

		node v;
		forall_nodes(v,PG)
		{
			if(PG.typeOf(v) != Graph::generalizationMerger)
				continue;

			adjEntry adj;
			forall_adj(adj,v) {
				if(adj->theEdge()->source() == v)
					break;
			}

			OGDF_ASSERT(adj->theEdge()->source() == v);

			node w = adj->theEdge()->target();
			bool isBase = true;

			adjEntry adj2;
			forall_adj(adj2, w) {
				edge e = adj2->theEdge();
				if(e->target() != w && PG.typeOf(e) == Graph::generalization) {
					isBase = false;
					break;
				}
			}

			if(isBase == false)
				continue;

			face f1 = E.leftFace(adj);
			face f2 = E.rightFace(adj);

			weight[f1] += v->indeg();
			if(f2 != f1)
				weight[f2] += v->indeg();
		}

		face fBest = E.firstFace();
		forall_faces(f,E)
			if(weight[f] > weight[fBest])
				fBest = f;

		return fBest;
	}


	//collects and stores nodes adjacent to centerNode in adjNodes
	void UMLPlanarizationLayout::fillAdjNodes(List<node>& adjNodes, 
		PlanRepUML& PG, 
		node centerNode,
		NodeArray<bool>& isClique,
		Layout& drawing)
	{
		//at this point, cages are collapsed, i.e. we have a center node
		//in the planrep representing the copy of the original node
		node cCopy = PG.copy(centerNode);
		OGDF_ASSERT(cCopy != 0)
			OGDF_ASSERT(cCopy->degree() == centerNode->degree())
			OGDF_ASSERT(cCopy->degree() > 1)
			//store the node with mostright position TODO: consider cage
			node rightNode = 0;

		//due to the implementation in PlanRepUML::collapseVertices, the 
		//ordering of the nodes in the copy does not correspond to the
		//ordering in the used embedding, but to the original graph
		//we therefore need to run around the cage to search for the 
		//attached edges

		adjEntry adjRun = cCopy->firstAdj();
		do
		{
			//we search for the edge outside the node cage
			//anchor node
			OGDF_ASSERT(adjRun->twin()->theNode()->degree() == 4)
				//should be cs->cs, but doesnt work, comb. embedding?
				//adjEntry outerEdgeAdj = adjRun->twin()->cyclicSucc()->cyclicSucc();
				//TODO: braucht man glaube ich gar nicht, da bereits erste Kante Original hat
				adjEntry outerEdgeAdj = adjRun->twin()->cyclicSucc();
			//this may fail in case of bends if there are no orig edges, but there
			//always is one!?
			while (!PG.original(outerEdgeAdj->theEdge()))
				outerEdgeAdj = outerEdgeAdj->cyclicSucc();
			OGDF_ASSERT(outerEdgeAdj != adjRun)

				//hier besser: if... und alle anderen ignorieren
				edge umlEdge = PG.original(outerEdgeAdj->theEdge());
			OGDF_ASSERT(umlEdge != 0)
				node u = umlEdge->opposite(centerNode);
			adjNodes.pushBack(u);
			isClique[PG.copy(u)] = true;

			//----------------------------------------------
			//part to delete all bends that lie within the clique rectangle
			//first we identify the copy node of the clique node we currently
			//look at
			node uCopy = PG.copy(u);
			OGDF_ASSERT(uCopy)
				adjEntry adjURun = uCopy->firstAdj();
			do
			{
				//we search for the edge outside the node cage
				OGDF_ASSERT(adjURun->twin()->theNode()->degree() == 4)
					adjEntry outerEdgeUAdj = adjURun->twin()->cyclicSucc();
				while (!PG.original(outerEdgeUAdj->theEdge()))
					outerEdgeUAdj = outerEdgeUAdj->cyclicSucc();
				OGDF_ASSERT(outerEdgeUAdj != adjRun)
					//outerEdgeUAdj points outwards, edge does too per implementation, 
					//but we don't want to rely on that fact
					bool outwards;
				edge potKill = outerEdgeUAdj->theEdge();
				node splitter;
				if (potKill->source() == outerEdgeUAdj->theNode()) //Could use opposite
				{
					splitter = potKill->target();
					outwards = true;
				}
				else 
				{
					splitter = potKill->source();
					outwards = false;
				}
				//we erase bends and should check the node type here, but the only 
				//case that can happen is bend
				while (splitter->degree() == 2) 
				{
					if (outwards)
					{
						PG.unsplit(potKill, potKill->adjTarget()->cyclicSucc()->theEdge());
						splitter = potKill->target();
					}
					else
					{
						edge ek = potKill->adjSource()->cyclicSucc()->theEdge();
						PG.unsplit(ek, potKill);
						potKill = ek;
						splitter = potKill->source();
					}
				}//while

				adjURun = adjURun->cyclicPred(); //counterclockwise, Succ clockwise
			} while (adjURun != uCopy->firstAdj());

			//----------------------------------------------

			//check if node is better suited to lie at the right position
			if (rightNode != 0)
			{
				if (drawing.x(PG.copy(u)) > drawing.x(PG.copy(rightNode)))
				{
					rightNode = u;
				}
			}
			else
			{
				rightNode = u;
			}

			adjRun = adjRun->cyclicPred(); //counterclockwise, Succ clockwise
		} while (adjRun != cCopy->firstAdj());

		//---------------------------------------
		//adjust ordering to start with rightNode
		if (true) //zum debuggen ausschalten koennen
			while (adjNodes.front() != rightNode)
			{
				node tempV = adjNodes.popFrontRet();
				adjNodes.pushBack(tempV);
			}
	}//filladjNodes

} // end namespace ogdf
