/** \file
 * \brief Offers variety of possible algorithm calls for simultaneous
 * drawing.
 * 
 * \author Michael Schulz
 * 
 * \par License:
 * This file is part of the Open Graph Drawing Framework (OGDF).
 * Copyright (C) 2005
 * 
 * \par
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * Version 2 as published by the Free Software Foundation.
 * 
 * \par
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * \par
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 * 
 * \see  http://www.gnu.org/copyleft/gpl.html
 ***************************************************************/

#include<ogdf/simultaneous/SimDrawCaller.h>
#include<ogdf/layered/SugiyamaLayout.h>
#include<ogdf/planarity/UMLPlanarizationLayout.h>
#include<ogdf/planarity/SubgraphPlanarizer.h>
#include<ogdf/planarity/VariableEmbeddingInserter.h>

#include <iostream>

namespace ogdf {

//*************************************************************
// refreshes m_esg
//
void SimDrawCaller::updateESG()
{
  edge e;
  forall_edges(e, *m_G)
    (*m_esg)[e] = m_GA->subGraphBits(e);

} // end updateESG


//*************************************************************
// Constructor
//
  SimDrawCaller::SimDrawCaller(SimDraw &SD) : SimDrawManipulatorModule(SD)
{
  m_esg = new EdgeArray<unsigned int>(*m_G);
  updateESG();

} // end constructor


//*************************************************************
// call for SugiyamaLayout
void SimDrawCaller::callSugiyamaLayout()
{    
  // Knoten erhalten default-Groessen
  m_SD->addAttribute(GraphAttributes::nodeGraphics);	  
  m_SD->addAttribute(GraphAttributes::edgeGraphics);
  node v;
  forall_nodes(v, *m_G)
    m_GA->height(v) = m_GA->width(v) = 5.0;

  /* eigentlicher call von SugiyamaLayout muss fuer SimDraw-Modus
   * setSubgraphs setzen */
  updateESG();
  SugiyamaLayout SL;
  SL.setSubgraphs(m_esg);
  SL.call(*m_GA);

} // end callSugiyamaLayout


//*************************************************************
// call for UMLPlanarizationLayout
void SimDrawCaller::callUMLPlanarizationLayout()
{
  // baue zugehoerigen UMLGraph
  UMLGraph UG(*m_G, GraphAttributes::edgeSubGraph);
  node v;
  forall_nodes(v,*m_G)
    UG.width(v) = UG.height(v) = 5.0;
  edge e;
  forall_edges(e,*m_G)
    UG.subGraphBits(e) = m_GA->subGraphBits(e);

  // eigentlicher call auf UMLGraph
  UMLPlanarizationLayout PL;
  PL.callSimDraw(UG);

  // uebertrage Koordinaten und Kantenknicke auf Original
  m_SD->addAttribute(GraphAttributes::nodeGraphics);
  m_SD->addAttribute(GraphAttributes::edgeGraphics);
  forall_nodes(v,*m_G)
    {
      m_GA->x(v) = UG.x(v);
      m_GA->y(v) = UG.y(v);
    }
  forall_edges(e,*m_G)
      m_GA->bends(e) = UG.bends(e);

} // end callUMLPlanarizationLayout


//*************************************************************
// call for SubgraphPlanarizer
// returns crossing number
int SimDrawCaller::callSubgraphPlanarizer(int cc, int numberOfPermutations)
{   
  // uebertrage evtl Kantenkosten
  EdgeArray<int> ec(*m_G, 1);
  if(m_GA->attributes() & GraphAttributes::edgeIntWeight)
    {
      edge e;
      forall_edges(e,*m_G)
	ec[e] = m_GA->intWeight(e);
    }

  // initialisiere
  updateESG();
  int crossNum = 0;
  PlanRep PR(*m_G);

  // eigentlicher call fuer ZshgsKomponente cc
  SubgraphPlanarizer SP;
  VariableEmbeddingInserter* vei = new VariableEmbeddingInserter();
  vei->removeReinsert(VariableEmbeddingInserter::rrIncremental);
  SP.setInserter(vei);
  SP.permutations(numberOfPermutations);
  SP.call(PR, cc, crossNum, &ec, 0, m_esg);
  //delete vei;

  // fuege alle Dummyknoten in Originalgraph *m_G ein
  NodeArray<node> newOrigNode(PR);
  node vPR;
  forall_nodes(vPR, PR)
    {
      if(PR.isDummy(vPR))
	{
	  node vOrig = m_G->newNode();
	  newOrigNode[vPR] = vOrig;
	  m_SD->isDummy(vOrig) = true;
	}
      else
	newOrigNode[vPR] = PR.original(vPR); 
        //der Einfachheithalber speichern wir Originalknoten fuer sich selber
    }

  // fuege alle zu Dummyknoten inzidenten Kanten in *m_G ein
  EdgeArray<bool> toBeDeleted(*m_G, false);
  EdgeArray<bool> visited(PR, false);
  forall_nodes(vPR, PR)
    {
      if(PR.isDummy(vPR))
	{
	  node vNewOrig = newOrigNode[vPR]; //lebt in *m_G
	  edge e;
	  forall_adj_edges(e, vPR) //lebt in PR
	    {
	      if(!visited[e])
		{
		  node w = e->opposite(vPR); //lebt in PR
		  node wNewOrig = newOrigNode[w]; //lebt in *m_G
		  edge eNewOrig = m_G->newEdge(vNewOrig,wNewOrig);
		  m_GA->subGraphBits(eNewOrig) = m_GA->subGraphBits(PR.original(e));
		  toBeDeleted[PR.original(e)] = true;
		  visited[e] = true;
		}
	    }
	}
    }

  // loesche alle alten Kanten in *m_G, die durch Dummyknotenkanten ersetzt wurden
  List<edge> LE;
  m_G->allEdges(LE);
  forall_listiterators(edge, it, LE) 
    {
      if(toBeDeleted[ (*it) ])
	m_G->delEdge( (*it) );
    }
      
  return crossNum;

} // end callSubgraphPlanarizer

} // end namespace ogdf
